//
//  bt_l2cap_resource_manager.c
//
//  Bluetooth Protocol Stack - L2CAP Resource Manager
//  Copyright (C) 2013 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Summary:
//		Bluetooth controler link keeping
//		Message send/receive
//		Message segmentation/reassemble
//		Root task
//		Server function support only
//
//	Reference:
//		Bluetooth SIG (www.bluetooth.com)
//			BLUETOOTH SPECIFICATION Version 4.0
//			[Vol 3] Core System Package [Host volume]
//			Part A: Logical Link Control and Adaptation Protocol Specification
//
//	Update history
//	---------- ----- -------------------------------------------
//	2013.01.21 v0.0  First cording
//	2013.04.21 v0.1  Commit
//
#include <stddef.h>
#include <string.h>
#include "bt_hci.h"						// USB-Bluetooth controler support API
#include "bt_spec_hci.h"				// HCI Message format
#include "bt_spec_gap.h"				//
#include "bt_l2cap_cb_device.h"			//
#include "bt_l2cap_cb_link.h"			//
#include "bt_l2cap_channel_manager.h"	//
#include "bt_l2cap_resource_manager.h"	//
#include "bt_spec_att.h"				// Service UUID
#include "bt_att_db.h"					// Service UUID

#define MIN(a,b)		((a)<(b)?(a):(b))
#define INIT_UU16(a)	ATT_INITUUID16TO128(a)	// Alias
#define INIT_UU128(a)	ATT_INITUUID128(a)		// Alias
#define INIT_HANDLE(a)	ATT_INIT16(a)			// Alias
//#define DEBUG

#ifdef DEBUG
#include <stdio.h>						// Debug
#define DEBUGP(...)		printf(__VA_ARGS__)
#else
#define DEBUGP(...)
#endif

#define BT_PIN_CODE					"0000"
#define BT_SEND_TIMEOUT				100000
#define BT_RECIEVE_TIMEOUT			100000
#define BT_INIT_TIMEOUT				100000
#define BT_INIT_DETECT_ENABLE
#define BT_INIT_LE_DETECT_ENABLE

int	bt_l2capResourceManagerRequest;

//
// Run loop
//
int bt_l2capResourceManager(void){
	static struct bt_hci_Command_Packet		cmd;
	static struct bt_hci_Event_Packet		evt;
	static struct bt_hci_ACL_Data_Packet	aclIn;
	static struct bt_hci_ACL_Data_Packet	aclOut;
	static size_t							aclInSize;
	static size_t							aclOutSize;

	static int						init = 0;
	static long						try;
	static unsigned char			*ci;
	static int						send_len;
	static struct bt_l2capDevice	*nowDev;
	static struct bt_l2capLink		*nowLnk;
	static unsigned char			myBdAddr[6];
	static int						hciVersion;

	static enum {
		UNPLUGED = 0,
		PLUGED,
		INIT_COMMAND_SEND,
		INIT_COMMAND_STATUS,
		LE_INIT_COMMAND_SEND,
		LE_INIT_COMMAND_STATUS,
		EVENT_WAIT,
		ACCEPT_CONNECT_SEND,
		REJECT_CONNECT_SEND,
		LINK_KEY_REP_SEND,
		LINK_KEY_NEG_SEND,
		PIN_CODE_REP_SEND,
		PIN_CODE_NEG_SEND,
		LE_ADV_START_SEND,
		LE_ADV_STOP_SEND,
		PROCESS,
		DATA_SEND
	}		state;
	static enum {
		LE_DISABLE = 0,
		LE_STANDBY,
		LE_ADVERTISING,
		LE_SCANNING,
		LE_INITIATING,
		LE_CONNECTION_MASTER_ROLE,
		LE_CONNECTION_SLAVE_ROLE
	}		leState;
	int		rc;

#define HCI_OPCODE_DATA(a)		(unsigned char)a,(unsigned char)(a >> 8)
#define HCI_UINT16_DATA(a)		(unsigned char)a,(unsigned char)(a >> 8)
	const unsigned char initCmds[] = {
		HCI_OPCODE_DATA(HCI_Reset),
		0,			// Len
		HCI_OPCODE_DATA(HCI_Read_Local_Version_Information),
		0,			// Len

		HCI_OPCODE_DATA(HCI_Read_Local_Supported_Commands),
		0,			// Len

		HCI_OPCODE_DATA(HCI_Read_Local_Supported_Features),
		0,			// Len

		HCI_OPCODE_DATA(HCI_Read_Local_Extended_Features),
		1,			// Len
		0,			// Page#

		HCI_OPCODE_DATA(HCI_Read_Buffer_Size),
		0,			// Len

		HCI_OPCODE_DATA(HCI_Read_BD_ADDR),
		0,			// Len

		HCI_OPCODE_DATA(HCI_Host_Buffer_Size),
		7,			// Len
		HCI_UINT16_DATA(sizeof(aclIn)),		// Host_ACL_Data_Packet_Length
		0,									// Host_Synchronous_Data_Packet_Length
		HCI_UINT16_DATA(1),					// Host_Total_Num_ACL_Data_Packets
		HCI_UINT16_DATA(0),					// Host_Total_Num_Synchronous_Data_Packets

		HCI_OPCODE_DATA(HCI_Write_Class_of_Device),
		3,			// Len
		0x12, 0x01, 0x80,

		HCI_OPCODE_DATA(HCI_Write_Local_Name),
		18,			// Len
		'S', 'T', 'M', '3', '2', 'F', '4', ' ', 'd', 'i', 's', 'c', 'o', 'v', 'e', 'r', 'y', '\0',
#ifdef BT_INIT_DETECT_ENABLE
		HCI_OPCODE_DATA(HCI_Write_Scan_Enable),
		1,			// Len
		HCI_Parm_Scan_Enable_Inquiry_Scan | HCI_Parm_Scan_Enable_Page_Scan,
#endif

		0xff
	};

	const unsigned char initLECmds[] = {
		HCI_OPCODE_DATA(HCI_Read_Local_Supported_Features),
		0,			// Len

		HCI_OPCODE_DATA(HCI_Set_Event_Mask),
		8,			// Len
		0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x3f,	// All enable

		HCI_OPCODE_DATA(HCI_LE_Set_Event_Mask),
		8,			// Len
		0x1f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,	// All enable

		HCI_OPCODE_DATA(HCI_LE_Read_Buffer_Size),
		0,			// Len

		HCI_OPCODE_DATA(HCI_LE_Read_Local_Supported_Features),
		0,			// Len

		HCI_OPCODE_DATA(HCI_LE_Read_White_List_Size),
		0,			// Len

		HCI_OPCODE_DATA(HCI_LE_Clear_White_List),
		0,			// Len

		HCI_OPCODE_DATA(HCI_LE_Set_Advertising_Parameters),
		15,			// Len
		0x00, 0x08, // Advertising_Interval_Min=1.28s
		0x00, 0x08,	// Advertising_Interval_Max=1.28s
		0x00,		// Advertising_Type=ADV_IND
		0x00,		// Own_Address_Type=public
		0x00,		// Direct_Address_Type=public
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	// Direct_Address
		0b00000111,	// Advertising_Channel_Map all(37-39)
		0x00,		// Advertising_Filter_Policy=any

		HCI_OPCODE_DATA(HCI_LE_Read_Advertising_Channel_TX_Power),
		0,			// Len

		HCI_OPCODE_DATA(HCI_LE_Set_Advertising_Data),
		32,			// Len
		21,			// Advertising_Data_Length
					// Advertising_Data
		2,  GAP_AD_Type_Flags,
			GAP_AD_Flags_Simultaneous_LE_BR_EDR_Host |
			GAP_AD_Flags_Simultaneous_LE_BR_EDR_Controller |
			GAP_AD_Flags_LE_General_Discoverable_Mode,
		17, GAP_AD_Type_128bit_Service_UUIDs_Complete,
			INIT_UU128(UUID_MY_SERVICE),
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00,

		HCI_OPCODE_DATA(HCI_LE_Set_Scan_Response_Data),
		32,			// Len
		10,			// Scan_Response_Data_Length,
					// Scan_Response_Data
		8,  GAP_AD_Type_Shortened_local_name,
			'S', 'T', 'M', '3', '2', 'F', '4',
		0x00, 0x00,	0x00, 0x00,	0x00, 0x00,	0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00,

		HCI_OPCODE_DATA(HCI_LE_Set_Scan_Parameters),
		7,			// Len
		0x00,		// LE_Scan_Type=passive
		0x10, 0x00,	// LE_Scan_Interval=10ms
		0x10, 0x00,	// LE_Scan_Window=10ms
		0x00,		// Own_Address_Type=public
		0x00,		// Scanning_Filter_Policy=all

#ifdef BT_INIT_LE_DETECT_ENABLE
		HCI_OPCODE_DATA(HCI_LE_Set_Advertise_Enable),
		1,			// Len
		0x01,		// Advertising_Enable=enable
#endif

		0xff
	};

	//
	// main
	//
	if (init == 0) {
		init = 1;
		bt_setHCIcommandBuf((unsigned char *)&cmd,    sizeof(cmd));
		bt_setHCIeventBuf(  (unsigned char *)&evt,    sizeof(evt));
		bt_setACLoutBuf(    (unsigned char *)&aclOut, sizeof(aclOut));		// bt_hci.c
		bt_setACLinBuf(     (unsigned char *)&aclIn,  sizeof(aclIn));		//
		aclInSize  = sizeof(aclIn);
		aclOutSize = sizeof(aclOut);
		state      = UNPLUGED;
		hciVersion = 0;
#ifdef DEBUG2
	} else {
		static int	s = 0;

		if (s != state) {
			s = state;
			DEBUGP("L2CAP-RM: New state=%d\n", state);
		}
#endif
	}

	// state machine
	rc = 0;
	switch(state) {
	case UNPLUGED:						// Phisical device removed
		if (bt_isAliveHCI()) {
			state = PLUGED;
		}
		break;

	case PLUGED:						// Phisical device pluged in
		try   = 0;
		ci    = (unsigned char *)initCmds;
		state = INIT_COMMAND_SEND;
		break;

	case INIT_COMMAND_SEND:
		if (*ci == 0xff) {
			DEBUGP("L2CAP-RM: init end aclOutSize=%d aclInSize=%d\n", aclOutSize, aclInSize);
			if (hciVersion >= HCI_Parm_HCI_Version_4_0) {
				ci      = (unsigned char *)initLECmds;
				leState = LE_STANDBY;
				state   = LE_INIT_COMMAND_SEND;
			} else {
				leState = LE_DISABLE;
				state   = EVENT_WAIT;
			}
			break;
		}
		if (bt_isIdleHCIcommand()) {
			memcpy(&cmd, ci, HCI_Command_Packet_HeadSize + *(ci+2));
			bt_sendHCIcommand();
			try   = 0;
			state = INIT_COMMAND_STATUS;
		} else if (try++ > BT_INIT_TIMEOUT) {
			DEBUGP("L2CAP-RM: INIT_COMMAND_SEND timeout\n");
			rc    = BT_L2CAP_ERR_HCI_CMD_SEND_TIMEOUT;
			state = UNPLUGED;
		}
		break;

	case INIT_COMMAND_STATUS:
		if (!bt_isAliveHCI()) {
			state = UNPLUGED;
		} else if (bt_recieveHCIevent()) {
			if ((evt.Event_Code == HCI_Event_Command_Status) &&
				(evt.Event_Parameter.Command_Status.Command_Opcode[0] == ci[0]) &&
				(evt.Event_Parameter.Command_Status.Command_Opcode[1] == ci[1])) {
				ci += (HCI_Command_Packet_HeadSize + *(ci+2));
				state = INIT_COMMAND_SEND;
			} else if ((evt.Event_Code == HCI_Event_Command_Complete) &&
					   (evt.Event_Parameter.Command_Complete.Command_Opcode[0] == ci[0]) &&
					   (evt.Event_Parameter.Command_Complete.Command_Opcode[1] == ci[1])) {
				ci += (HCI_Command_Packet_HeadSize + *(ci+2));
				state = INIT_COMMAND_SEND;

				switch(HCI_UINT16(evt.Event_Parameter.Command_Complete.Command_Opcode)){
				case HCI_Read_Local_Version_Information:
					if (evt.Event_Parameter.Command_Complete.Return_Parameter.Read_Local_Version_Information.Status == 0) {
						hciVersion = evt.Event_Parameter.Command_Complete.Return_Parameter.Read_Local_Version_Information.HCI_Version;
					} else {
						DEBUGP("L2CAP-RM: Read_Local_Version_Information error=0x%x\n", evt.Event_Parameter.Command_Complete.Return_Parameter.Read_Local_Version_Information.Status);
						rc = BT_L2CAP_ERR_HCI_READ_LOCAL_VER;
					}
					break;
				case HCI_Read_Buffer_Size:
					if (evt.Event_Parameter.Command_Complete.Return_Parameter.Read_Buffer_Size.Status == 0) {
						size_t	l;

						l = HCI_UINT16(evt.Event_Parameter.Command_Complete.Return_Parameter.Read_Buffer_Size.HC_ACL_Data_Packet_Length);
						if (l > sizeof(aclOut)) {
							aclOutSize = sizeof(aclOut);
						} else {
							aclOutSize = l;
						}
					} else {
						DEBUGP("L2CAP-RM: Read_Buffer_Size error=0x%x\n", evt.Event_Parameter.Command_Complete.Return_Parameter.Read_Buffer_Size.Status);
						rc = BT_L2CAP_ERR_HCI_READ_BUFF_SIZE;
					}
					break;
				case HCI_Read_BD_ADDR:
					if (evt.Event_Parameter.Command_Complete.Return_Parameter.Read_BD_ADDR.Status == 0) {
						memcpy(myBdAddr, evt.Event_Parameter.Command_Complete.Return_Parameter.Read_BD_ADDR.BD_ADDR, sizeof(myBdAddr));
					} else {
						DEBUGP("L2CAP-RM: Read_BD_ADDR error=0x%x\n", evt.Event_Parameter.Command_Complete.Return_Parameter.Read_BD_ADDR.Status);
						rc = BT_L2CAP_ERR_HCI_READ_BDADDR;
					}
					break;
				default:
					break;
				}
			}
		} else if (try++ > BT_INIT_TIMEOUT) {
			DEBUGP("L2CAP-RM: INIT_COMMAND_STATUS timeout\n");
			rc    = BT_L2CAP_ERR_HCI_STAT_RECV_TIMEOUT;
			state = UNPLUGED;
		}
		break;

	case LE_INIT_COMMAND_SEND:
		if (*ci == 0xff) {
			DEBUGP("L2CAP-RM: LE init end aclOutSize=%d aclInSize=%d\n", aclOutSize, aclInSize);
#ifdef BT_INIT_LE_DETECT_ENABLE
			leState = LE_ADVERTISING;
#endif
			state = EVENT_WAIT;
			break;
		}
		if (bt_isIdleHCIcommand()) {
			memcpy(&cmd, ci, HCI_Command_Packet_HeadSize + *(ci+2));
			bt_sendHCIcommand();
			try   = 0;
			state = LE_INIT_COMMAND_STATUS;
		} else if (try++ > BT_INIT_TIMEOUT) {
			DEBUGP("L2CAP-RM: LE_INIT_COMMAND_SEND timeout\n");
			rc    = BT_L2CAP_ERR_HCI_CMD_SEND_TIMEOUT;
			state = UNPLUGED;
		}
		break;

	case LE_INIT_COMMAND_STATUS:
		if (!bt_isAliveHCI()) {
			state = UNPLUGED;
		} else if (bt_recieveHCIevent()) {
			if ((evt.Event_Code == HCI_Event_Command_Status) &&
				(evt.Event_Parameter.Command_Status.Command_Opcode[0] == ci[0]) &&
				(evt.Event_Parameter.Command_Status.Command_Opcode[1] == ci[1])) {
				ci += (HCI_Command_Packet_HeadSize + *(ci+2));
				state = LE_INIT_COMMAND_SEND;
			} else if ((evt.Event_Code == HCI_Event_Command_Complete) &&
					   (evt.Event_Parameter.Command_Complete.Command_Opcode[0] == ci[0]) &&
					   (evt.Event_Parameter.Command_Complete.Command_Opcode[1] == ci[1])) {
				ci += (HCI_Command_Packet_HeadSize + *(ci+2));
				state = LE_INIT_COMMAND_SEND;
			}
			switch(HCI_UINT16(evt.Event_Parameter.Command_Complete.Command_Opcode)){
			case HCI_LE_Read_Buffer_Size:
				if (evt.Event_Parameter.Command_Complete.Return_Parameter.LE_Read_Buffer_Size.Status == 0) {
					size_t	l;

					l = HCI_UINT16(evt.Event_Parameter.Command_Complete.Return_Parameter.LE_Read_Buffer_Size.HC_LE_Data_Packet_Length);
					if ((l != 0) && (l < aclOutSize)) {
						aclOutSize = l;
					}
				} else {
					DEBUGP("L2CAP-RM: LE Read_Buffer_Size error=0x%x\n", evt.Event_Parameter.Command_Complete.Return_Parameter.LE_Read_Buffer_Size.Status);
					rc = BT_L2CAP_ERR_HCI_READ_BUFF_SIZE;
				}
				break;
			default:
				break;
			}
		} else if (try++ > BT_INIT_TIMEOUT) {
			DEBUGP("L2CAP-RM: LE_INIT_COMMAND_STATUS timeout\n");
			rc    = BT_L2CAP_ERR_HCI_STAT_RECV_TIMEOUT;
			state = UNPLUGED;
		}
		break;

	case EVENT_WAIT:		// Phisical device active

		if (!bt_isAliveHCI()) {
			bt_l2capLinkClear();
			nowLnk = NULL;
			rc     = BT_L2CAP_ERR_HCI_DEAD;
			state  = UNPLUGED;

		// Event message from HCI
		} else if (bt_recieveHCIevent()) {
			try = 0;
			switch(evt.Event_Code){

//			case HCI_Event_Inquiry_Result:
//				break;

//			case HCI_Event_Inquiry_Complete:
//				break;

			case HCI_Event_Connection_Request:
				state = ACCEPT_CONNECT_SEND;
				break;

			case HCI_Event_Connection_Complete:
				if (evt.Event_Parameter.Connection_Complete.Status == 0) {
					struct bt_l2capDevice	*p;
					struct bt_l2capLink		*q;
					unsigned short			h;

					p = bt_l2capDeviceAdd(evt.Event_Parameter.Connection_Complete.BD_ADDR);
					if (p != NULL) {
						h = HCI_UINT16(evt.Event_Parameter.Connection_Complete.Connection_Handle);
						q = bt_l2capLinkAdd(p->bdAddr, h);
						if (q != NULL) {
							q->linkType   = evt.Event_Parameter.Connection_Complete.Link_Type;
							q->encryption = evt.Event_Parameter.Connection_Complete.Encryption_Mode;
							q->role       = HCI_Parm_Role_Unknown;
							q->buffSize   = sizeof(q->buff);
							q->nowSize    = 0;
							q->sendedSize = 0;
							DEBUGP("L2CAP-RM: New Link: handle=%x\n", h);
						} else {
							DEBUGP("L2CAP-RM: ENOMEM\n");
						}
					} else {
						DEBUGP("L2CAP-RM: ENOMEM\n");
					}
				}
				break;

			case HCI_Event_Disconnection_Complete:
				if (evt.Event_Parameter.Disconnection_Complete.Status == 0) {
					unsigned short		h;
					struct bt_l2capLink	*l;

					h = HCI_UINT16(evt.Event_Parameter.Disconnection_Complete.Connection_Handle);
					DEBUGP("L2CAP-RM: Del handle: %x\n", h);
					l = bt_l2capLinkFind(NULL, h);
					bt_l2capLinkDelete(l);

					if (leState == LE_ADVERTISING) {
						state = LE_ADV_START_SEND;
					}
				}
				break;

			case HCI_Event_Command_Complete:
				break;

			case HCI_Event_Command_Status:
				break;

			case HCI_Event_Hardware_Error:
				DEBUGP("L2CAP-RM: Bluetooth hardware error!!\n");
				bt_l2capLinkClear();
				nowLnk = NULL;
				rc     = BT_L2CAP_ERR_HCI_HARDWARE;
				state  = UNPLUGED;
				break;

			case HCI_Event_Number_Of_Completed_Packets:
				break;

			case HCI_Event_PIN_Code_Request:
				DEBUGP("L2CAP-RM: PIN_Code_Request\n");
				nowDev = bt_l2capDeviceFind(evt.Event_Parameter.PIN_Code_Request.BD_ADDR);
				if (nowDev != NULL) {
					DEBUGP("L2CAP-RM: PIN_Code_Request -> OK\n");
					state = PIN_CODE_REP_SEND;
				} else {
					DEBUGP("L2CAP-RM: PIN_Code_Request -> skip\n");
				}
				break;

			case HCI_Event_Link_Key_Request:
				nowDev = bt_l2capDeviceFind(evt.Event_Parameter.Link_Key_Request.BD_ADDR);
				if ((nowDev != NULL) && (nowDev->linkKeyValid)) {
					DEBUGP("L2CAP-RM: Link_Key_Request -> OK\n");
					state = LINK_KEY_REP_SEND;
				} else {
					DEBUGP("L2CAP-RM: Link_Key_Request -> NG\n");
					state = LINK_KEY_NEG_SEND;
				}
				break;

			case HCI_Event_Link_Key_Notification:
				{
					struct bt_l2capDevice	*p;

					DEBUGP("L2CAP-RM: Link_Key_Notification\n");
					p = bt_l2capDeviceAdd(evt.Event_Parameter.Link_Key_Notification.BD_ADDR);
					if (p != NULL) {
						memcpy(p->bdAddr,  evt.Event_Parameter.Link_Key_Notification.BD_ADDR,  sizeof(p->bdAddr));
						memcpy(p->linkKey, evt.Event_Parameter.Link_Key_Notification.Link_Key, sizeof(p->linkKey));
						p->keyType       = evt.Event_Parameter.Link_Key_Notification.Key_Type;
						p->linkKeyValid  = 1;
					} else {
						DEBUGP("L2CAP-RM: ENOMEM\n");
					}
				}
				break;

			case HCI_Event_Max_Slots_Change:
				break;

			case HCI_Event_Page_Scan_Repetition_Mode_Change:
				break;

			case HCI_Event_LE_Meta_Event:
				switch(evt.Event_Parameter.LE_Connection_Complete.Subevent_Code){
				case HCI_Event_LE_Connection_Complete:
					if (evt.Event_Parameter.LE_Connection_Complete.Status == 0) {
						struct bt_l2capDevice	*p;
						struct bt_l2capLink		*q;
						unsigned short			h;

						p = bt_l2capDeviceAdd(evt.Event_Parameter.LE_Connection_Complete.Peer_Address);
						if (p != NULL) {
							p->peerAddressType = evt.Event_Parameter.LE_Connection_Complete.Peer_Address_Type;
							h = HCI_UINT16(evt.Event_Parameter.LE_Connection_Complete.Connection_Handle);
							q = bt_l2capLinkAdd(p->bdAddr, h);
							if (q != NULL) {
								q->role       = evt.Event_Parameter.LE_Connection_Complete.Role;
								q->buffSize   = sizeof(q->buff);
								q->nowSize    = 0;
								q->sendedSize = 0;
								DEBUGP("L2CAP-RM: New LE handle: %x role=%d\n", h, q->role);
							} else {
								DEBUGP("L2CAP-RM: ENOMEM\n");
							}
						} else {
							DEBUGP("L2CAP-RM: ENOMEM\n");
						}
					}
					break;
//				case HCI_Event_LE_Advertising_Report:
//					break;
//				case HCI_Event_LE_Connection_Update_Complete:
//					break;
//				case HCI_Event_LE_Read_Remote_Used_Features_Complete:
//					break;
//				case HCI_Event_LE_Long_Term_Key_Request:
//					break;
				default:
					DEBUGP("L2CAP-RM: Undef LE-event: %x\n", evt.Event_Parameter.LE_Connection_Complete.Subevent_Code);
					rc = BT_L2CAP_ERR_HCI_UNDEF_EVENT;
					break;
				}
				break;

			default:
				DEBUGP("L2CAP-RM: Undef event: %x\n", evt.Event_Code);
				rc = BT_L2CAP_ERR_HCI_UNDEF_EVENT;
				break;
			}

		// ACL message
		} else if (bt_recieveACLin()) {
			unsigned short	h;
			unsigned short	boundary;
			size_t			len;
			unsigned short	handle;
			struct bt_l2capLink	*q;

			h = HCI_UINT16(aclIn.Connection_Handle);
			handle   = h & HCI_CONNECTION_HANDLE_MASK;
			boundary = h & HCI_PACKET_BOUNDARY_FLAG_MASK;
			len      = HCI_UINT16(aclIn.Data_Total_Length);

			// Searching the buffer
			q = bt_l2capLinkFind(NULL, handle);
			if (q != NULL) {
				switch(boundary){
				case HCI_PACKET_BOUNDARY_FLAG_FIRST:
				case HCI_PACKET_BOUNDARY_FLAG_FIRST_NOFL:
					if (len <= q->buffSize) {
						memcpy(q->buff, aclIn.Data, len);
						q->nowSize = len;
					}
					break;
				case HCI_PACKET_BOUNDARY_FLAG_COMPLETE:
					state  = PROCESS;
					nowLnk = q;
				case HCI_PACKET_BOUNDARY_FLAG_CONTINUE:
					if ((len + q->nowSize) <= q->buffSize) {
						memcpy(&(q->buff[q->nowSize]), aclIn.Data, len);
						q->nowSize += len;
					}
					break;
				default:
					break;
				}
				if (len < sizeof(aclIn.Data)) {
					state  = PROCESS;
					nowLnk = q;
				}
			}

		} else if (bt_l2capResourceManagerRequest) {
			;
		} else {	// Idle
			static struct bt_l2capLink	*l = NULL;

			// Channels task refresh
			if ((nowLnk = bt_l2capLinkGetNext(&l)) != NULL) {	// Round-robin scheduling
				send_len = bt_l2capChannelManager(nowLnk->handle, NULL, 0, nowLnk->buff, nowLnk->buffSize);
				if (send_len > 0) {
					state = DATA_SEND;
					try   = 0;
					nowLnk->sendedSize = 0;
				}
			}
		}
		break;

	case ACCEPT_CONNECT_SEND:
		if (bt_isIdleHCIcommand()) {
			HCI_STORE16(cmd.Op_Code, HCI_Accept_Connection_Request);
			cmd.Parameter_Total_Length = sizeof(cmd.Parameter.Accept_Connection_Request);
			memcpy(cmd.Parameter.Accept_Connection_Request.BD_ADDR, evt.Event_Parameter.Connection_Request.BD_ADDR, sizeof(evt.Event_Parameter.Connection_Request.BD_ADDR));
			cmd.Parameter.Accept_Connection_Request.Role = HCI_Parm_Role_Remain_the_Slave;
			bt_sendHCIcommand();
			state = EVENT_WAIT;
		} else if (try++ > BT_SEND_TIMEOUT) {
			DEBUGP("L2CAP-RM: ACCEPT_CONNECT_SEND timeout handle=%04x\n", nowLnk->handle);
			rc    = BT_L2CAP_ERR_HCI_CMD_SEND_TIMEOUT;
			state = EVENT_WAIT;
		}
		break;

	case LINK_KEY_NEG_SEND:
		if (bt_isIdleHCIcommand()) {
			HCI_STORE16(cmd.Op_Code, HCI_Link_Key_Request_Negative_Reply);
			cmd.Parameter_Total_Length = sizeof(cmd.Parameter.Link_Key_Request_Negative_Reply);
			memcpy(cmd.Parameter.Link_Key_Request_Negative_Reply.BD_ADDR, evt.Event_Parameter.Link_Key_Request.BD_ADDR, sizeof(evt.Event_Parameter.Link_Key_Request.BD_ADDR));
			bt_sendHCIcommand();
			state = EVENT_WAIT;
		} else if (try++ > BT_SEND_TIMEOUT) {
			DEBUGP("L2CAP-RM: LINK_KEY_NEG_SEND timeout handle=%04x\n", nowLnk->handle);
			rc    = BT_L2CAP_ERR_HCI_CMD_SEND_TIMEOUT;
			state = EVENT_WAIT;
		}
		break;

	case LINK_KEY_REP_SEND:
		if (bt_isIdleHCIcommand()) {
			HCI_STORE16(cmd.Op_Code, HCI_Link_Key_Request_Reply);
			cmd.Parameter_Total_Length = sizeof(cmd.Parameter.Link_Key_Request_Reply);
			memcpy(cmd.Parameter.Link_Key_Request_Reply.BD_ADDR, nowDev->bdAddr, sizeof(cmd.Parameter.Link_Key_Request_Reply.BD_ADDR));
			memcpy(cmd.Parameter.Link_Key_Request_Reply.Link_Key, nowDev->linkKey, sizeof(cmd.Parameter.Link_Key_Request_Reply.Link_Key));
			bt_sendHCIcommand();
			state = EVENT_WAIT;
		} else if (try++ > BT_SEND_TIMEOUT) {
			DEBUGP("L2CAP-RM: LINK_KEY_REP_SEND timeout handle=%04x\n", nowLnk->handle);
			rc    = BT_L2CAP_ERR_HCI_CMD_SEND_TIMEOUT;
			state = EVENT_WAIT;
		}
		break;

	case PIN_CODE_REP_SEND:
		if (bt_isIdleHCIcommand()) {
			HCI_STORE16(cmd.Op_Code, HCI_PIN_Code_Request_Reply);
			cmd.Parameter_Total_Length = sizeof(cmd.Parameter.PIN_Code_Request_Reply);
			memcpy(cmd.Parameter.PIN_Code_Request_Reply.BD_ADDR, evt.Event_Parameter.PIN_Code_Request.BD_ADDR, sizeof(cmd.Parameter.PIN_Code_Request_Reply.BD_ADDR));
			memset(cmd.Parameter.PIN_Code_Request_Reply.PIN_Code, 0, sizeof(cmd.Parameter.PIN_Code_Request_Reply.PIN_Code));
			memcpy(cmd.Parameter.PIN_Code_Request_Reply.PIN_Code, BT_PIN_CODE, sizeof(BT_PIN_CODE) - 1);
			cmd.Parameter.PIN_Code_Request_Reply.PIN_Code_Length = sizeof(BT_PIN_CODE) - 1;
			bt_sendHCIcommand();
			state = EVENT_WAIT;
		} else if (try++ > BT_SEND_TIMEOUT) {
			DEBUGP("L2CAP-RM: PIN_CODE_REP_SEND timeout handle=%04x\n", nowLnk->handle);
			rc    = BT_L2CAP_ERR_HCI_CMD_SEND_TIMEOUT;
			state = EVENT_WAIT;
		}
		break;

	case PROCESS:
		state = EVENT_WAIT;
		if (nowLnk != NULL) {
			send_len = bt_l2capChannelManager(nowLnk->handle, nowLnk->buff, nowLnk->nowSize, nowLnk->buff, nowLnk->buffSize);
			if (send_len) {
				state = DATA_SEND;
				try = 0;
				nowLnk->sendedSize = 0;
			}
		}
		break;

	case DATA_SEND:

		if (try++ > BT_SEND_TIMEOUT) {
			DEBUGP("DATA_SEND timeout handle=%04x\n", nowLnk->handle);
			rc    = BT_L2CAP_ERR_HCI_DATA_SEND_TIMEOUT;
			state = EVENT_WAIT;
		}
		if (bt_isIdleACLout() && (nowLnk != NULL)) {		// Message Segmentation
			size_t	l;

			if (nowLnk->sendedSize == 0) {	// 1st fragment
				HCI_STORE16(aclOut.Connection_Handle, (nowLnk->handle | HCI_PACKET_BOUNDARY_FLAG_FIRST));
			} else {						// continue fragment
				HCI_STORE16(aclOut.Connection_Handle, (nowLnk->handle | HCI_PACKET_BOUNDARY_FLAG_CONTINUE));
			}
			l = MIN((send_len - nowLnk->sendedSize), (aclOutSize - HCI_ACL_Data_Packet_HeadSize));
			HCI_STORE16(aclOut.Data_Total_Length, l);
			memcpy(aclOut.Data, &(nowLnk->buff[nowLnk->sendedSize]), l);
			nowLnk->sendedSize += l;

			// Send request
			bt_sendACLout();

			// Next fragment
			if (nowLnk->sendedSize >= send_len) {
				state = EVENT_WAIT;
			} else {
				try = 0;
			}
		}
		break;

	case LE_ADV_START_SEND:
	case LE_ADV_STOP_SEND:
		if (bt_isIdleHCIcommand()) {
			HCI_STORE16(cmd.Op_Code, HCI_LE_Set_Advertise_Enable);
			cmd.Parameter_Total_Length = sizeof(cmd.Parameter.LE_Set_Advertise_Enable);
			cmd.Parameter.LE_Set_Advertise_Enable.Advertising_Enable = (state == LE_ADV_START_SEND) ? HCI_Parm_Advertising_Enabled : HCI_Parm_Advertising_Disabled;
			bt_sendHCIcommand();
			state = EVENT_WAIT;
		} else if (try++ > BT_SEND_TIMEOUT) {
			DEBUGP("L2CAP-RM: LE_ADV_START_SEND timeout handle=%04x\n", nowLnk->handle);
			rc    = BT_L2CAP_ERR_HCI_CMD_SEND_TIMEOUT;
			state = EVENT_WAIT;
		}
		break;

	default:
		break;
	}

	return(rc);
}
