//
//  bt_sdp_server.c
//
//  Bluetooth Protocol Stack - SDP Server
//  Copyright (C) 2013 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Reference:
//		Bluetooth SIG (www.bluetooth.com)
//			BLUETOOTH SPECIFICATION Version 2.0 + EDR
//			Volume 3: Core System Package [Host volume]
//			PartB: Service Discovery Protocol (SDP) Specification
//
//	Update history
//	---------- ----- -------------------------------------------
//	2013.02.01 v0.0  First cording
//	2013.04.21 v0.1  Commit
//
#include <stddef.h>
#include <stdint.h>
#include <string.h>

//#define DEBUG

//
//	Protocol format
//
struct bt_sdpFormat {
	uint8_t	PDU_ID;
	uint8_t	TransactionID[2];
	uint8_t	ParameterLength[2];
	union {
		struct {
			uint8_t	ErrorCode[2];
		}		ErrorResponse;
		struct {
			uint8_t	ServiceSearchPattern[1];		// Data Element Sequence UUID format
//			uint8_t	MaximumServiceRecordCount[2];
//			uint8_t	ContinuationState[1..17];
//			uint8_t	ServiceSearchPattern[];
		}		ServiceSearchRequest;
		struct {
			uint8_t	TotalServiceRecordCount[2];
			uint8_t	CurrentServiceRecordCount[2];
			uint8_t	ServiceRecordHandleList[];		// CurrentServiceRecordCount*4 bytes
//			uint8_t	ContinuationState[1..17];
		}		ServiceSearchResponse;
		struct {
			uint8_t	ServiceRecordHandle[4];
			uint8_t	MaximumAttributeByteCount[2];
			uint8_t	AttributeIDList[];				// Data Element Sequence
//			uint8_t	ContinuationState[1..17];
		}		ServiceAttributeRequest;
		struct {
			uint8_t	AttributeListByteCount[2];
			uint8_t	AttributeList[];
//			uint8_t	ContinuationState[1..17];
		}		ServiceAttributeResponse;
		struct {
			uint8_t	ServiceSearchPattern[1];		// Data Element Sequence UUID format
//			uint8_t	MaximumAttributeByteCount[2];
//			uint8_t	AttributeIDList[];				// Data Element Sequence
//			uint8_t	ContinuationState[1..17];
		}		ServiceSearchAttributeRequest;
		struct {
			uint8_t	AttributeListByteCount[2];
			uint8_t	AttributeList[];
//			uint8_t	ContinuationState[1..17];
		}		ServiceSearchAttributeResponse;
	}		Parameter;
}   __attribute__ ((packed));
#define SDP_HEAD_SIZE								5

#define SDP_ErrorResponse							0x01
#define SDP_ServiceSearchRequest					0x02
#define SDP_ServiceSearchResponse					0x03
#define SDP_ServiceAttributeRequest					0x04
#define SDP_ServiceAttributeResponse				0x05
#define SDP_ServiceSearchAttributeRequest			0x06
#define SDP_ServiceSearchAttributeResponse			0x07

#define SDP_ErrorCode_Invalid_unsupported_SDP_version			0x0001
#define SDP_ErrorCode_Invalid_Service_Record_Handle				0x0002
#define SDP_ErrorCode_Invalid_request_syntax					0x0003
#define SDP_ErrorCode_Invalid_PDU_Size							0x0004
#define SDP_ErrorCode_Invalid_Continuation_State				0x0005
#define SDP_ErrorCode_Insufficient_Resources_to_satisfy_Request	0x0006

#define SDP_UINT16(a)		(uint16_t)(                              (a[0] << 8) | a[1])
#define SDP_UINT24(a)		(uint32_t)(               (a[0] << 16) | (a[1] << 8) | a[2])
#define SDP_UINT32(a)		(uint32_t)((a[0] << 24) | (a[1] << 16) | (a[2] << 8) | a[3])
#define SDP_STORE16(d,s)	(                                                          d[0] = (uint8_t)((s) >> 8), d[1] = (uint8_t)(s))
#define SDP_STORE24(d,s)	(                             d[0] = (uint8_t)((s) >> 16), d[1] = (uint8_t)((s) >> 8), d[2] = (uint8_t)(s))
#define SDP_STORE32(d,s)	(d[0] = (uint8_t)((s) >> 24), d[1] = (uint8_t)((s) >> 16), d[2] = (uint8_t)((s) >> 8), d[3] = (uint8_t)(s))

#define SDP_DATA_ELEMENT_TYPE_UNSIGNED_INTEGER		0b00001000
#define SDP_DATA_ELEMENT_TYPE_SIGNED_INTEGER		0b00010000
#define SDP_DATA_ELEMENT_TYPE_UUID					0b00011000
#define SDP_DATA_ELEMENT_TYPE_TEXT					0b00100000
#define SDP_DATA_ELEMENT_TYPE_BOOLEAN				0b00101000
#define SDP_DATA_ELEMENT_TYPE_ELEMENT_SEQUENCE		0b00110000
#define SDP_DATA_ELEMENT_TYPE_ELEMENT_ALTERNATIVE	0b00111000
#define SDP_DATA_ELEMENT_TYPE_URL					0b01000000

#define SDP_DATA_ELEMENT_SIZE_8BIT					0b00000000
#define SDP_DATA_ELEMENT_SIZE_16BIT					0b00000001
#define SDP_DATA_ELEMENT_SIZE_32BIT					0b00000010
#define SDP_DATA_ELEMENT_SIZE_64BIT					0b00000011
#define SDP_DATA_ELEMENT_SIZE_128BIT				0b00000100
#define SDP_DATA_ELEMENT_SIZE_VARIABLE_8			0b00000101
#define SDP_DATA_ELEMENT_SIZE_VARIABLE_16			0b00000110
#define SDP_DATA_ELEMENT_SIZE_VARIABLE_32			0b00000111

#define SDP_DATA_UINT8			(SDP_DATA_ELEMENT_TYPE_UNSIGNED_INTEGER | SDP_DATA_ELEMENT_SIZE_8BIT)
#define SDP_DATA_UINT16			(SDP_DATA_ELEMENT_TYPE_UNSIGNED_INTEGER | SDP_DATA_ELEMENT_SIZE_16BIT)
#define SDP_DATA_UINT32			(SDP_DATA_ELEMENT_TYPE_UNSIGNED_INTEGER | SDP_DATA_ELEMENT_SIZE_32BIT)
#define SDP_DATA_UUID16			(SDP_DATA_ELEMENT_TYPE_UUID | SDP_DATA_ELEMENT_SIZE_16BIT)
#define SDP_DATA_UUID32			(SDP_DATA_ELEMENT_TYPE_UUID | SDP_DATA_ELEMENT_SIZE_32BIT)
#define SDP_DATA_STRING8		(SDP_DATA_ELEMENT_TYPE_TEXT | SDP_DATA_ELEMENT_SIZE_VARIABLE_8)
#define SDP_DATA_ELEMENT8		(SDP_DATA_ELEMENT_TYPE_ELEMENT_SEQUENCE | SDP_DATA_ELEMENT_SIZE_VARIABLE_8)
#define SDP_DATA_ELEMENT16		(SDP_DATA_ELEMENT_TYPE_ELEMENT_SEQUENCE | SDP_DATA_ELEMENT_SIZE_VARIABLE_16)

#define SDP_DATA_UINT8_2(a)		SDP_DATA_UINT8,a
#define SDP_DATA_UINT16_3(a)	SDP_DATA_UINT16,(uint8_t)(a>>8),(uint8_t)a
#define SDP_DATA_UINT32_5(a)	SDP_DATA_UINT32,(uint8_t)(a>>24),(uint8_t)(a>>16),(uint8_t)(a>>8),(uint8_t)a
#define SDP_DATA_UUID16_3(a)	SDP_DATA_UUID16,(uint8_t)(a>>8),(uint8_t)a
#define SDP_DATA_UUID32_5(a)	SDP_DATA_UUID32,(uint8_t)(a>>24),(uint8_t)(a>>16),(uint8_t)(a>>8),(uint8_t)a
#define SDP_DATA_ELEMENT8_2(a)	SDP_DATA_ELEMENT8,a
#define SDP_DATA_ELEMENT16_3(a)	SDP_DATA_ELEMENT16,(uint8_t)(a>>8),(uint8_t)a

//
//	https://www.bluetooth.org/Technical/AssignedNumbers/service_discovery.htm
//
// Protocol Identifiers
#define SDP_PROTOCOL_SDP	0x0001
#define SDP_PROTOCOL_RFCOMM	0x0003
#define SDP_PROTOCOL_L2CAP	0x0100
// Service classes and Profile Identifiers
#define SDP_SERVICE_CLASS_ServiceDiscoveryServerServiceClassID	0x1000
	#define SDP_ATTRIBUTE_VersionNumberList						0x0200
	#define SDP_ATTRIBUTE_ServiceDatabaseState					0x0201
#define SDP_SERVICE_CLASS_BrowseGroupDescriptorServiceClassID	0x1001
	#define SDP_ATTRIBUTE_GroupID								0x0200
#define SDP_SERVICE_CLASS_SerialPort							0x1101	// Serial Port Profile (SPP)
// Bluetooth Core Specification: Universal Attributes
#define SDP_ATTRIBUTE_ServiceRecordHandle				0x0000	// uint32
#define SDP_ATTRIBUTE_ServiceClassIDList				0x0001	// element seq (uuid x n)
#define SDP_ATTRIBUTE_ServiceRecordState				0x0002	// uint32
#define SDP_ATTRIBUTE_ServiceID							0x0003	// uuid
#define SDP_ATTRIBUTE_ProtocolDescriptorList			0x0004	// elemnt seq or alter
#define SDP_ATTRIBUTE_BrowseGroupList					0x0005	// elem seq (uuid x n)
#define SDP_ATTRIBUTE_LanguageBaseAttributeIDList		0x0006	// elem seq (uint16 x 3)
#define SDP_ATTRIBUTE_ServiceInfoTimeToLive				0x0007	// uint32
#define SDP_ATTRIBUTE_ServiceAvailability				0x0008	// uint8
#define SDP_ATTRIBUTE_BluetoothProfileDescriptorList	0x0009	// elem seq
#define SDP_ATTRIBUTE_DocumentationURL					0x000a	// elem seq
#define SDP_ATTRIBUTE_ClientExecutableURL				0x000b	// elem seq
#define SDP_ATTRIBUTE_IconURL							0x000c	// elem seq
#define SDP_ATTRIBUTE_AdditionalProtocolDescriptorList	0x000d	// element seq
#define SDP_ATTRIBUTE_ServiceName						0x0100	// text

//
//	Debug
//
#ifdef DEBUG
#include <stdio.h>						// Debug
#define DEBUGP(...)		printf(__VA_ARGS__)
#else
#define DEBUGP(...)
#endif

//
//	SDP database
//
const uint8_t bt_sdpRecords[] = {
	SDP_ServiceSearchAttributeResponse,		// PDU ID
	0x00, 0x01,	// Transaction ID
	0, 109,		// ParameterLength

	0, 106,					// AttributeListsByteCount
	SDP_DATA_ELEMENT16_3(103),	// AttributeLists
		SDP_DATA_ELEMENT16_3(100),	// AttributeLists#0
			SDP_DATA_UINT16_3(SDP_ATTRIBUTE_ServiceRecordHandle),			SDP_DATA_UINT32_5(12345),
			SDP_DATA_UINT16_3(SDP_ATTRIBUTE_ServiceName),					SDP_DATA_STRING8, 11, 'S', 'e', 'r', 'i', 'a', 'l', ' ', 'P', 'o', 'r', 't',
			SDP_DATA_UINT16_3(SDP_ATTRIBUTE_BrowseGroupList),				SDP_DATA_ELEMENT8_2(3),
																				SDP_DATA_UUID16_3(SDP_ATTRIBUTE_GroupID),
			SDP_DATA_UINT16_3(SDP_ATTRIBUTE_ServiceClassIDList),			SDP_DATA_ELEMENT8_2(5),
																				SDP_DATA_UUID32_5(SDP_SERVICE_CLASS_SerialPort),
			SDP_DATA_UINT16_3(SDP_ATTRIBUTE_ProtocolDescriptorList),		SDP_DATA_ELEMENT8_2(16),
																				SDP_DATA_ELEMENT8_2(5),			// Protocol#0
																					SDP_DATA_UUID32_5(SDP_PROTOCOL_L2CAP),
																				SDP_DATA_ELEMENT8_2(7),			// Protocol#1
																					SDP_DATA_UUID32_5(SDP_PROTOCOL_RFCOMM),
																					SDP_DATA_UINT8_2(1),		// Server Channel# 1..30
			SDP_DATA_UINT16_3(SDP_ATTRIBUTE_BluetoothProfileDescriptorList),SDP_DATA_ELEMENT8_2(10),
																				SDP_DATA_ELEMENT8_2(8),			// Profile#0
																					SDP_DATA_UUID32_5(SDP_SERVICE_CLASS_SerialPort),	// Supported Profiles
																					SDP_DATA_UINT16_3(0x0102),	// Profile Version
			SDP_DATA_UINT16_3(SDP_ATTRIBUTE_LanguageBaseAttributeIDList),	SDP_DATA_ELEMENT8_2(9),
																				SDP_DATA_UINT16, 'e', 'n',		// Language code: ISO639: english
																				SDP_DATA_UINT16_3(106),			// Encode: IANA's database: UTF-8
																				SDP_DATA_UINT16_3(0x0100),		// ?
			SDP_DATA_UINT16_3(SDP_ATTRIBUTE_ServiceRecordState),			SDP_DATA_UINT32_5(15),
	0		// ContinuationState
};


//
//	Server
//
size_t bt_sdpServer(unsigned short handle, unsigned char *inBuff, size_t inLen, unsigned char *outBuff, size_t outSize) {
	struct bt_sdpFormat	*in;
	struct bt_sdpFormat	*out;
	size_t				len;

	in  = (struct bt_sdpFormat *)inBuff;
	out = (struct bt_sdpFormat *)outBuff;
	len = 0;
	if (in != NULL) {
		unsigned short	tran;
		size_t			plen;

		tran = SDP_UINT16(in->TransactionID);
		plen = SDP_UINT16(in->ParameterLength);
		switch(in->PDU_ID) {
		case SDP_ServiceSearchRequest:
			DEBUGP("SDP: ServiceSearchRequest len=%d\n", plen);
			out->PDU_ID = SDP_ServiceSearchResponse;
			SDP_STORE16(out->TransactionID, tran);
			SDP_STORE16(out->ParameterLength, sizeof(out->Parameter.ServiceSearchResponse) + 5);
			SDP_STORE16(out->Parameter.ServiceSearchResponse.TotalServiceRecordCount, 1);
			SDP_STORE16(out->Parameter.ServiceSearchResponse.CurrentServiceRecordCount, 1);
			SDP_STORE32(out->Parameter.ServiceSearchResponse.ServiceRecordHandleList, 12345);
			out->Parameter.ServiceSearchResponse.ServiceRecordHandleList[4] = 0;	// No ContinuationState
			len = SDP_HEAD_SIZE + sizeof(out->Parameter.ServiceSearchResponse) + 5 ;
			break;
		case SDP_ServiceAttributeRequest:
			DEBUGP("SDP: ServiceAttributeRequest len=%d\n", plen);
			memcpy(out, bt_sdpRecords, sizeof(bt_sdpRecords));
			out->PDU_ID = SDP_ServiceAttributeResponse;
			SDP_STORE16(out->TransactionID, tran);
			len = sizeof(bt_sdpRecords);
			break;
		case SDP_ServiceSearchAttributeRequest:
			DEBUGP("SDP: ServiceSearchAttributeRequest len=%d\n", plen);
			memcpy(out, bt_sdpRecords, sizeof(bt_sdpRecords));
			out->PDU_ID = SDP_ServiceSearchAttributeResponse;
			SDP_STORE16(out->TransactionID, tran);
			len = sizeof(bt_sdpRecords);
			break;
		default:
			DEBUGP("SDP: Unsupported Request %02x len=%d\n", in->PDU_ID, plen);
			out->PDU_ID = SDP_ErrorResponse;
			SDP_STORE16(out->TransactionID, tran);
			SDP_STORE16(out->ParameterLength, sizeof(out->Parameter.ErrorResponse));
			SDP_STORE16(out->Parameter.ErrorResponse.ErrorCode, SDP_ErrorCode_Invalid_request_syntax);
			len = SDP_HEAD_SIZE + sizeof(out->Parameter.ErrorResponse);
			break;
		}
	}
	return(len);
}
