//
//  s95160.h
//  Graphic LCD module "SUCCESS ELECTRONIC S95160" I/O library
//  Copyright(C)2013-2014 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Description
//		I/O control for Graphic LCD module.
//		LCD module: SUCCESS ELECTRONIC S95160. (Controller chip is SOLOMON SYSTECH SSD1289Z)
//		Interface: 16bits 8080 parallel by software control.
//			D0-15: 16bit-port or split 2 port    default PD0-3 PE4-15
//			CS   : Any output port               default PC11
//			D/C  : Any output port               default PC6
//			WR   : Any output port               default PC8
//			RD   : Any output port               default PC9
//                 See "a95160_1.h"
//			Y-   : Any analog port               default PC1
//			X-   : Any analog port               default PC2
//			Y+   : Any analog port               default PC3
//			X+   : Any analog port               default PC4
//                 See "a95160_t1.h"
//		Screen direction:
//			X: 0=Left .. 239=Right
//			Y: 0=Top .. 319=Bottom
//		Color mode: 64k
//
//  Device: STM32F4
//      IDE     : Eclipse IDE for C/C++ Developers, Juno Service Release 1
//      Compiler: gcc-arm-none-eabi 4.7
//      Library : STM32F4xx Standard Peripherals Library Drivers (StdPeriph_Driver) V1.0.0
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2013.01.22 v0.0  First cording
//    2014.01.02 v0.1b Support for GUI
//

#ifndef S95160_H_
#define S95160_H_

#include "s95160_2.h"
#include "s95160_3.h"
#include "s95160_t2.h"
#include "s95160_s.h"					// ADD 2014.01.02

//
// Prototype
//
//	Character display emulation
//		printf(), puts()...  ("s95160_3.h" VCD_STDIO option)
//		void lcd_print(char *String)
//		void lcd_position(short Column, short Row)
//		void lcd_control(unsigned char Command)
//		void lcd_color(unsigned short ColorCode)
//		void lcd_colors(unsigned short FaceColor, unsigned short BackColor)
//	Character display emulation (Multi-view version)
//		void vcd_init(vcd_desc *P, short X, short Y, short Width, short Height);
//		void vcd_print(vcd_desc *P, char *String);
// 		void vcd_position(vcd_desc *P, short column, short row);
// 		void vcd_control(vcd_desc *P, unsigned char Command);
//		void vcd_color(vcd_desc *P, color);
//		void vcd_colors(vcd_desc *P, unsigned short FaceColor, unsigned short BackColor);
//	Primitive figure elements
//		void lcd_drawOpen(void);
//		void lcd_drawRect(short X, short Y, short Width, short Height, unsigned short Color);
//		void lcd_drawLine(short X1, short Y1, short X2, short Y2, short LineSize, unsigned short Color);
//		void lcd_drawImage(short X, short Y, short width, short Height, unsigned short *Image);
//		void lcd_drawChar(short X, short Y, unsigned short FaceColor, unsigned short BackColor, char Char);
//		void lcd_drawStr(short X, short Y, unsigned short FaceColor, unsigned short BackColor, char *String);
//		void lcd_drawInit(void);
//	Touch screen input
//		short lcd_touchPoint(short *X, short *Y);		// PushRate[%]
//		void  lcd_touchCalibrate(void);
//	GUI assist parts
//		struct lcd_segmentFigure *		lcd_segmentAllocFigure(size_t len);
//		struct lcd_segmentRect*			lcd_segmentAllocRect(void)
//		struct lcd_segmentPoly*			lcd_segmentAllocPoly(void)
//		struct lcd_segmentStr*			lcd_segmentAllocStr(void)
//		struct lcd_segmentImage*		lcd_segmentAllocImage(void)
//		struct lcd_segmentText*			lcd_segmentAllocText(void)
//		struct lcd_segmentGage*			lcd_segmentAllocGage(void)
//		struct lcd_segmentBarChart*		lcd_segmentAllocBarChart(void)
//		struct lcd_segment7seg*			lcd_segmentAlloc7seg(void)
//		struct lcd_segmentGroup*		lcd_segmentAllocGroup(void);
//		struct lcd_segmentButton*		lcd_segmentAllocButton(void);
//		struct lcd_segmentKeyPad *		lcd_segmentAllocKeyPad(void);
//		struct lcd_segmentTenKeyPad * 	lcd_segmentAllocTenKeyPad(void);
//		struct lcd_segmentSelectionList*lcd_segmentAllocSelectionList(n);
//		struct lcd_segmentTextField * 	lcd_segmentAllocTextField(void);
//		struct lcd_segmentIntegerField *lcd_segmentAllocIntegerField(void);
//		struct lcd_segmentFloatField * 	lcd_segmentAllocFloatField(void);
//		struct lcd_segmentPullDownField*lcd_segmentAllocPullDownField(void);
//		void							lcd_segmentDeallocFigure(struct lcd_segmentFigure *fig);
//		struct lcd_segment *			lcd_segmentNewFigure(struct lcd_segmentFigure *brother, struct lcd_segmentFigure *fig, size_t l, int (*dispProg)(struct lcd_segmentFigure *fig, struct lcd_segmentCoordinate *base));
//		struct lcd_segment* 			lcd_segmentNewRect(struct lcd_segmentFigure *brother,  struct lcd_segmentRect  *rect);
//		struct lcd_segment* 			lcd_segmentNewPoly(struct lcd_segmentFigure *brother,  struct lcd_segmentPoly  *poly);
//		struct lcd_segment*				lcd_segmentNewStr(struct lcd_segmentFigure *brother,  struct lcd_segmentStr   *str);
//		struct lcd_segment*				lcd_segmentNewImage(struct lcd_segmentFigure *brother,  struct lcd_segmentImage *image);
//		struct lcd_segment*				lcd_segmentNewText(struct lcd_segmentFigure *brother,  struct lcd_segmentText  *str);
//		struct lcd_segment*				lcd_segmentNewGage(struct lcd_segmentFigure *brother,  struct lcd_segmentGage  *gage);
//		struct lcd_segment*				lcd_segmentNewBarChart(struct lcd_segmentFigure *brother,  struct lcd_segmentBarChart  *bar);
//		struct lcd_segment*				lcd_segmentNew7seg( struct lcd_segmentFigure *brother,  struct lcd_segment7seg  *led7seg);
//		struct lcd_segment *			lcd_segmentNewGroup(struct lcd_segmentGroup *parentGroup, struct lcd_segmentGroup *myGroup);
//		struct lcd_segment *			lcd_segmentNewButton(struct lcd_segmentGroup *g, struct lcd_segmentButton *button);
//		struct lcd_segment *			lcd_segmentNewKeyPad(struct lcd_segmentGroup *g, struct lcd_segmentKeyPad *keyPad);
//		struct lcd_segment *			lcd_segmentNewTenKeyPad(struct lcd_segmentGroup *g, struct lcd_segmentTenKeyPad *keyPad);
//		struct lcd_segment *			lcd_segmentNewSelectionList(struct lcd_segmentGroup *g, struct lcd_segmentSelectionList *list);
//		struct lcd_segment *			lcd_segmentNewTextField(struct lcd_segmentGroup *g, struct lcd_segmentTextField *textField);
//		struct lcd_segment *			lcd_segmentNewIntegerField(struct lcd_segmentGroup *g, struct lcd_segmentIntegerField *field);
//		struct lcd_segment *			lcd_segmentNewFloatField(struct lcd_segmentGroup *g, struct lcd_segmentFloatField *field);
//		struct lcd_segment *			lcd_segmentNewPullDownField(struct lcd_segmentGroup *g, struct lcd_segmentPullDownField *field);
//		void							lcd_segmentDelete(void *fig);
//		void							lcd_segmentSetDirty(void *fig);
//		void							lcd_segmentSetHidden(void *fig, int f);
//		int								lcd_segmentSetIntegerFieldValue(struct lcd_segmentIntegerField *g, int i);
//		int								lcd_segmentSetFloatFieldValue(struct lcd_segmentFloatField *g, float f);
//		void							lcd_segmentSetPullDownFieldSelect(struct lcd_segmentPullDownField *g, int select);
//		struct lcd_segment *			lcd_segmentGetGroupToSegment(struct lcd_segmentGroup *fig);
//		struct lcd_segmentGroup *		lcd_segmentGetGroupFromSegment(struct lcd_segment *seg);
//		struct lcd_segmentGroup *		lcd_segmentGetGroupParent(struct lcd_segmentGroup *fig);
//		int								lcd_segmentGetIntegerFieldValue(struct lcd_segmentIntegerField *g);
//		float							lcd_segmentGetFloatFieldValue(struct lcd_segmentFloatField *g);
//		int								lcd_segmentGetPullDownFieldSelect(struct lcd_segmentPullDownField *g);
//		struct lcd_segmentGroup *		lcd_segmentFindGroupByPoint(struct lcd_segmentGroup *fig, struct lcd_segmentCoordinate *point);
//		void							lcd_segmentHandler(struct lcd_segmentGroup *rootGroup, struct lcd_segmentCoordinate *point);
//		Refer:
//			s95160_s1.h: Layer1 (Primitive segment, horizontal relation)
//			s95160_s2.h: Layer2 (Hierarchy segment, Vertical relation, Touch action)
//			s95160_s3.h: Layer3 (Composite segment)
//
//	Control character code
//		'\n' New line
//		'\r' Carriage return
//		'\f' Clear home
//		'\v' Cursor home
//
//	Control command
//		LCD_DISP_CLEAR_HOME
//		LCD_CURSOR_ON
//		LCD_CURSOR_OFF
//		LCD_CURSOR_HOME
//		LCD_CURSOR_LEFT
//		LCD_CURSOR_RIGHT
//
//	Color code
//		LCD_COLOR_CODE(float red, float green, float blue)
//		LCD_COLOR_BLACK
//		LCD_COLOR_DIMGRAY
//		LCD_COLOR_GRAY
//		LCD_COLOR_DARKGRAY
//		LCD_COLOR_LIGHTGRAY
//		LCD_COLOR_WHITE
//		LCD_COLOR_IVORY
//		LCD_COLOR_LAVENDER
//		LCD_COLOR_STEELBLUE
//		LCD_COLOR_DARKBLUE
//		LCD_COLOR_BLUE
//		LCD_COLOR_ROYALBLUE
//		LCD_COLOR_SKYBLUE
//		LCD_COLOR_LIGHTBLUE
//		LCD_COLOR_CYAN
//		LCD_COLOR_LIGHTCYAN
//		LCD_COLOR_DARKCYAN
//		LCD_COLOR_DARKGREEN
//		LCD_COLOR_GREEN
//		LCD_COLOR_FORESTGREEN
//		LCD_COLOR_SEAGREEN
//		LCD_COLOR_LIGHTGREEN
//		LCD_COLOR_GREENYELLOW
//		LCD_COLOR_LIME
//		LCD_COLOR_YELLOWGREEN
//		LCD_COLOR_OLIVEDRAB
//		LCD_COLOR_OLIVE
//		LCD_COLOR_TAN
//		LCD_COLOR_KHAKI
//		LCD_COLOR_YELLOW
//		LCD_COLOR_GOLD
//		LCD_COLOR_ORANGE
//		LCD_COLOR_CHOCOLATE
//		LCD_COLOR_DARKRED
//		LCD_COLOR_BROWN
//		LCD_COLOR_SALMON
//		LCD_COLOR_CORAL
//		LCD_COLOR_TOMATO
//		LCD_COLOR_RED
//		LCD_COLOR_DEEPPINK
//		LCD_COLOR_PINK
//		LCD_COLOR_MAGENTA
//		LCD_COLOR_VIOLET
//		LCD_COLOR_PURPLE
//		LCD_COLOR_INDIGO
//		LCD_COLOR_SLATEBLUE
//
//	General parameter
//		LCD_SCREEN_WIDTH	240
//		LCD_SCREEN_HEIGHT	320
//		LCD_FONT_WIDTH		6
//		LCD_FONT_HEIGHT		8
//
#endif /* S95160_H_ */
