//
//  s95160_s1.c	  Segment buffer support Layer1 (Primitive segment, horizontal relation)
//  Graphic LCD module "SUCCESS ELECTRONIC S95160" I/O library
//  Copyright(C)2013-2014 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2013.12.20 v0.0  First cording
//    2013.12.27 v0.1  Release
//    2013.12.28 v0.1a Reducing the arguments
//    2013.12.30 v0.1b Scratch and build, value base -> refer base arguments
//    2014.01.02 v0.1b Support the Gage, 7segLED figure
//    2014.01.15 v0.1b Support the Bar chart figure
//    2014.01.15 v0.1b Bug fix
//

#include <stdio.h>				// DEBUG
#include <stdlib.h>				// calloc,rand...
#include <string.h>				// strcpy...
#include "s95160_2.h"			// Display driver
#include "s95160_s1.h"			// My definition for external

// Inline function
#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define ABS(a)		(((a) >  0 ) ? (a) : -(a))

#define FILL_TEXT
#define V01B

//
//	Global area
//
struct lcd_segmentGlobalFlags	lcd_segmentGlobalFlags;


//
//	Segment control block - header
//
struct lcd_segment *lcd_segmentAllocSeg(struct lcd_segment *top){
	struct lcd_segment *new;

	new  = calloc(1, sizeof(struct lcd_segment));
	if (new != NULL) {
		if (top != NULL) {
			new->next       = top->next;
			top->next       = new;

			new->prev       = top;
			if (new->next != NULL) {
				new->next->prev = new;
			}
		}
	}
//	printf("%8x %8x,%8x,%8x\n", top, new, new->next, new->prev);
	return(new);
}

void lcd_segmentDeallocSeg(struct lcd_segment *s){
	if (s != NULL) {
		if (s->next != NULL) {
			s->next->prev = s->prev;
		}
		if (s->prev != NULL) {
			s->prev->next = s->next;
		}

		free(s);
	}
	return;
}

//
//	General segments
//
struct lcd_segmentFigure *lcd_segmentAllocFigure(size_t len) {
	struct lcd_segmentFigure	*m;

	m = calloc(1, len);
	if (m) {
		m->flags.malloc = 1;	// Marking
	}
	return(m);
}

void lcd_segmentDeallocFigure(struct lcd_segmentFigure *m) {
	if (m) {
		if (m->flags.malloc) {
			free(m);
		} else {
			m->seg = NULL;
		}
	}
	return;
}

struct lcd_segment *lcd_segmentNewFigure(struct lcd_segmentFigure *brother, struct lcd_segmentFigure *fig, size_t l, int (*dispProg)(struct lcd_segmentFigure *fig, struct lcd_segmentCoordinate *base)) {
	struct lcd_segment			*top;
	struct lcd_segment			*my;
	struct lcd_segmentFigure	*new;

	new = NULL;
	if (brother) {
		top = brother->seg;
	} else {
		top = NULL;						// Independent group (root)
	}
	my  = lcd_segmentAllocSeg(top);
	if (fig == NULL) {
		new = lcd_segmentAllocFigure(l);
		fig = new;
	}
	if (my && fig) {
		my->link         = fig;
		my->dispProg     = dispProg;
		fig->seg         = my;
		lcd_segmentSetFigureDirty(fig);
	} else {
		if (my) {
			lcd_segmentDeallocSeg(my);
		}
		if (new) {
			lcd_segmentDeallocFigure(new);
		}
		my = NULL;
	}
	return(my);
}

void lcd_segmentDeleteFigure(struct lcd_segmentFigure *target){
	if (target != NULL) {
		lcd_segmentDeallocSeg(target->seg);
		lcd_segmentDeallocFigure(target);
	}
	return;
}

void lcd_segmentSetFigureDirty(struct lcd_segmentFigure	*s){
	if (s) {
		s->flags.dirty               = 1;
		lcd_segmentGlobalFlags.dirty = 1;
	}
	return;
}

void lcd_segmentSetFigureHidden(struct lcd_segmentFigure *s, int f){
	if (s) {
		if (s->flags.hidden != f) {
			lcd_segmentSetFigureDirty(s);
			s->flags.hidden = f;
		}
	}
	return;
}



//
//	Rect segment
//
int lcd_segmentDisplayRect(struct lcd_segmentFigure *fig, struct lcd_segmentCoordinate *cardinalPoint){
	struct lcd_segmentRect *p;

	p = (struct lcd_segmentRect *)fig;
	lcd_drawRect(
			p->area.x + cardinalPoint->x,
			p->area.y + cardinalPoint->y,
			p->area.width,
			p->area.height,
			p->color);
	return(1);
};


//
//	Poly line segment
//
int lcd_segmentDisplayPoly(struct lcd_segmentFigure *fig, struct lcd_segmentCoordinate *cardinalPoint){
	struct lcd_segmentPoly			*p;
	struct lcd_segmentCoordinate	*c;
	int								n;

	p = (struct lcd_segmentPoly *)fig;
	c = p->point;
	for(n = p->n - 1; n > 0; --n) {
		lcd_drawLine(
			c->x     + cardinalPoint->x,
			c->y     + cardinalPoint->y,
			(c+1)->x + cardinalPoint->x,
			(c+1)->y + cardinalPoint->y,
			p->lineSize,
			p->color);
		c++;
	}
	return(1);
};

//
//	Character string segment
//
int lcd_segmentDisplayStr(struct lcd_segmentFigure *fig, struct lcd_segmentCoordinate *cardinalPoint){
	struct lcd_segmentStr			*p;
	struct lcd_segmentCoordinate	a;

	p   = (struct lcd_segmentStr *)fig;
	a.x = p->point.x + cardinalPoint->x;
	a.y = p->point.y + cardinalPoint->y;

	lcd_drawStr(
			a.x,
			a.y,
			p->color.face,
			p->color.back,
			p->str);
	return(1);
};

//
//	Bit image segment
//
int lcd_segmentDisplayImage(struct lcd_segmentFigure *fig, struct lcd_segmentCoordinate *cardinalPoint){
	struct lcd_segmentImage *p;

	p = (struct lcd_segmentImage *)fig;
	lcd_drawImage(
			p->area.x + cardinalPoint->x,
			p->area.y + cardinalPoint->y,
			p->area.width,
			p->area.height,
			p->image);
	return(1);
};

//
//	Text string segment
//
int lcd_segmentDisplayText(struct lcd_segmentFigure *fig, struct lcd_segmentCoordinate *cardinalPoint){
	struct lcd_segmentText	*p;
	struct lcd_segmentArea	a;
	char					*str;
	int						maxLine;
	int						maxColumn;
	int						i;
	int						j;
	int						k;

	p         = (struct lcd_segmentText *)fig;
	maxLine   = p->area.height / LCD_FONT_HEIGHT;
	maxColumn = MIN(p->area.width / LCD_FONT_WIDTH, LCD_SEGMENT_STRING_COLUMN_MAX);
	k   = 0;
	str = p->str;
	for(j = 0; j < maxLine; j++) {
		for(i = 0; i < maxColumn; i++) {
			if ((*str == '\0') || (*str == '\n')) {
				break;
			}
			str++;
		}
		k = MAX(k, i);
		if (*str == '\0') {
			break;
		}
	}
	maxLine   = MIN(maxLine, j + 1);
	maxColumn = k;

	a.x       = p->area.x + cardinalPoint->x;
	a.y       = p->area.y + cardinalPoint->y;
	a.width   = maxColumn * LCD_FONT_WIDTH;
	a.height  = maxLine * LCD_FONT_HEIGHT;
	switch(p->attr.horizontalAlign){
	case LCD_SEGMENT_TEXT_ALIGN_CENTER:
		a.x += ((p->area.width - a.width) / 2);
		break;
	case LCD_SEGMENT_TEXT_ALIGN_RIGHT:
		a.x += (p->area.width - a.width);
		break;
	default:
		break;
	}
	switch(p->attr.varticalAlign){
	case LCD_SEGMENT_TEXT_ALIGN_CENTER:
		a.y += ((p->area.height - a.height) / 2);
		break;
	case LCD_SEGMENT_TEXT_ALIGN_DOWN:
		a.y += (p->area.height - a.height);
		break;
	default:
		break;
	}

#ifdef FILL_TEXT
	lcd_drawRect(
			p->area.x + cardinalPoint->x,
			p->area.y + cardinalPoint->y,
			p->area.width,
			p->area.height,
			p->color.back);
#endif
	str = p->str;
	for(; maxLine > 0; --maxLine) {
		char	buf[LCD_SEGMENT_STRING_COLUMN_MAX+1];
		char	*c;
		int		i;

		c = buf;
		for(i = 0; i < maxColumn; i++) {
			if ((*str == '\0') || (*str == '\n')) {
				break;
			}
			*c++ = *str++;
		}
		*c = '\0';
		lcd_drawStr(
				a.x,
				a.y,
				p->color.face,
				p->color.back,
				buf);
		if (*str == '\0') {
			break;
		}
		a.y += LCD_FONT_HEIGHT;
	}
	return(1);
};

//
//	Gage segment
//
int lcd_segmentDisplayGage(struct lcd_segmentFigure *fig, struct lcd_segmentCoordinate *cardinalPoint){
	struct lcd_segmentGage *p;
	int						pos;
	int						dlt;
	float					value;

	p = (struct lcd_segmentGage *)fig;
	value = MIN(p->value, 1.0);
	value = MAX(value, 0.0);
#ifdef V01B
	if (p->fig.flags.init || !(rand() & 0x1f)) {
		lcd_drawRect(
				p->area.x + cardinalPoint->x,
				p->area.y + cardinalPoint->y,
				p->area.width,
				p->area.height,
				p->color.back);
		p->fig.flags.init     = 0;
		p->internal.beforePos = 0;
	}
	if (p->attr.vartical) {
		pos = value * p->area.height + 0.5;
		dlt = pos - p->internal.beforePos;
		if (dlt > 0) {			// INC
			lcd_drawRect(
					p->area.x + cardinalPoint->x,
					p->area.y + cardinalPoint->y + p->area.height - pos,
					p->area.width,
					dlt,
					p->color.face);
		} else if (dlt < 0) {	// DEC
			dlt = -dlt;
			lcd_drawRect(
					p->area.x + cardinalPoint->x,
					p->area.y + cardinalPoint->y + p->area.height - p->internal.beforePos,
					p->area.width,
					dlt,
					p->color.back);
		}
	} else {
		pos = value * p->area.width + 0.5;
		dlt = pos - p->internal.beforePos;
		if (dlt > 0) {
			lcd_drawRect(
					p->area.x + cardinalPoint->x + p->internal.beforePos,
					p->area.y + cardinalPoint->y,
					dlt,
					p->area.height,
					p->color.face);
		} else if (dlt < 0) {
			dlt = -dlt;
			lcd_drawRect(
					p->area.x + cardinalPoint->x + pos,
					p->area.y + cardinalPoint->y,
					dlt,
					p->area.height,
					p->color.back);
		}
	}
	p->internal.beforePos = pos;
#else
	if (p->attr.vartical) {		// Vertical
		pos = value * p->area.height + 0.5;
		lcd_drawRect(
				p->area.x + cardinalPoint->x,
				p->area.y + cardinalPoint->y,
				p->area.width,
				p->area.height - pos,
				p->color.back);
		lcd_drawRect(
				p->area.x + cardinalPoint->x,
				p->area.y + cardinalPoint->y + p->area.height - pos,
				p->area.width,
				pos,
				p->color.face);
	} else {					// Horizontal
		pos = value * p->area.width + 0.5;
		lcd_drawRect(
				p->area.x + cardinalPoint->x,
				p->area.y + cardinalPoint->y,
				pos,
				p->area.height,
				p->color.face);
		lcd_drawRect(
				p->area.x + cardinalPoint->x + pos,
				p->area.y + cardinalPoint->y,
				p->area.width - pos,
				p->area.height,
				p->color.back);
	}
#endif
	return(1);
};

//
//	Bar chart
//
int lcd_segmentDisplayBarChart(struct lcd_segmentFigure *fig, struct lcd_segmentCoordinate *cardinalPoint){
	struct lcd_segmentBarChart *p;
	int				i;
	short			x;
	short			y;
	short			h;
	short			w;
	float			*v;
	unsigned short	*c;
	short			s;

	p = (struct lcd_segmentBarChart *)fig;
	x = p->area.x + cardinalPoint->x;
	y = p->area.y + cardinalPoint->y;
	if (p->attr.vartical) {		// Vertical
		v = p->value + (p->number-1);
		c = p->color + (p->number-1);
		h = p->area.height;
		for(i = p->number; i > 0; --i) {
			s = *v * p->area.height + 0.5;
			h -= s;
			lcd_drawRect(x, y+h, p->area.width, s, *c);
			--v;
			--c;
		}
		lcd_drawRect(x, y, p->area.width, h, p->backColor);
	} else {					// Horizontal
		v = p->value;
		c = p->color;
		w = p->area.width;
		for(i = p->number; i > 0; --i) {
			s = *v * p->area.width + 0.5;
			lcd_drawRect(x, y, s, p->area.height, *c);
			x += s;
			w -= s;
			v++;
			c++;
		}
		lcd_drawRect(x, y, w, p->area.height, p->backColor);
	}
	return(1);
};

//
// 7SEG LED segment
//
void disp7seg(unsigned char image, short x, short y, short w, short h, int	width, unsigned fcolor, unsigned bcolor){
	const struct {
		short	sx;
		short	sy;
		short	ex;
		short	ey;
	} sengemtLayout[8] = {
			{15, 0, 34, 0},	// a
			{32,26, 36, 2},	// b
			{26,58, 30,32},	// c
			{ 4,62, 23,62},	// d
			{ 6,32,  0,58},	// e
			{12, 2,  7,26},	// f
			{12,30, 30,30},	// g
			{36,60, 36,62},	// .		40x64
	};
	int				j;
	unsigned 		color;

	for(j = 0; j < 8; j++) {
		if (image & 0x80) {
			color = fcolor;
		} else {
			color = bcolor;
		}
		lcd_drawLine(
				sengemtLayout[j].sx * w / 40 + x,
				sengemtLayout[j].sy * h / 64 + y,
				sengemtLayout[j].ex * w / 40 + x,
				sengemtLayout[j].ey * h / 64 + y,
				width, color);
		image <<= 1;
	}
	return;
}

void disp7segs(int i, int n, int p, short x, short y, short w, short h, int width, unsigned fcolor, unsigned bcolor){
	const unsigned char	segmentFont[] = {
//            abcdefg.
			0b11111100,	// 0
			0b01100000,	// 1
			0b11011010,	// 2
			0b11110010,	// 3
			0b01100110,	// 4
			0b10110110,	// 5
			0b10111110,	// 6
			0b11100000,	// 7
			0b11111110,	// 8
			0b11110110,	// 9
	};
	short	cw;

	cw = w / n;
	x += (n * cw);
	for(; n > 0; --n) {
		unsigned char	f;

		x -= cw;
		f = segmentFont[i % 10];
		if (p == 0) {
			f |= 0b00000001;			// Dot point
		}
		disp7seg(f, x, y, cw, h, width, fcolor, bcolor);
		--p;
		i /= 10;
		if ((i == 0) && (p < 0)) {
			break;
		}
	}
	for(; n > 1; --n) {
		x -= cw;
		disp7seg(0b00000000, x, y, cw, h, width, fcolor, bcolor);
	}
	return;
}
int lcd_segmentDisplay7seg(struct lcd_segmentFigure *fig, struct lcd_segmentCoordinate *base){
	struct lcd_segment7seg *p;

	p = (struct lcd_segment7seg *)fig;
	disp7segs(
			p->value,
			p->column,
			p->dotPoint,
			p->area.x + base->x,
			p->area.y + base->y,
			p->area.width,
			p->area.height,
			p->lineWidth,
			p->color.face,
			p->color.back);
	return(1);
};
