//
//  s95160_s3.c	  Segment buffer support Layer3 (Composite segment)
//  Graphic LCD module "SUCCESS ELECTRONIC S95160" I/O library
//  Copyright(C)2013-2014 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2014.01.04 v0.1b New, Splitting a source code (s2 -> s2+s3)
//

#include <stdlib.h>						// atoi...
#include <string.h>						// strlen...
#include <ctype.h>						// toupper(),tolower()
#include <math.h>						// pow()
#include <stdio.h>						// sprintf
#include "s95160_s3.h"					//

// Inline function
#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define ABS(a)		(((a) >  0 ) ? (a) : -(a))

//
//	Input method control block
//
struct lcd_segmentGroup			*activePopup = NULL;

//
//	Button
//
void lcd_segmentTouchInButton(struct lcd_segmentGroup *g, struct lcd_segmentCoordinate *point){
	struct lcd_segmentButton	*button;

	button = (struct lcd_segmentButton *)g;
	button->attr.state               = button->attr.alternateMode ? (!button->attr.state) : 1;
	button->internal.indicator.color = button->attr.state ? LCD_COLOR_LIME : button->color.back;
	lcd_segmentSetDirty(&(button->internal.indicator));
	return;
}

void lcd_segmentTouchOutButton(struct lcd_segmentGroup *g){
	struct lcd_segmentButton	*button;

	button = (struct lcd_segmentButton *)g;
	if (!button->attr.alternateMode) {
		button->attr.state               = 0;
		button->internal.indicator.color = button->color.back;
		lcd_segmentSetDirty(&(button->internal.indicator));
	}
	if (button->actionProg != NULL) {
		button->actionProg(button);
	}
	return;
}

struct lcd_segment *lcd_segmentNewButton(struct lcd_segmentGroup *g, struct lcd_segmentButton *button) {
	struct lcd_segment	*my;

	my = lcd_segmentNewGroup(g, (struct lcd_segmentGroup *)button);		// Appropriation
	if (my != NULL) {
		button->grp.touchInProg  = lcd_segmentTouchInButton;
		button->grp.touchOutProg = lcd_segmentTouchOutButton;

		// Push status lamp
		button->internal.indicator.area.x      = 2;
		button->internal.indicator.area.y      = 2;
		button->internal.indicator.area.width  = 3;
		button->internal.indicator.area.height = 3;
		button->internal.indicator.color       = button->attr.state ? LCD_COLOR_LIME : button->color.back;

		// Embossment
		button->internal.frame[0].x  = 0;							button->internal.frame[0].y = 0;
		button->internal.frame[1].x  = button->grp.area.width-1;	button->internal.frame[1].y = 0;
		button->internal.frame[2].x  = button->grp.area.width-1;	button->internal.frame[2].y = button->grp.area.height-1;
		button->internal.frame[3].x  = 0;							button->internal.frame[3].y = button->grp.area.height-1;
		button->internal.frame[4].x  = 0;							button->internal.frame[4].y = 0;
		button->internal.edge1.color    = LCD_COLOR_WHITE;
		button->internal.edge1.lineSize = 1;
		button->internal.edge1.n        = 3;
		button->internal.edge1.point    = &(button->internal.frame[0]);
		button->internal.edge2.color    = LCD_COLOR_DARKGRAY;
		button->internal.edge2.lineSize = 1;
		button->internal.edge2.n        = 3;
		button->internal.edge2.point    = &(button->internal.frame[2]);

		// Caption text
		button->internal.text.area.x      = 1;
		button->internal.text.area.y      = 1;
		button->internal.text.area.width  = button->grp.area.width - 2;
		button->internal.text.area.height = button->grp.area.height - 2;
		button->internal.text.color.face  = button->color.face;
		button->internal.text.color.back  = button->color.back;
		button->internal.text.str         = button->caption;
		button->internal.text.attr.horizontalAlign = LCD_SEGMENT_TEXT_ALIGN_CENTER;
		button->internal.text.attr.varticalAlign   = LCD_SEGMENT_TEXT_ALIGN_CENTER;

		lcd_segmentNewPoly((struct lcd_segmentFigure *)button, &(button->internal.edge1));
		lcd_segmentNewPoly((struct lcd_segmentFigure *)button, &(button->internal.edge2));
		lcd_segmentNewText((struct lcd_segmentFigure *)button, &(button->internal.text));
		lcd_segmentNewRect((struct lcd_segmentFigure *)button, &(button->internal.indicator));
	}
	return(my);
}


//
//	ASCII Key pad
//
struct lcd_segment *lcd_segmentNewKeyPad(struct lcd_segmentGroup *g, struct lcd_segmentKeyPad *keyPad) {
	const char				*cap[4][10] = {
			{"1", "2", "3", "4", "5", "6", "7", "8", "9", "0"},
			{"Q", "W", "E", "R", "T", "Y", "U", "I", "O", "P"},
			{"A", "S", "D", "F", "G", "H", "J", "K", "L", "*"},
			{"Z", "X", "C", "V", "B", "N", "M", ".", "-", "/"},
	};
	const char				*cap2[] = {"Shift", " ", " ", "Delete", "Enter"};
	struct lcd_segment		*my;
	struct lcd_segmentArea	a;
	int						i;
	int						j;

	my = lcd_segmentNewGroup(g, (struct lcd_segmentGroup *)keyPad);		// Appropriation
	if (my != NULL) {
		a.width  = keyPad->grp.area.width / 10;
		a.height = keyPad->grp.area.height / 5;
		a.x      = 0;
		a.y      = 0;
		for(i = 0; i < 4; i++) {
			for(j = 0; j < 10; j++) {
				keyPad->internal.keyButton[i][j].grp.area.x      = a.x;
				keyPad->internal.keyButton[i][j].grp.area.y      = a.y;
				keyPad->internal.keyButton[i][j].grp.area.width  = a.width;
				keyPad->internal.keyButton[i][j].grp.area.height = a.height;
				keyPad->internal.keyButton[i][j].color.face  = keyPad->color.face;
				keyPad->internal.keyButton[i][j].color.back  = keyPad->color.back;
				keyPad->internal.keyButton[i][j].caption         = (char *)cap[i][j];
				keyPad->internal.keyButton[i][j].actionProg      = (void (*)(struct lcd_segmentButton *s))keyPad->grp.touchOutProg;
				lcd_segmentNewButton((struct lcd_segmentGroup *)keyPad, &(keyPad->internal.keyButton[i][j]));
				a.x += a.width;
			}
			a.y += a.height;
			a.x = 0;
		}
		a.width *= 2;
		for(j = 0; j < 5; j++) {
			keyPad->internal.keyButton2[j].grp.area.x      = a.x;
			keyPad->internal.keyButton2[j].grp.area.y      = a.y;
			keyPad->internal.keyButton2[j].grp.area.width  = a.width;
			keyPad->internal.keyButton2[j].grp.area.height = a.height;
			keyPad->internal.keyButton2[j].color.face  = keyPad->color.face;
			keyPad->internal.keyButton2[j].color.back  = keyPad->color.back;
			keyPad->internal.keyButton2[j].caption         = (char *)cap2[j];
			keyPad->internal.keyButton2[j].actionProg      = (j == 0) ? NULL : (void (*)(struct lcd_segmentButton *s))keyPad->grp.touchOutProg;
			keyPad->internal.keyButton2[j].attr.alternateMode = (j == 0) ? 1 : 0;
			lcd_segmentNewButton((struct lcd_segmentGroup *)keyPad, &(keyPad->internal.keyButton2[j]));
			a.x += a.width;
		}
	}
	return(my);
}


//
//	10 key pad
//
struct lcd_segment *lcd_segmentNewTenKeyPad(struct lcd_segmentGroup *g, struct lcd_segmentTenKeyPad *keyPad) {
	const char	*cap[TENKEY_NUM_Y][TENKEY_NUM_X] = {
			{"7", "8", "9","Esc"},
			{"4", "5", "6", "-"},
			{"1", "2", "3", "+"},
			{"0", ".", "C", "="},
	};
	struct lcd_segment 			*my;
	struct lcd_segmentArea		a;
	int							i;
	int							j;

	my = lcd_segmentNewGroup(g, (struct lcd_segmentGroup *)keyPad);		// Appropriation
	if (my != NULL) {
		a.width  = keyPad->grp.area.width  / TENKEY_NUM_X;
		a.height = keyPad->grp.area.height / TENKEY_NUM_Y;
		a.x      = 0;
		a.y      = 0;

		for(i = 0; i < TENKEY_NUM_Y; i++) {
			for(j = 0; j < TENKEY_NUM_X; j++) {
				keyPad->internal.keyButton[i][j].grp.area.x      = a.x;
				keyPad->internal.keyButton[i][j].grp.area.y      = a.y;
				keyPad->internal.keyButton[i][j].grp.area.width  = a.width;
				keyPad->internal.keyButton[i][j].grp.area.height = a.height;
				keyPad->internal.keyButton[i][j].color.face  = keyPad->color.face;
				keyPad->internal.keyButton[i][j].color.back  = keyPad->color.back;
				keyPad->internal.keyButton[i][j].caption         = (char *)cap[i][j];
				keyPad->internal.keyButton[i][j].actionProg      = (void (*)(struct lcd_segmentButton *s))keyPad->grp.touchOutProg;
				lcd_segmentNewButton((struct lcd_segmentGroup *)keyPad, &(keyPad->internal.keyButton[i][j]));
				a.x += a.width;
			}
			a.y += a.height;
			a.x = 0;
		}
	}
	return(my);
}


//
//	Selection list
//
struct lcd_segment *lcd_segmentNewSelectionList(struct lcd_segmentGroup *g, struct lcd_segmentSelectionList *list) {
	struct lcd_segment 			*my;
	struct lcd_segmentArea		a;
	int							i;

	my = lcd_segmentNewGroup(g, (struct lcd_segmentGroup *)list);		// Appropriation
	if (my) {
		a.width  = list->grp.area.width;
		a.height = list->grp.area.height / list->number;
		a.x      = 0;
		a.y      = 0;

		for(i = 0; i < list->number; i++) {
			list->internal.label[i].grp.area.x      = a.x;
			list->internal.label[i].grp.area.y      = a.y;
			list->internal.label[i].grp.area.width  = a.width;
			list->internal.label[i].grp.area.height = a.height;
			list->internal.label[i].color.face      = list->color.face;
			list->internal.label[i].color.back      = list->color.back;
			list->internal.label[i].caption         = (char *)list->label[i];
			list->internal.label[i].actionProg      = (void (*)(struct lcd_segmentButton *s))list->grp.touchOutProg;
			lcd_segmentNewButton((struct lcd_segmentGroup *)list, &(list->internal.label[i]));
			a.y += a.height;
		}
	}
	return(my);
}


//
//	Text field
//
void lcd_segmentTouchTextPad(struct lcd_segmentGroup *g){
	struct lcd_segmentKeyPad		*keyPad;
	struct lcd_segmentButton		*button;
	char							*target;
	size_t							targetLen;
	char							*caption;
	size_t							captionLen;
	struct lcd_segmentTextField		*field;

	button     = (struct lcd_segmentButton *)g;
	keyPad     = (struct lcd_segmentKeyPad *)lcd_segmentGetGroupParent(g);
	field      = (struct lcd_segmentTextField *)keyPad->targetField;
	target     = field->str;
	caption    = button->caption;
	targetLen  = strlen(target);
	captionLen = strlen(caption);

	if (!strcmp(caption, "Delete")) {
		if (targetLen > 0) {
			*(target + targetLen - 1) = '\0';
		}
	} else if (!strcmp(caption, "Enter")) {
		lcd_segmentDelete(keyPad);
		activePopup = 0;
		if (field->actionProg) {
			field->actionProg(field);
		}
	} else {
		if ((targetLen + captionLen) < field->len) {
			if (keyPad->internal.keyButton2[0].attr.state) {		// Shift key
				for(target += targetLen; captionLen > 0; --captionLen) {
					*(target++) = toupper((int)*(caption++));
				}
			} else {
				for(target += targetLen; captionLen > 0; --captionLen) {
					*(target++) = tolower((int)*(caption++));
				}
			}
			*target = '\0';
		}
	}
	lcd_segmentSetDirty(field);
	return;
}

void lcd_segmentTouchTextField(struct lcd_segmentGroup *g){
	struct lcd_segmentKeyPad	*keyPad;
	struct lcd_segmentTextField	*field;
	struct lcd_segmentGroup		*root;
	struct lcd_segmentGroup		*t;

	if (activePopup) {
		lcd_segmentDelete(activePopup);
	}
	keyPad  = lcd_segmentAllocKeyPad();
	if (keyPad != NULL) {
		field                    = (struct lcd_segmentTextField *)g;
		keyPad->color.face   = LCD_COLOR_WHITE;
		keyPad->color.back   = LCD_COLOR_GRAY;
		keyPad->targetField      = field;
		keyPad->grp.touchOutProg = lcd_segmentTouchTextPad;
		keyPad->grp.area.x       = 0;
		keyPad->grp.area.y       = field->grp.area.height;
		keyPad->grp.area.height  = KEYPAD_SIZE_H;
		keyPad->grp.area.width   = KEYPAD_SIZE_W;
		root = g;
		for(t = g; t != NULL; t = lcd_segmentGetGroupParent(t)) {	// Search the root
			keyPad->grp.area.x += t->area.x;		// Calculate the field coordinate on root window
			keyPad->grp.area.y += t->area.y;
			root = t;
		}
		keyPad->grp.area.x = (root->area.width - KEYPAD_SIZE_W) / 2;
		if ((keyPad->grp.area.y + KEYPAD_SIZE_H) >= root->area.height) {
			keyPad->grp.area.y = keyPad->grp.area.y - KEYPAD_SIZE_H - field->grp.area.height;
		}
		lcd_segmentNewKeyPad(root, keyPad);
		activePopup = (struct lcd_segmentGroup *)keyPad;
	}
	return;
}

struct lcd_segment *lcd_segmentNewTextField(struct lcd_segmentGroup *g, struct lcd_segmentTextField *field) {
	struct lcd_segment	*my;

	my = lcd_segmentNewGroup(g, (struct lcd_segmentGroup *)field);		// Appropriation
	if (my) {
		field->grp.touchOutProg = lcd_segmentTouchTextField;

		// Embossment
		field->internal.frame[0].x       = field->grp.area.width-1;		field->internal.frame[0].y = field->grp.area.height-1;
		field->internal.frame[1].x       = 0;							field->internal.frame[1].y = field->grp.area.height-1;
		field->internal.frame[2].x       = 0;							field->internal.frame[2].y = 0;
		field->internal.edge.color       = LCD_COLOR_LIGHTGRAY;
		field->internal.edge.lineSize    = 1;
		field->internal.edge.n           = 3;
		field->internal.edge.point       = field->internal.frame;

		// Text string
		field->internal.text.area.x      = 1;
		field->internal.text.area.y      = 0;
		field->internal.text.area.width  = field->grp.area.width - 1;
		field->internal.text.area.height = field->grp.area.height - 1;
		field->internal.text.color.face  = field->color.face;
		field->internal.text.color.back  = field->color.back;
		field->internal.text.str         = field->str;
		field->internal.text.attr        = field->attr;

		lcd_segmentNewPoly((struct lcd_segmentFigure *)field, &(field->internal.edge));
		lcd_segmentNewText((struct lcd_segmentFigure *)field, &(field->internal.text));
	}
	return(my);
}

//
//	Integer field
//
void lcd_segmentTouchIntegerPad(struct lcd_segmentGroup *g){
	struct lcd_segmentTenKeyPad		*keyPad;
	struct lcd_segmentButton		*button;
	char							*caption;
	struct lcd_segmentIntegerField	*field;

	button  = (struct lcd_segmentButton *)g;
	caption = button->caption;
	keyPad  = (struct lcd_segmentTenKeyPad *)lcd_segmentGetGroupParent(g);
	field   = (struct lcd_segmentIntegerField*)keyPad->targetField;
	if (!strcmp(caption, "=")) {
		if (!lcd_segmentSetIntegerFieldValue(field, atoi(field->internal.buf))) {
			lcd_segmentDelete(keyPad);
			activePopup = NULL;
			if (field->actionProg) {
				field->actionProg(field);
			}
		}
	} else if (!strcmp(caption, "Esc")){
		lcd_segmentDelete(keyPad);
		activePopup = NULL;
		sprintf(field->internal.buf, field->format, field->value);
		lcd_segmentSetDirty(field);
	} else if (!strcmp(caption, "C")){
		field->internal.buf[0] = '\0';
		lcd_segmentSetDirty(field);
	} else if ((caption[0] != '.') && (caption[0] != '+')){
		if (strlen(field->internal.buf) < (sizeof(field->internal.buf)-2)) {
			strcat(field->internal.buf, caption);
			lcd_segmentSetDirty(field);
		}
	}
	return;
}

void lcd_segmentTouchIntegerField(struct lcd_segmentGroup *g){
	struct lcd_segmentTenKeyPad		*keyPad;
	struct lcd_segmentIntegerField	*field;
	struct lcd_segmentGroup			*root;
	struct lcd_segmentGroup			*t;

	if (activePopup) {
		lcd_segmentDelete(activePopup);
	}
	keyPad = lcd_segmentAllocTenKeyPad();
	if (keyPad != NULL) {
		field                    = (struct lcd_segmentIntegerField *)g;
		keyPad->targetField      = field;
		keyPad->grp.touchOutProg = lcd_segmentTouchIntegerPad;
		keyPad->color.face   = LCD_COLOR_WHITE;
		keyPad->color.back   = LCD_COLOR_GRAY;
		keyPad->grp.area.x       = 0;
		keyPad->grp.area.y       = field->grp.area.height;
		keyPad->grp.area.width   = TENKEYPAD_SIZE_W;
		keyPad->grp.area.height  = TENKEYPAD_SIZE_H;
		root = g;
		for(t = g; t != NULL; t = lcd_segmentGetGroupParent(t)) {	// Search the root
			keyPad->grp.area.x += t->area.x;		// Calculate the field coordinate on root window
			keyPad->grp.area.y += t->area.y;
			root = t;
		}
		if ((keyPad->grp.area.x + TENKEYPAD_SIZE_W) >= root->area.width) {
			keyPad->grp.area.x = root->area.width - TENKEYPAD_SIZE_W;
		}
		if ((keyPad->grp.area.y + TENKEYPAD_SIZE_H) >= root->area.height) {
			keyPad->grp.area.y = keyPad->grp.area.y - TENKEYPAD_SIZE_H - field->grp.area.height;
		}
		lcd_segmentNewTenKeyPad(root, keyPad);
		activePopup = (struct lcd_segmentGroup *)keyPad;
		sprintf(field->internal.buf, "%d", field->value);
	}
	return;

}

struct lcd_segment *lcd_segmentNewIntegerField(struct lcd_segmentGroup *g, struct lcd_segmentIntegerField *field) {
	struct lcd_segment	*my;

	my = lcd_segmentNewGroup(g, (struct lcd_segmentGroup *)field);		// Appropriation
	if (my) {
		field->grp.touchOutProg = lcd_segmentTouchIntegerField;

		// Embossment
		field->internal.frame[0].x       = field->grp.area.width-1;		field->internal.frame[0].y = field->grp.area.height-1;
		field->internal.frame[1].x       = 0;							field->internal.frame[1].y = field->grp.area.height-1;
		field->internal.frame[2].x       = 0;							field->internal.frame[2].y = 0;
		field->internal.edge.color       = LCD_COLOR_LIGHTGRAY;
		field->internal.edge.lineSize    = 1;
		field->internal.edge.n           = 3;
		field->internal.edge.point       = &(field->internal.frame[0]);

		// Text string
		field->internal.text.area.x      = 1;
		field->internal.text.area.y      = 0;
		field->internal.text.area.width  = field->grp.area.width - 1;
		field->internal.text.area.height = field->grp.area.height - 1;
		field->internal.text.color.face  = field->color.face;
		field->internal.text.color.back  = field->color.back;
		field->internal.text.str         = field->internal.buf;
		field->internal.text.attr.varticalAlign = LCD_SEGMENT_TEXT_ALIGN_CENTER;
		sprintf(field->internal.buf, field->format, field->value);

		lcd_segmentNewPoly((struct lcd_segmentFigure *)field, &(field->internal.edge));
		lcd_segmentNewText((struct lcd_segmentFigure *)field, &(field->internal.text));
	}
	return(my);
}

int lcd_segmentSetIntegerFieldValue(struct lcd_segmentIntegerField *field, int i) {
	if ((i >= field->min) && (i <= field->max)) {
		field->value = i;
		sprintf(field->internal.buf, field->format, i);
		lcd_segmentSetDirty(field);
		return(0);
	} else {
		return(1);
	}
}

//
//	Float field
//
void lcd_segmentTouchFloatPad(struct lcd_segmentGroup *g){
	struct lcd_segmentTenKeyPad		*keyPad;
	struct lcd_segmentButton		*button;
	char							*caption;
	struct lcd_segmentFloatField	*field;

	button  = (struct lcd_segmentButton *)g;
	caption = button->caption;
	keyPad  = (struct lcd_segmentTenKeyPad *)lcd_segmentGetGroupParent(g);
	field   = (struct lcd_segmentFloatField*)keyPad->targetField;
	if (!strcmp(caption, "=")) {
		if (!lcd_segmentSetFloatFieldValue(field, atof(field->internal.buf))) {
			lcd_segmentDelete(keyPad);
			activePopup = NULL;
			if (field->actionProg) {
				field->actionProg(field);
			}
		}
	} else if (!strcmp(caption, "Esc")){
		lcd_segmentDelete(keyPad);
		activePopup = NULL;
		sprintf(field->internal.buf, field->format, field->value);
		lcd_segmentSetDirty(field);
	} else if (!strcmp(caption, "C")){
		field->internal.buf[0] = '\0';
		lcd_segmentSetDirty(field);
	} else if (caption[0] != '+'){
		if (strlen(field->internal.buf) < (sizeof(field->internal.buf)-2)) {
			strcat(field->internal.buf, caption);
			lcd_segmentSetDirty(field);
		}
	}
	return;
}

void lcd_segmentTouchFloatField(struct lcd_segmentGroup *g){
	struct lcd_segmentTenKeyPad		*keyPad;
	struct lcd_segmentFloatField	*field;
	struct lcd_segmentGroup			*root;
	struct lcd_segmentGroup			*t;

	if (activePopup) {
		lcd_segmentDelete(activePopup);
	}
	keyPad  = lcd_segmentAllocTenKeyPad();
	if (keyPad != NULL) {
		field                    = (struct lcd_segmentFloatField *)g;
		keyPad->targetField      = field;
		keyPad->grp.touchOutProg = lcd_segmentTouchFloatPad;
		keyPad->color.face   = LCD_COLOR_WHITE;
		keyPad->color.back   = LCD_COLOR_GRAY;
		keyPad->grp.area.x       = 0;
		keyPad->grp.area.y       = field->grp.area.height;
		keyPad->grp.area.width   = TENKEYPAD_SIZE_W;
		keyPad->grp.area.height  = TENKEYPAD_SIZE_H;
		root = g;
		for(t = g; t != NULL; t = lcd_segmentGetGroupParent(t)) {	// Search the root
			keyPad->grp.area.x += t->area.x;		// Calculate the field coordinate on root window
			keyPad->grp.area.y += t->area.y;
			root = t;
		}
		if ((keyPad->grp.area.x + TENKEYPAD_SIZE_W) >= root->area.width) {
			keyPad->grp.area.x = root->area.width - TENKEYPAD_SIZE_W;
		}
		if ((keyPad->grp.area.y + TENKEYPAD_SIZE_H) >= root->area.height) {
			keyPad->grp.area.y = keyPad->grp.area.y - TENKEYPAD_SIZE_H - field->grp.area.height;
		}
		lcd_segmentNewTenKeyPad(root, keyPad);
		activePopup = (struct lcd_segmentGroup *)keyPad;
		sprintf(field->internal.buf, "%f", field->value);
	}
	return;

}

struct lcd_segment *lcd_segmentNewFloatField(struct lcd_segmentGroup *g, struct lcd_segmentFloatField *field) {
	struct lcd_segment	*my;

	my = lcd_segmentNewGroup(g, (struct lcd_segmentGroup *)field);		// Appropriation
	if (my) {
		field->grp.touchOutProg = lcd_segmentTouchFloatField;

		// Embossment
		field->internal.frame[0].x       = field->grp.area.width-1;		field->internal.frame[0].y = field->grp.area.height-1;
		field->internal.frame[1].x       = 0;							field->internal.frame[1].y = field->grp.area.height-1;
		field->internal.frame[2].x       = 0;							field->internal.frame[2].y = 0;
		field->internal.edge.color       = LCD_COLOR_LIGHTGRAY;
		field->internal.edge.lineSize    = 1;
		field->internal.edge.n           = 3;
		field->internal.edge.point       = &(field->internal.frame[0]);

		// Text string
		field->internal.text.area.x      = 1;
		field->internal.text.area.y      = 0;
		field->internal.text.area.width  = field->grp.area.width - 1;
		field->internal.text.area.height = field->grp.area.height - 1;
		field->internal.text.color.face  = field->color.face;
		field->internal.text.color.back  = field->color.back;
		field->internal.text.str         = field->internal.buf;
		field->internal.text.attr.varticalAlign = LCD_SEGMENT_TEXT_ALIGN_CENTER;
		sprintf(field->internal.buf, field->format, field->value);

		lcd_segmentNewPoly((struct lcd_segmentFigure *)field, &(field->internal.edge));
		lcd_segmentNewText((struct lcd_segmentFigure *)field, &(field->internal.text));
	}
	return(my);
}

int lcd_segmentSetFloatFieldValue(struct lcd_segmentFloatField *field, float f) {
	if ((f >= field->min) && (f <= field->max)) {
		field->value = f;
		sprintf(field->internal.buf, field->format, f);
		lcd_segmentSetDirty(field);
		return(0);
	} else {
		return(1);
	}
}


//
//	Pull down field
//
void lcd_segmentTouchPullDownList(struct lcd_segmentGroup *g){
	struct lcd_segmentSelectionList	*list;
	struct lcd_segmentPullDownField	*field;
	int								i;

	list  = (struct lcd_segmentSelectionList *)lcd_segmentGetGroupParent(g);
	field = (struct lcd_segmentPullDownField *)list->targetField;
	for(i = 0; i < list->number; i++) {
		if (&(list->internal.label[i]) == (struct lcd_segmentButton *)g) {
			break;
		}
	}
	if (lcd_segmentGetPullDownFieldSelect(field) != i) {
		lcd_segmentSetPullDownFieldSelect(field, i);
		if (field->actionProg) {
			field->actionProg(field);
		}
	}
	lcd_segmentDelete(list);
	activePopup = NULL;
	return;
}

void lcd_segmentTouchPullDownField(struct lcd_segmentGroup *g){
	struct lcd_segmentPullDownField	*field;
	struct lcd_segmentSelectionList	*list;
	struct lcd_segmentGroup			*root;
	struct lcd_segmentGroup			*t;

	if (activePopup) {
		lcd_segmentDelete(activePopup);
	}
	field = (struct lcd_segmentPullDownField *)g;
	list  = lcd_segmentAllocSelectionList(field->number);
	if (list != NULL) {
		list->number           = field->number;
		list->label            = field->label;
		list->targetField      = field;
		list->grp.touchOutProg = lcd_segmentTouchPullDownList;
		list->color.face   = LCD_COLOR_WHITE;
		list->color.back   = LCD_COLOR_GRAY;
		list->grp.area.x       = field->grp.area.width;
		list->grp.area.y       = 0;
		{
			int		max;
			int		i;
			int		w;

			max = 0;
			for(i = 0; i < field->number; i++) {
				max = MAX(max, strlen(field->label[i]));
			}
			w = max * LCD_FONT_WIDTH;
			w = MIN(w, PULLDOWN_SIZE_W_MAX);
			w = MAX(w, PULLDOWN_SIZE_W_MIN);
			list->grp.area.width = w;
		}
		list->grp.area.height  = PULLDOWN_SIZE_H * field->number;
		root = g;
		for(t = g; t != NULL; t = lcd_segmentGetGroupParent(t)) {	// Search the root
			list->grp.area.x += t->area.x;		// Calculate the field coordinate on root window
			list->grp.area.y += t->area.y;
			root = t;
		}
		if ((list->grp.area.x + list->grp.area.width) >= root->area.width) {
			list->grp.area.x -= field->grp.area.width;
			list->grp.area.x -= list->grp.area.width;
		}
		if ((list->grp.area.y + PULLDOWN_SIZE_H) >= root->area.height) {
			list->grp.area.y = root->area.height - PULLDOWN_SIZE_H;
		}
		lcd_segmentNewSelectionList(root, list);
		activePopup = (struct lcd_segmentGroup *)list;
	}
	return;
}

struct lcd_segment *lcd_segmentNewPullDownField(struct lcd_segmentGroup *g, struct lcd_segmentPullDownField *field) {
	struct lcd_segment		 		*my;

	my = lcd_segmentNewGroup(g, (struct lcd_segmentGroup *)field);		// Appropriation
	if (my) {
		field->grp.touchOutProg = lcd_segmentTouchPullDownField;

		// Embossment
		field->internal.frame[0].x       = field->grp.area.width-1;		field->internal.frame[0].y = field->grp.area.height-1;
		field->internal.frame[1].x       = 0;							field->internal.frame[1].y = field->grp.area.height-1;
		field->internal.frame[2].x       = 0;							field->internal.frame[2].y = 0;
		field->internal.edge.color       = LCD_COLOR_LIGHTGRAY;
		field->internal.edge.lineSize    = 1;
		field->internal.edge.n           = 3;
		field->internal.edge.point       = &(field->internal.frame[0]);

		// Text string
		field->internal.text.area.x      = 1;
		field->internal.text.area.y      = 0;
		field->internal.text.area.width  = field->grp.area.width - 1;
		field->internal.text.area.height = field->grp.area.height - 1;
		field->internal.text.color.face  = field->color.face;
		field->internal.text.color.back  = field->color.back;
		field->internal.text.str         = field->label[field->select];
		field->internal.text.attr.horizontalAlign = LCD_SEGMENT_TEXT_ALIGN_CENTER;
		field->internal.text.attr.varticalAlign   = LCD_SEGMENT_TEXT_ALIGN_CENTER;

		lcd_segmentNewPoly((struct lcd_segmentFigure *)field, &(field->internal.edge));
		lcd_segmentNewText((struct lcd_segmentFigure *)field, &(field->internal.text));
	}
	return(my);
}

void lcd_segmentSetPullDownFieldSelect(struct lcd_segmentPullDownField *attr, int select){
	attr->select = select;
	attr->internal.text.str = attr->label[select];
	lcd_segmentSetDirty(attr);
	return;
}


