//
//  usbh_wcc_bluetooth.c
//
//  USB Host Protocol Stack - USB Wireless controller class for bluetooth dongle
//  Copyright (C) 2013 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Reference:
//		Bluetooth SIG (www.bluetooth.com)
//			BLUETOOTH SPECIFICATION Version 4.0
//				[Vol 4] Host Controller Interface [Transport Layer]
//				Part B: USB Transport Layer
//		STMicroelectronics
//			STM32 USB HOST Library V2.1.0
//
//	Device:
//		STM32Fxx
//
//	Update history
//	---------- ----- -------------------------------------------
//	2013.01.21 v0.0  First cording
//	2013.04.21 v0.1  Commit
//	2013.12.17 v0.1a EP_TYPE_INTR polling is certain to 1ms interval
//	                 USBH_WCC_Bluetooth_Handle_evt(), Changed the caller this code to IRQ
//	                 Need modify the interrupt handler(stm32f4xx_it.c)
//
#include <string.h>
#include "usbh_wcc_bluetooth.h"
#include "usbh_core.h"
#include "usbh_stdreq.h"
#include "usbh_hcs.h"
#include "usbh_ioreq.h"
#include "usbh_conf.h"					// ADD 2013.12.17

#include "Bluetooth/bt_hci.h"

//#define MIN(a,b)	((a) < (b) ? (a) : (b))
//#define DEBUG

#ifdef DEBUG
#include <stdio.h>
#define DEBUG_DUMP(s,b,l) { int i; printf(" %s",s); for(i=0; i<l; i++) printf("%02x", (b)[i]); printf("\n"); }
#else
#define DEBUG_DUMP(s,b,l)
#endif


#define USB_WCC_BT_TRYCOUNT		200		// Intrrupt recieve request -> recieve loop count limit
#define USB_WCC_BT_INTR_POLL	1		// Minimum poling wait time (USB frame count)

#define BT_PIPE_NAME_CMD		"c"
#define BT_PIPE_NAME_EVT		"e"
#define BT_PIPE_NAME_OUT		"o"
#define BT_PIPE_NAME_IN			"i"

struct USBH_WCC_Bluetooth_CB	USBH_WCC_Bluetooth_CB;		// Entity

#define CB	USBH_WCC_Bluetooth_CB		// Short name


//------------------------------------------------------------------------------
//	USB pluged in
//------------------------------------------------------------------------------
USBH_Status USBH_WCC_Bluetooth_InterfaceInit(USB_OTG_CORE_HANDLE *pdev, void *phost){
	USBH_HOST *pphost = phost;
  
	if ((pphost->device_prop.Itf_Desc[0].bInterfaceClass    == USB_WCC_CLASS) &&
		(pphost->device_prop.Itf_Desc[0].bInterfaceSubClass == USB_WCC_CLASS_RF) &&
		(pphost->device_prop.Itf_Desc[0].bInterfaceProtocol == USB_PROTOCOL_BLUETOOTH)){
		unsigned char	m;
		unsigned char	i;

		// Addition USB path
		CB.intrEp = 0;
		CB.inEp   = 0;
		CB.outEp  = 0;
		m = MIN(pphost->device_prop.Itf_Desc[0].bNumEndpoints, USBH_MAX_NUM_ENDPOINTS);

		for (i = 0; i < m; i++) {
			uint8_t		a;
			uint16_t	p;
			uint8_t		h;									// ADD 2013.12.17

			a = pphost->device_prop.Ep_Desc[0][i].bEndpointAddress;
			p = pphost->device_prop.Ep_Desc[0][i].wMaxPacketSize;
			h = USBH_Alloc_Channel(pdev, a);				// ADD 2013.12.17
			switch(a) {
			case 0x81:					// HCI Events pipe
				USBH_Open_Channel(pdev,						// ADD 2013.12.17
						h,									// ADD 2013.12.17
						pphost->device_prop.address,		// ADD 2013.12.17
						pphost->device_prop.speed,			// ADD 2013.12.17
						EP_TYPE_INTR,						// ADD 2013.12.17
						p);									// ADD 2013.12.17
				CB.intrEp     = a;
				CB.intrEpSize = p;
				CB.intrHcNum  = h;							// ADD 2013.12.17
				break;
			case 0x82:					// ACL Data IN pipe
				USBH_Open_Channel(pdev,						// ADD 2013.12.17
						h,									// ADD 2013.12.17
						pphost->device_prop.address,		// ADD 2013.12.17
						pphost->device_prop.speed,			// ADD 2013.12.17
						EP_TYPE_BULK,						// ADD 2013.12.17
						p);									// ADD 2013.12.17
				CB.inEp       = a;
				CB.inEpSize   = p;
				CB.inHcNum    = h;							// ADD 2013.12.17
				break;
			case 0x02:					// ACL Data OUT pipe
				USBH_Open_Channel(pdev,						// ADD 2013.12.17
						h,									// ADD 2013.12.17
						pphost->device_prop.address,		// ADD 2013.12.17
						pphost->device_prop.speed,			// ADD 2013.12.17
						EP_TYPE_BULK,						// ADD 2013.12.17
						p);									// ADD 2013.12.17
				CB.outEp      = a;
				CB.outEpSize  = p;
				CB.outHcNum   = h;							// ADD 2013.12.17
				break;
			default:
				break;
			}
		}
#ifdef DEL													// DEL START 2013.12.17
		if ((CB.intrEp != 0) && (CB.inEp != 0) && (CB.outEp != 0)) {
			CB.intrHcNum = USBH_Alloc_Channel(pdev, CB.intrEp);
			CB.inHcNum   = USBH_Alloc_Channel(pdev, CB.inEp);
			CB.outHcNum  = USBH_Alloc_Channel(pdev, CB.outEp);
			USBH_Open_Channel(pdev,
					CB.intrHcNum,
					pphost->device_prop.address,
					pphost->device_prop.speed,
					EP_TYPE_INTR,
					CB.intrEpSize);
			USBH_Open_Channel(pdev,
					CB.inHcNum,
					pphost->device_prop.address,
					pphost->device_prop.speed,
					EP_TYPE_BULK,
					CB.inEpSize);
			USBH_Open_Channel(pdev,
					CB.outHcNum,
					pphost->device_prop.address,
					pphost->device_prop.speed,
					EP_TYPE_BULK,
					CB.outEpSize);

			// To L2CAP communication path
			bt_setAliveHCI();
			DEBUG_DUMP("USBH_WCC_Bluetooth_InterfaceInit OK", "", 0);
		} else {
			pphost->usr_cb->DeviceNotSupported();
		}
#endif	// DEL												// DEL END 2013.12.17

		// To L2CAP communication path						// ADD 2013.12.17
		bt_setAliveHCI();									// ADD 2013.12.17
		DEBUG_DUMP("USBH_WCC_Bluetooth_InterfaceInit","",0);// ADD 2013.12.17
	} else {
		pphost->usr_cb->DeviceNotSupported();
	}

	return(USBH_OK);
}


//------------------------------------------------------------------------------
//	USB unpluged
//------------------------------------------------------------------------------
void USBH_WCC_Bluetooth_InterfaceDeInit(USB_OTG_CORE_HANDLE *pdev, void *phost) {
	if (CB.intrHcNum) {
		USB_OTG_HC_Halt(pdev, CB.intrHcNum);
		USBH_Free_Channel(pdev, CB.intrHcNum);
	}
	if (CB.inHcNum) {
		USB_OTG_HC_Halt(pdev, CB.inHcNum);
		USBH_Free_Channel(pdev, CB.inHcNum);
	}
	if (CB.outHcNum) {
		USB_OTG_HC_Halt(pdev, CB.outHcNum);
		USBH_Free_Channel(pdev, CB.outHcNum);
	}

	// To L2CAP communication path
	bt_setDeadHCI();
//	CB.outHcNum  = 0;     /* Free mark */
//	CB.inHcNum   = 0;     /* Free mark */
//	CB.intrHcNum = 0;     /* Free mark */
	memset(&CB, 0x00, sizeof(CB));
	DEBUG_DUMP("USBH_WCC_Bluetooth_InterfaceDeInit OK", "", 0);
	return;
}


//------------------------------------------------------------------------------
//	Handler
//------------------------------------------------------------------------------
USBH_Status USBH_WCC_Bluetooth_ClassRequest(USB_OTG_CORE_HANDLE *pdev, void *phost) {
	return(USBH_OK);
}

//------------------------------------------------------------------------------
//	Handlers
//------------------------------------------------------------------------------
void USBH_WCC_Bluetooth_Handle_cmd(USB_OTG_CORE_HANDLE *pdev, USBH_HOST *pphost) {
	USBH_Status status;
	int			rc;

	switch(CB.cmdState){
	case IDLE:
	case COMMAND_START:
		rc = bt_getHCIcommand(CB.cmdBuf, sizeof(CB.cmdBuf));
		if (rc > 0) {
			CB.cmdBufLen = rc;
			CB.cmdState  = COMMAND_POST;
		} else {
			CB.cmdState  = COMMAND_END;
		}
		break;
	case COMMAND_POST:
//		pphost->Control.setup.b.bmRequestType = USB_H2D | USB_REQ_TYPE_CLASS | USB_REQ_RECIPIENT_INTERFACE;	// composite device
		pphost->Control.setup.b.bmRequestType = USB_H2D | USB_REQ_TYPE_CLASS | USB_REQ_RECIPIENT_DEVICE;
		pphost->Control.setup.b.bRequest      = 0;
		pphost->Control.setup.b.wValue.w      = 0;
		pphost->Control.setup.b.wIndex.w      = 0;		// Descriptor interface#
		pphost->Control.setup.b.wLength.w     = CB.cmdBufLen;
		status = USBH_CtlReq(pdev, pphost, CB.cmdBuf, CB.cmdBufLen);
		switch(status){
		case USBH_OK:
			CB.cmdState = COMMAND_END;
			DEBUG_DUMP("CtrlSend:", CB.cmdBuf, CB.cmdBufLen);
			break;
		case USBH_BUSY:
			break;
		default:
			CB.cmdState = COMMAND_END;
			DEBUG_DUMP("CtrlSend: error:", CB.cmdBuf, CB.cmdBufLen);
			break;
		}
		break;
	case COMMAND_END:
	default:
		CB.cmdState = IDLE;
		break;
	}
	return;
}

void USBH_WCC_Bluetooth_Handle_evt(USB_OTG_CORE_HANDLE *pdev, USBH_HOST *pphost) {
	URB_STATE 	status;
	int			frames;

	switch(CB.evtState){
	case IDLE:
	case EVENT_START:
		if (CB.intrHcNum != 0) {
			CB.evtState = EVENT_POLL;
		}
	    break;
	case EVENT_POLL:
		USBH_InterruptReceiveData(pdev, CB.evtBuf, MIN(CB.intrEpSize, sizeof(CB.evtBuf)), CB.intrHcNum);
		CB.frame       = HCD_GetCurrentFrame(pdev);
		CB.evtState    = EVENT_POLL_WAIT;
		break;
	case EVENT_POLL_WAIT:
		status = HCD_GetURB_State(pdev, CB.intrHcNum);
		frames = HCD_GetCurrentFrame(pdev) - (int)CB.frame;
		if(frames < 0) {
			frames += 16384;
		}
		switch(status){
		case URB_DONE:
			CB.evtBufLen = HCD_GetXferCnt(pdev, CB.intrHcNum);
			CB.evtState  = EVENT_POST_WAIT;
#ifdef DEBUG
			{
				char	buf[32];

				sprintf(buf, "IntrRecv: f=%d ", frames);
				DEBUG_DUMP(buf, CB.evtBuf, CB.evtBufLen);
			}
#endif
			break;
		case URB_STALL:
			DEBUG_DUMP("IntrRecv: URB_STALL", "", 0);
			if ((USBH_ClrFeature(pdev, pphost, CB.intrEp, CB.intrHcNum)) == USBH_OK) {
				CB.evtState = EVENT_POLL;
			}
			break;
		case URB_IDLE:
			if (frames > USB_WCC_BT_INTR_POLL) {
				CB.evtState = EVENT_POLL;
			}
			break;
		default:
			DEBUG_DUMP("IntrRecv: error=0x", (char *)&status, sizeof(status));
			CB.evtState = EVENT_POLL;
			break;
		}
		break;
	case EVENT_GET:
	case EVENT_POST_WAIT:
		if (bt_putHCIevent(CB.evtBuf, CB.evtBufLen) == CB.evtBufLen) {
			CB.evtState = EVENT_POLL;
		}
		break;
	case EVENT_END:
	default:
		CB.evtState = IDLE;
		break;
	}
	return;
}

void USBH_WCC_Bluetooth_Handle_out(USB_OTG_CORE_HANDLE *pdev, USBH_HOST *pphost) {
	USBH_Status status;
	int			rc;

	switch(CB.outState){
	case IDLE:
	case ACL_OUT_START:
		rc = bt_getACLout(CB.outBuf, sizeof(CB.outBuf));
		if (rc > 0) {
			CB.outBufLen = rc;
			CB.outState  = ACL_OUT_POST;
		} else {
			CB.outState  = ACL_OUT_END;
		}
		break;
	case ACL_OUT_POST:
		USBH_BulkSendData(pdev, CB.outBuf, CB.outBufLen, CB.outHcNum);
		CB.outState    = ACL_OUT_WAIT;
		CB.outTryCount = 0;
		break;
	case ACL_OUT_WAIT:
		status = HCD_GetURB_State(pdev, CB.outHcNum);
		switch(status){
		case URB_DONE:
			CB.outState = ACL_OUT_END;
			DEBUG_DUMP("BulkSend:", CB.outBuf, CB.outBufLen);
			break;
		case URB_IDLE:
			if (CB.outTryCount++ > USB_WCC_BT_TRYCOUNT) {
				DEBUG_DUMP("BulkSend: time out:", CB.outBuf, CB.outBufLen);
				CB.outState = ACL_OUT_END;
			}
			break;
		case URB_STALL:
			/* Issue Clear Feature on interrupt IN endpoint */
			if ((USBH_ClrFeature(pdev, pphost, CB.outEp, CB.outHcNum)) == USBH_OK) {
				/* Change state to issue next IN token */
				CB.outState = ACL_OUT_END;
			}
			break;
		default:
			CB.outState = ACL_OUT_END;
			DEBUG_DUMP("BulkSend: error:", CB.outBuf, CB.outBufLen);
			break;
		}
		break;
	case ACL_OUT_END:
	default:
		CB.outState = IDLE;
		break;
	}
	return;
}

void USBH_WCC_Bluetooth_Handle_in(USB_OTG_CORE_HANDLE *pdev, USBH_HOST *pphost) {
	USBH_Status status;

	switch(CB.inState){
	case IDLE:
	case ACL_IN_START:
	case ACL_IN_POLL:
		USBH_BulkReceiveData(pdev, CB.inBuf, sizeof(CB.inBuf), CB.inHcNum);
		CB.inTryCount = 0;
		CB.inState    = ACL_IN_POLL_WAIT;
		break;
	case ACL_IN_POLL_WAIT:		// Wait at next frame boundery (true communication start timing)
	case ACL_IN_GET:
		status = HCD_GetURB_State(pdev, CB.inHcNum);
		switch(status){
		case URB_DONE:
//			CB.inBufLen = pdev->host.hc[CB.inHcNum].xfer_count;	// DEL 2013.12.17
			CB.inBufLen = HCD_GetXferCnt(pdev, CB.inHcNum);		// ADD 2013.12.17
			CB.inState  = ACL_IN_POST_WAIT;
			DEBUG_DUMP("BulkRecv:", CB.inBuf, CB.inBufLen);
			break;
		case URB_IDLE:
			if (CB.inTryCount++ > USB_WCC_BT_TRYCOUNT) {
				CB.inState = ACL_IN_POLL;
			}
			break;
		case URB_STALL:
			/* Issue Clear Feature on interrupt IN endpoint */
			if ((USBH_ClrFeature(pdev, pphost, CB.intrEp, CB.intrHcNum)) == USBH_OK) {
				/* Change state to issue next IN token */
				CB.inState = ACL_IN_END;
			}
			break;
		default:
			CB.inState = ACL_IN_END;
#ifdef DEBUG
			{
				char	buf[32];

				sprintf(buf, "BulkRecv: error=%d", status);
				DEBUG_DUMP(buf, CB.inBuf, CB.inBufLen);
			}
#endif
			break;
		}
		break;
	case ACL_IN_POST_WAIT:
		if (bt_putHCIin(CB.inBuf, CB.inBufLen) == CB.inBufLen) {
			CB.inState = ACL_IN_END;
		}
		break;
	case ACL_IN_END:

	default:
		CB.inState = IDLE;
		break;
	}
	return;
}

USBH_Status USBH_WCC_Bluetooth_Handle(USB_OTG_CORE_HANDLE *pdev, void *phost) {
	USBH_HOST	*pphost;
	USBH_Status	status;
	static int	r = 0;

	pphost = phost;
	status = USBH_OK;
	switch(r){
	case 0:
		USBH_WCC_Bluetooth_Handle_cmd(pdev, pphost);
		break;
	case 1:
//		USBH_WCC_Bluetooth_Handle_evt(pdev, pphost);	// DEL 2013.12.17  Move to systick interrupt(100us interval)
		break;
	case 2:
		USBH_WCC_Bluetooth_Handle_out(pdev, pphost);
		break;
	default:
		USBH_WCC_Bluetooth_Handle_in(pdev, pphost);
		break;
	}
	r++;
	r &= 0x03;
	return(status);
}

// Callbacks jump table
USBH_Class_cb_TypeDef  USBH_WCC_Bluetooth_cb ={
  USBH_WCC_Bluetooth_InterfaceInit,
  USBH_WCC_Bluetooth_InterfaceDeInit,
  USBH_WCC_Bluetooth_ClassRequest,
  USBH_WCC_Bluetooth_Handle
};

//------------------------------------------------------------------------------
