//
//  disp.c	WHM0.8 display process
//  Copyright(C)2014 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2014.01.10 v0.0  First cording
//    2014.01.18 v0.8c An improvement of display performance
//    2014.01.31 v0.8d Modification of save unit(day->hour)
//    2014.02.06 v0.8d The USB memory was supported
//
//

#include <stdlib.h>						// rand()
#include <string.h>
#include <stdio.h>
#include <math.h>						// libm log10
#include <time.h>
#include <sys/time.h>					// system call

#include "s95160.h"						// LCD module support
#include "bt_l2cap_cb_link.h"			// Bluetooth link information

#include "config.h"						// WHM0.8 configuration
#include "sensor_data.h"				// WHM0.8 Control block


// Inline function
#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define ABS(a)		(((a) >  0 ) ? (a) : -(a))

// main
extern void saveSetup(struct sensorData *d, int ch, struct tm *tm);
extern void adcToValuePrepare(struct sensorData sensorData[]);
extern char	*usbDeviceClassName;
extern int	usbDeviceReady;
extern int	usbMemoryLogInterval;
extern int	usbMemorySaveRequest;
extern int	usbMemoryStatus;



//
//	Common area
//
enum dispMode_t {
	DISP_NOW_DIGEST = 0,
	DISP_NOW_DETAIL,
	DISP_TODAY_GRAPH,
	DISP_TODAY_DETAIL,
	DISP_MONTH_GRAPH,
	DISP_MONTH_DETAIL,
	DISP_YEAR_GRAPH,
	DISP_YEAR_DETAIL,
	DISP_SETUP_CHANNEL,
	DISP_SETUP_TIME,
	DISP_OFF
}	dispMode;						// Screen select
struct sensorData	*dispData;		// Interface data
int					dispCh;			// Interface data
struct tm			*dispTm;		// Interface data
#define DISP_OVER_CURRENT	7		// 10A,15A,20A,30A,40A,50A,60A
struct {
	long	check;
	long	sec;
}	dispOverCurrent[DISP_OVER_CURRENT] = {
		{10000, 0},
		{15000, 0},
		{20000, 0},
		{30000, 0},
		{40000, 0},
		{50000, 0},
		{60000, 0},
};


//
//	Custom display method for s95160_s1 libs
//
struct waveFigure {					//  Figure member
	struct lcd_segmentFigure		fig;			// Essential item
	struct lcd_segmentArea			area;
	struct lcd_segmentColors		color;
	int								num;			// width dot count
	int								interval;		// CH_COUNT
	int								offset;			// target channel
	short							*val;
};
int dispWaveFigure(struct lcd_segmentFigure *fig, struct lcd_segmentCoordinate *cardinalPoint){
	struct waveFigure *p;
	int		i;
	int		m;
	int		s;
	int		x1, y1;
	int		x2, y2;
	short	h;
	short	*val;

	p = (struct waveFigure *)fig;
	h = p->area.height;
	if (p->color.back != 0) {
		lcd_drawRect(p->area.x + cardinalPoint->x, p->area.y + cardinalPoint->y, p->area.width, p->area.height, p->color.back);
	}
	p->num = MIN(p->num, p->area.width);
	m = 0;
	val = p->val + p->offset;
	for(i = 0; i < p->num; i++) {
		m = MAX(m, *val);
		val += p->interval;
	}
	h /= 2;
	s = (m + (h - 1)) / h;

	val = p->val + p->offset;
	for(i = 0; i < p->num; i++) {
		x1 = i + p->area.x + cardinalPoint->x;
		y1 = (*val / s) + p->area.y + cardinalPoint->y + h;
		if (i == 0) {
			x2 = x1;
			y2 = y1;
		}
		lcd_drawLine(x1, y1, x2, y2, 1, p->color.face);
		x2 = x1;
		y2 = y1;
		val += p->interval;
	}
	return(1);
};
#define newWaveFigure(brother, wave)		lcd_segmentNewFigure((struct lcd_segmentFigure *)(brother), (struct lcd_segmentFigure *)(wave), sizeof(struct waveFigure), dispWaveFigure)


// Screen layout
//
// +----------------+
// |      HEAD      |
// +---+------------+
// |   |            |
// |   |            |
// | L |            |
// | E |    BODY    |
// | F |            |
// | T |            |
// |   |            |
// |   |            |
// +---+------------+
// |      FOOT      |
// +----------------+
//
#define HEAD_H		20
#define HEAD_W		LCD_SCREEN_WIDTH
#define FOOT_H		20
#define FOOT_W		LCD_SCREEN_WIDTH
#define LEFT_H		(LCD_SCREEN_HEIGHT - HEAD_H - FOOT_H)
#define LEFT_W		24
#define BODY_H		LEFT_H
#define BODY_W		(LCD_SCREEN_WIDTH - LEFT_W)
#define BODY_H_CH	(BODY_H/CH_COUNT)
#define FIG			{NULL,{1,0,0,1}}

//
// Display data
//

// **** Action *****

void dispShowNowDigestScreen(struct lcd_segmentButton *s);
void dispShowTodayGraphScreen(struct lcd_segmentButton *s);
void dispShowMonthGraphScreen(struct lcd_segmentButton *s);
void dispShowYearGraphScreen(struct lcd_segmentButton *s);
void dispShowNowDetailScreen(struct lcd_segmentGroup *fig, struct lcd_segmentCoordinate *touch);
void dispShowTodayDetailScreen(struct lcd_segmentGroup *fig, struct lcd_segmentCoordinate *touch);
void dispShowMonthDetailScreen(struct lcd_segmentGroup *fig, struct lcd_segmentCoordinate *touch);
void dispShowYearDetailScreen(struct lcd_segmentGroup *fig, struct lcd_segmentCoordinate *touch);
void dispCloseNowDetailScreen(struct lcd_segmentButton *s);
void dispShowSetupChannelScreen(struct lcd_segmentButton *s);
void dispCloseTodayDetailScreen(struct lcd_segmentButton *s);
void dispCloseMonthDetailScreen(struct lcd_segmentButton *s);
void dispCloseYearDetailScreen(struct lcd_segmentButton *s);
void dispResetYearDetailScreen(struct lcd_segmentButton *s);
void dispSaveSetupChannelScreen(struct lcd_segmentButton *s);
void dispCloseSetupChannelScreen(struct lcd_segmentButton *s);
void dispSaveSetupChannelScreen(struct lcd_segmentButton *s);
void dispChangeSetupChannelSensorType(struct lcd_segmentPullDownField *s);
void dispChangeSetupChannelVoltSensNumber(struct lcd_segmentPullDownField *f);
void dispShowSetupTimeScreen(struct lcd_segmentGroup *s);
void dispCloseSetupTimeScreen(struct lcd_segmentButton *s);
void dispSaveSetupTimeScreen(struct lcd_segmentButton *s);
void dispEpromSetupTimeScreen(struct lcd_segmentButton *s);

const unsigned short	iconBluetoothAdvertisement[6*11] = {
		LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_BLUE,    LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_BLACK,
		LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_BLUE,    LCD_COLOR_BLUE,     LCD_COLOR_BLACK,    LCD_COLOR_BLACK,
		LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_BLUE,    LCD_COLOR_BLACK,    LCD_COLOR_BLUE,     LCD_COLOR_BLACK,
		LCD_COLOR_BLUE,     LCD_COLOR_BLACK,    LCD_COLOR_BLUE,    LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_BLUE,
		LCD_COLOR_BLACK,    LCD_COLOR_BLUE,     LCD_COLOR_BLUE,    LCD_COLOR_BLACK,    LCD_COLOR_BLUE,     LCD_COLOR_BLACK,
		LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_BLUE,    LCD_COLOR_BLUE,     LCD_COLOR_BLACK,    LCD_COLOR_BLACK,
		LCD_COLOR_BLACK,    LCD_COLOR_BLUE,     LCD_COLOR_BLUE,    LCD_COLOR_BLACK,    LCD_COLOR_BLUE,     LCD_COLOR_BLACK,
		LCD_COLOR_BLUE,     LCD_COLOR_BLACK,    LCD_COLOR_BLUE,    LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_BLUE,
		LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_BLUE,    LCD_COLOR_BLACK,    LCD_COLOR_BLUE,     LCD_COLOR_BLACK,
		LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_BLUE,    LCD_COLOR_BLUE,     LCD_COLOR_BLACK,    LCD_COLOR_BLACK,
		LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_BLUE,    LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_BLACK,
	};
const unsigned short	iconBluetoothConnect[6*11] = {
		LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_SKYBLUE, LCD_COLOR_DARKBLUE, LCD_COLOR_BLACK,    LCD_COLOR_BLACK,
		LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_SKYBLUE, LCD_COLOR_SKYBLUE,  LCD_COLOR_DARKBLUE, LCD_COLOR_BLACK,
		LCD_COLOR_DARKBLUE, LCD_COLOR_BLACK,    LCD_COLOR_SKYBLUE, LCD_COLOR_DARKBLUE, LCD_COLOR_SKYBLUE,  LCD_COLOR_DARKBLUE,
		LCD_COLOR_SKYBLUE,  LCD_COLOR_DARKBLUE, LCD_COLOR_SKYBLUE, LCD_COLOR_BLACK,    LCD_COLOR_DARKBLUE, LCD_COLOR_SKYBLUE,
		LCD_COLOR_DARKBLUE, LCD_COLOR_SKYBLUE,  LCD_COLOR_SKYBLUE, LCD_COLOR_DARKBLUE, LCD_COLOR_SKYBLUE,  LCD_COLOR_DARKBLUE,
		LCD_COLOR_BLACK,    LCD_COLOR_DARKBLUE, LCD_COLOR_SKYBLUE, LCD_COLOR_SKYBLUE,  LCD_COLOR_BLACK,    LCD_COLOR_BLACK,
		LCD_COLOR_DARKBLUE, LCD_COLOR_SKYBLUE,  LCD_COLOR_SKYBLUE, LCD_COLOR_DARKBLUE, LCD_COLOR_SKYBLUE,  LCD_COLOR_DARKBLUE,
		LCD_COLOR_SKYBLUE,  LCD_COLOR_DARKBLUE, LCD_COLOR_SKYBLUE, LCD_COLOR_BLACK,    LCD_COLOR_DARKBLUE, LCD_COLOR_SKYBLUE,
		LCD_COLOR_DARKBLUE, LCD_COLOR_BLACK,    LCD_COLOR_SKYBLUE, LCD_COLOR_DARKBLUE, LCD_COLOR_SKYBLUE,  LCD_COLOR_DARKBLUE,
		LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_SKYBLUE, LCD_COLOR_SKYBLUE,  LCD_COLOR_DARKBLUE, LCD_COLOR_BLACK,
		LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_SKYBLUE, LCD_COLOR_DARKBLUE, LCD_COLOR_BLACK,    LCD_COLOR_BLACK,
	};
const unsigned short	iconUsbMemory[6*11] = {
		LCD_COLOR_BLACK,    LCD_COLOR_WHITE,    LCD_COLOR_WHITE,   LCD_COLOR_WHITE,    LCD_COLOR_WHITE,    LCD_COLOR_BLACK,
		LCD_COLOR_WHITE,    LCD_COLOR_GRAY,     LCD_COLOR_GRAY,    LCD_COLOR_GRAY,     LCD_COLOR_GRAY,     LCD_COLOR_WHITE,
		LCD_COLOR_WHITE,    LCD_COLOR_GRAY,     LCD_COLOR_GRAY,    LCD_COLOR_GRAY,     LCD_COLOR_GRAY,     LCD_COLOR_WHITE,
		LCD_COLOR_WHITE,    LCD_COLOR_GRAY,     LCD_COLOR_GRAY,    LCD_COLOR_GRAY,     LCD_COLOR_GRAY,     LCD_COLOR_WHITE,
		LCD_COLOR_WHITE,    LCD_COLOR_GRAY,     LCD_COLOR_GRAY,    LCD_COLOR_GRAY,     LCD_COLOR_GRAY,     LCD_COLOR_WHITE,
		LCD_COLOR_WHITE,    LCD_COLOR_GRAY,     LCD_COLOR_GRAY,    LCD_COLOR_GRAY,     LCD_COLOR_GRAY,     LCD_COLOR_WHITE,
		LCD_COLOR_WHITE,    LCD_COLOR_GRAY,     LCD_COLOR_GRAY,    LCD_COLOR_GRAY,     LCD_COLOR_GRAY,     LCD_COLOR_WHITE,
		LCD_COLOR_WHITE,    LCD_COLOR_GRAY,     LCD_COLOR_GRAY,    LCD_COLOR_GRAY,     LCD_COLOR_GRAY,     LCD_COLOR_WHITE,
		LCD_COLOR_BLACK,    LCD_COLOR_YELLOW,   LCD_COLOR_YELLOW,  LCD_COLOR_YELLOW,   LCD_COLOR_YELLOW,   LCD_COLOR_BLACK,
		LCD_COLOR_BLACK,    LCD_COLOR_YELLOW,   LCD_COLOR_YELLOW,  LCD_COLOR_YELLOW,   LCD_COLOR_YELLOW,   LCD_COLOR_BLACK,
		LCD_COLOR_BLACK,    LCD_COLOR_YELLOW,   LCD_COLOR_YELLOW,  LCD_COLOR_YELLOW,   LCD_COLOR_YELLOW,   LCD_COLOR_BLACK,
	};
const unsigned short	iconUsbMemoryWrite[6*11] = {
		LCD_COLOR_BLACK,    LCD_COLOR_WHITE,    LCD_COLOR_WHITE,   LCD_COLOR_WHITE,    LCD_COLOR_WHITE,    LCD_COLOR_BLACK,
		LCD_COLOR_WHITE,    LCD_COLOR_GRAY,     LCD_COLOR_GRAY,    LCD_COLOR_GRAY,     LCD_COLOR_GRAY,     LCD_COLOR_WHITE,
		LCD_COLOR_WHITE,    LCD_COLOR_GRAY,     LCD_COLOR_LIME,    LCD_COLOR_LIME,     LCD_COLOR_GRAY,     LCD_COLOR_WHITE,
		LCD_COLOR_WHITE,    LCD_COLOR_GRAY,     LCD_COLOR_LIME,    LCD_COLOR_LIME,     LCD_COLOR_GRAY,     LCD_COLOR_WHITE,
		LCD_COLOR_WHITE,    LCD_COLOR_LIME,     LCD_COLOR_LIME,    LCD_COLOR_LIME,     LCD_COLOR_LIME,     LCD_COLOR_WHITE,
		LCD_COLOR_WHITE,    LCD_COLOR_GRAY,     LCD_COLOR_LIME,    LCD_COLOR_LIME,     LCD_COLOR_GRAY,     LCD_COLOR_WHITE,
		LCD_COLOR_WHITE,    LCD_COLOR_GRAY,     LCD_COLOR_LIME,    LCD_COLOR_LIME,     LCD_COLOR_GRAY,     LCD_COLOR_WHITE,
		LCD_COLOR_WHITE,    LCD_COLOR_GRAY,     LCD_COLOR_GRAY,    LCD_COLOR_GRAY,     LCD_COLOR_GRAY,     LCD_COLOR_WHITE,
		LCD_COLOR_BLACK,    LCD_COLOR_YELLOW,   LCD_COLOR_YELLOW,  LCD_COLOR_YELLOW,   LCD_COLOR_YELLOW,   LCD_COLOR_BLACK,
		LCD_COLOR_BLACK,    LCD_COLOR_YELLOW,   LCD_COLOR_YELLOW,  LCD_COLOR_YELLOW,   LCD_COLOR_YELLOW,   LCD_COLOR_BLACK,
		LCD_COLOR_BLACK,    LCD_COLOR_YELLOW,   LCD_COLOR_YELLOW,  LCD_COLOR_YELLOW,   LCD_COLOR_YELLOW,   LCD_COLOR_BLACK,
	};
const unsigned short	iconUsbError[6*11] = {
		LCD_COLOR_BLACK,    LCD_COLOR_WHITE,   LCD_COLOR_WHITE,    LCD_COLOR_WHITE,    LCD_COLOR_WHITE,    LCD_COLOR_BLACK,
		LCD_COLOR_RED,      LCD_COLOR_BLACK,   LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_WHITE,
		LCD_COLOR_RED,      LCD_COLOR_RED,     LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_WHITE,
		LCD_COLOR_WHITE,    LCD_COLOR_RED,     LCD_COLOR_RED,      LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_WHITE,
		LCD_COLOR_WHITE,    LCD_COLOR_BLACK,   LCD_COLOR_RED,      LCD_COLOR_RED,      LCD_COLOR_BLACK,    LCD_COLOR_WHITE,
		LCD_COLOR_WHITE,    LCD_COLOR_BLACK,   LCD_COLOR_BLACK,    LCD_COLOR_RED,      LCD_COLOR_RED,      LCD_COLOR_WHITE,
		LCD_COLOR_WHITE,    LCD_COLOR_BLACK,   LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_RED,      LCD_COLOR_RED,
		LCD_COLOR_WHITE,    LCD_COLOR_WHITE,   LCD_COLOR_WHITE,    LCD_COLOR_WHITE,    LCD_COLOR_WHITE,    LCD_COLOR_RED,
		LCD_COLOR_BLACK,    LCD_COLOR_WHITE,   LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_WHITE,    LCD_COLOR_BLACK,
		LCD_COLOR_BLACK,    LCD_COLOR_WHITE,   LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_WHITE,    LCD_COLOR_BLACK,
		LCD_COLOR_BLACK,    LCD_COLOR_WHITE,   LCD_COLOR_WHITE,    LCD_COLOR_WHITE,    LCD_COLOR_WHITE,    LCD_COLOR_BLACK,
	};
const unsigned short	iconUsbUnplug[6*11] = {
		LCD_COLOR_BLACK,    LCD_COLOR_GRAY,     LCD_COLOR_GRAY,     LCD_COLOR_GRAY,     LCD_COLOR_GRAY,     LCD_COLOR_BLACK,
		LCD_COLOR_GRAY,     LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_GRAY,
		LCD_COLOR_GRAY,     LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_GRAY,
		LCD_COLOR_GRAY,     LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_GRAY,
		LCD_COLOR_GRAY,     LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_GRAY,
		LCD_COLOR_GRAY,     LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_GRAY,
		LCD_COLOR_GRAY,     LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_GRAY,
		LCD_COLOR_GRAY,     LCD_COLOR_GRAY,     LCD_COLOR_GRAY,     LCD_COLOR_GRAY,     LCD_COLOR_GRAY,     LCD_COLOR_GRAY,
		LCD_COLOR_BLACK,    LCD_COLOR_GRAY,     LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_GRAY,     LCD_COLOR_BLACK,
		LCD_COLOR_BLACK,    LCD_COLOR_GRAY,     LCD_COLOR_BLACK,    LCD_COLOR_BLACK,    LCD_COLOR_GRAY,     LCD_COLOR_BLACK,
		LCD_COLOR_BLACK,    LCD_COLOR_GRAY,     LCD_COLOR_GRAY,     LCD_COLOR_GRAY,     LCD_COLOR_GRAY,     LCD_COLOR_BLACK,
	};


// **** root and main layout *****

struct {
	struct lcd_segmentGroup			root;

	struct lcd_segmentGroup			head;
	struct lcd_segmentGroup			left;
	struct lcd_segmentGroup			body;
	struct lcd_segmentGroup			foot;

	struct lcd_segmentRect			hbg;
	struct lcd_segmentRect			hBorder;
	struct lcd_segmentStr			hname;
	struct lcd_segmentStr			hdate;
	struct lcd_segmentStr			htime;
	struct lcd_segmentStr			hmsg;
	struct lcd_segmentImage			hicon;

	struct lcd_segmentRect			lBorder1;
	struct lcd_segmentRect			lBorder2[CH_COUNT];
	struct lcd_segmentText			lineName[CH_COUNT];

	struct lcd_segmentRect			fbg;
	struct lcd_segmentButton		nowButton;
	struct lcd_segmentButton		todayButton;
	struct lcd_segmentButton		monthButton;
	struct lcd_segmentButton		yearButton;

	char							dateStr[16];
	char							timeStr[16];
	char							msgStr[16];
	char							lineNameStr[CH_COUNT][9];
} dispRootScreen = {
	{FIG, {0, 0, LCD_SCREEN_WIDTH, LCD_SCREEN_HEIGHT}},

	{FIG, {0, 0, HEAD_W,HEAD_H}, NULL, dispShowSetupTimeScreen},
	{FIG, {0, HEAD_H, LEFT_W,LEFT_H}},
	{FIG, {LEFT_W, HEAD_H, BODY_W,BODY_H}},
	{FIG, {0, LCD_SCREEN_HEIGHT-FOOT_H, FOOT_W, FOOT_H}},

	{FIG, {0, 0, HEAD_W,HEAD_H-1}, LCD_COLOR_BLACK},
	{FIG, {0, HEAD_H-1, HEAD_W, 1}, LCD_COLOR_WHITE},
	{FIG, {0,   0}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_BLACK}, "WHM0.8"},
	{FIG, {60,  0}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_BLACK}, dispRootScreen.dateStr},
	{FIG, {132, 0}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_BLACK}, dispRootScreen.timeStr},
	{FIG, {190, 0}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_BLACK}, dispRootScreen.msgStr},
	{FIG, {230, 0, 6, 11}, (unsigned short *)iconUsbUnplug},

	{FIG, {LEFT_W-1, 0, 1, LEFT_H}, LCD_COLOR_WHITE},
	{
		{FIG, {0, LEFT_H/CH_COUNT*1-1, LEFT_W, 1}, LCD_COLOR_WHITE},
		{FIG, {0, LEFT_H/CH_COUNT*2-1, LEFT_W, 1}, LCD_COLOR_WHITE},
		{FIG, {0, LEFT_H/CH_COUNT*3-1, LEFT_W, 1}, LCD_COLOR_WHITE},
		{FIG, {0, LEFT_H/CH_COUNT*4-1, LEFT_W, 1}, LCD_COLOR_WHITE},
		{FIG, {0, LEFT_H/CH_COUNT*5-1, LEFT_W, 1}, LCD_COLOR_WHITE},
		{FIG, {0, LEFT_H/CH_COUNT*6-1, LEFT_W, 1}, LCD_COLOR_WHITE},
		{FIG, {0, LEFT_H/CH_COUNT*7-1, LEFT_W, 1}, LCD_COLOR_WHITE},
	},
	{
		{FIG, {0, LEFT_H/CH_COUNT*0, LEFT_W-1, LEFT_H/CH_COUNT-1}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_BLACK}, {1,1}, dispRootScreen.lineNameStr[0]},
		{FIG, {0, LEFT_H/CH_COUNT*1, LEFT_W-1, LEFT_H/CH_COUNT-1}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_BLACK}, {1,1}, dispRootScreen.lineNameStr[1]},
		{FIG, {0, LEFT_H/CH_COUNT*2, LEFT_W-1, LEFT_H/CH_COUNT-1}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_BLACK}, {1,1}, dispRootScreen.lineNameStr[2]},
		{FIG, {0, LEFT_H/CH_COUNT*3, LEFT_W-1, LEFT_H/CH_COUNT-1}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_BLACK}, {1,1}, dispRootScreen.lineNameStr[3]},
		{FIG, {0, LEFT_H/CH_COUNT*4, LEFT_W-1, LEFT_H/CH_COUNT-1}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_BLACK}, {1,1}, dispRootScreen.lineNameStr[4]},
		{FIG, {0, LEFT_H/CH_COUNT*5, LEFT_W-1, LEFT_H/CH_COUNT-1}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_BLACK}, {1,1}, dispRootScreen.lineNameStr[5]},
		{FIG, {0, LEFT_H/CH_COUNT*6, LEFT_W-1, LEFT_H/CH_COUNT-1}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_BLACK}, {1,1}, dispRootScreen.lineNameStr[6]},
	},

	{FIG, {0, 0, FOOT_W,FOOT_H}, LCD_COLOR_BLACK},
	{{FIG, { (FOOT_W-50*4)/5,          1, 50, 18}}, {LCD_COLOR_WHITE, LCD_COLOR_GRAY}, "Now",   {0, 0}, dispShowNowDigestScreen},
	{{FIG, { ((FOOT_W-50*4)/5)*2+50,   1, 50, 18}}, {LCD_COLOR_WHITE, LCD_COLOR_GRAY}, "Today", {0, 0}, dispShowTodayGraphScreen},
	{{FIG, { ((FOOT_W-50*4)/5)*3+50*2, 1, 50, 18}}, {LCD_COLOR_WHITE, LCD_COLOR_GRAY}, "Month", {0, 0}, dispShowMonthGraphScreen},
	{{FIG, { ((FOOT_W-50*4)/5)*4+50*3, 1, 50, 18}}, {LCD_COLOR_WHITE, LCD_COLOR_GRAY}, "Year"  ,{0, 0}, dispShowYearGraphScreen},

};

// ****** Now *******

struct {
	struct lcd_segmentGroup	group;
	struct {
		struct lcd_segmentGroup	group;
		struct lcd_segmentRect	bg;
		struct lcd_segmentGage	gage;
		struct lcd_segmentStr	w;
		struct lcd_segmentStr	kwh;
		struct lcd_segment7seg	power;
		struct lcd_segment7seg	energy;
	}	ch[CH_COUNT];
} dispNowDigestScreen = {
	{FIG, {0, 0, BODY_W, BODY_H}, dispShowNowDetailScreen, NULL},
	{
		{
				{FIG, {0, BODY_H_CH*0, BODY_W, BODY_H_CH}},
				{FIG, {(int)(BODY_W*0.0),  (int)(BODY_H_CH*0.0), (int)(BODY_W*1.0), (int)(BODY_H_CH*0.9)}, LCD_COLOR_BLACK},
				{FIG, {(int)(BODY_W*0.0),  (int)(BODY_H_CH*0.9), (int)(BODY_W*1.0), (int)(BODY_H_CH*0.1)}, {LCD_COLOR_RED, LCD_COLOR_GRAY}, {0}, 0.0},
				{FIG, {(int)(BODY_W*0.55), (int)(BODY_H_CH*0.7)}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_BLACK}, "W"},
				{FIG, {(int)(BODY_W*0.9),  (int)(BODY_H_CH*0.7)}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_BLACK}, "kWh"},
				{FIG, {(int)(BODY_W*0.05), (int)(BODY_H_CH*0.05),(int)(BODY_W*0.5), (int)(BODY_H_CH*0.8)}, {LCD_COLOR_WHITE, LCD_COLOR_DIMGRAY}, 2, 5, -1, 0},
				{FIG, {(int)(BODY_W*0.6),  (int)(BODY_H_CH*0.2), (int)(BODY_W*0.3), (int)(BODY_H_CH*0.6)}, {LCD_COLOR_WHITE, LCD_COLOR_DIMGRAY}, 1, 4, 1, 0},
		},
		{
				{FIG, {0, BODY_H_CH*1, BODY_W, BODY_H_CH}},
				{FIG, {(int)(BODY_W*0.0),  (int)(BODY_H_CH*0.0), (int)(BODY_W*1.0), (int)(BODY_H_CH*0.9)}, LCD_COLOR_BLACK},
				{FIG, {(int)(BODY_W*0.0),  (int)(BODY_H_CH*0.9), (int)(BODY_W*1.0), (int)(BODY_H_CH*0.1)}, {LCD_COLOR_RED, LCD_COLOR_GRAY}, {0}, 0.0},
				{FIG, {(int)(BODY_W*0.55), (int)(BODY_H_CH*0.7)}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_BLACK}, "W"},
				{FIG, {(int)(BODY_W*0.9),  (int)(BODY_H_CH*0.7)}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_BLACK}, "kWh"},
				{FIG, {(int)(BODY_W*0.05), (int)(BODY_H_CH*0.05),(int)(BODY_W*0.5), (int)(BODY_H_CH*0.8)}, {LCD_COLOR_WHITE, LCD_COLOR_DIMGRAY}, 2, 5, -1, 0},
				{FIG, {(int)(BODY_W*0.6),  (int)(BODY_H_CH*0.2), (int)(BODY_W*0.3), (int)(BODY_H_CH*0.6)}, {LCD_COLOR_WHITE, LCD_COLOR_DIMGRAY}, 1, 4, 1, 0},
		},
		{
				{FIG, {0, BODY_H_CH*2, BODY_W, BODY_H_CH}},
				{FIG, {(int)(BODY_W*0.0),  (int)(BODY_H_CH*0.0), (int)(BODY_W*1.0), (int)(BODY_H_CH*0.9)}, LCD_COLOR_BLACK},
				{FIG, {(int)(BODY_W*0.0),  (int)(BODY_H_CH*0.9), (int)(BODY_W*1.0), (int)(BODY_H_CH*0.1)}, {LCD_COLOR_RED, LCD_COLOR_GRAY}, {0}, 0.0},
				{FIG, {(int)(BODY_W*0.55), (int)(BODY_H_CH*0.7)}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_BLACK}, "W"},
				{FIG, {(int)(BODY_W*0.9),  (int)(BODY_H_CH*0.7)}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_BLACK}, "kWh"},
				{FIG, {(int)(BODY_W*0.05), (int)(BODY_H_CH*0.05),(int)(BODY_W*0.5), (int)(BODY_H_CH*0.8)}, {LCD_COLOR_WHITE, LCD_COLOR_DIMGRAY}, 2, 5, -1, 0},
				{FIG, {(int)(BODY_W*0.6),  (int)(BODY_H_CH*0.2), (int)(BODY_W*0.3), (int)(BODY_H_CH*0.6)}, {LCD_COLOR_WHITE, LCD_COLOR_DIMGRAY}, 1, 4, 1, 0},
		},
		{
				{FIG, {0, BODY_H_CH*3, BODY_W, BODY_H_CH}},
				{FIG, {(int)(BODY_W*0.0),  (int)(BODY_H_CH*0.0), (int)(BODY_W*1.0), (int)(BODY_H_CH*0.9)}, LCD_COLOR_BLACK},
				{FIG, {(int)(BODY_W*0.0),  (int)(BODY_H_CH*0.9), (int)(BODY_W*1.0), (int)(BODY_H_CH*0.1)}, {LCD_COLOR_RED, LCD_COLOR_GRAY}, {0}, 0.0},
				{FIG, {(int)(BODY_W*0.55), (int)(BODY_H_CH*0.7)}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_BLACK}, "W"},
				{FIG, {(int)(BODY_W*0.9),  (int)(BODY_H_CH*0.7)}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_BLACK}, "kWh"},
				{FIG, {(int)(BODY_W*0.05), (int)(BODY_H_CH*0.05),(int)(BODY_W*0.5), (int)(BODY_H_CH*0.8)}, {LCD_COLOR_WHITE, LCD_COLOR_DIMGRAY}, 2, 5, -1, 0},
				{FIG, {(int)(BODY_W*0.6),  (int)(BODY_H_CH*0.2), (int)(BODY_W*0.3), (int)(BODY_H_CH*0.6)}, {LCD_COLOR_WHITE, LCD_COLOR_DIMGRAY}, 1, 4, 1, 0},
		},
		{
				{FIG, {0, BODY_H_CH*4, BODY_W, BODY_H_CH}},
				{FIG, {(int)(BODY_W*0.0),  (int)(BODY_H_CH*0.0), (int)(BODY_W*1.0), (int)(BODY_H_CH*0.9)}, LCD_COLOR_BLACK},
				{FIG, {(int)(BODY_W*0.0),  (int)(BODY_H_CH*0.9), (int)(BODY_W*1.0), (int)(BODY_H_CH*0.1)}, {LCD_COLOR_RED, LCD_COLOR_GRAY}, {0}, 0.0},
				{FIG, {(int)(BODY_W*0.55), (int)(BODY_H_CH*0.7)}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_BLACK}, "W"},
				{FIG, {(int)(BODY_W*0.9),  (int)(BODY_H_CH*0.7)}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_BLACK}, "kWh"},
				{FIG, {(int)(BODY_W*0.05), (int)(BODY_H_CH*0.05),(int)(BODY_W*0.5), (int)(BODY_H_CH*0.8)}, {LCD_COLOR_WHITE, LCD_COLOR_DIMGRAY}, 2, 5, -1, 0},
				{FIG, {(int)(BODY_W*0.6),  (int)(BODY_H_CH*0.2), (int)(BODY_W*0.3), (int)(BODY_H_CH*0.6)}, {LCD_COLOR_WHITE, LCD_COLOR_DIMGRAY}, 1, 4, 1, 0},
		},
		{
				{FIG, {0, BODY_H_CH*5, BODY_W, BODY_H_CH}},
				{FIG, {(int)(BODY_W*0.0),  (int)(BODY_H_CH*0.0), (int)(BODY_W*1.0), (int)(BODY_H_CH*0.9)}, LCD_COLOR_BLACK},
				{FIG, {(int)(BODY_W*0.0),  (int)(BODY_H_CH*0.9), (int)(BODY_W*1.0), (int)(BODY_H_CH*0.1)}, {LCD_COLOR_RED, LCD_COLOR_GRAY}, {0}, 0.0},
				{FIG, {(int)(BODY_W*0.55), (int)(BODY_H_CH*0.7)}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_BLACK}, "W"},
				{FIG, {(int)(BODY_W*0.9),  (int)(BODY_H_CH*0.7)}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_BLACK}, "kWh"},
				{FIG, {(int)(BODY_W*0.05), (int)(BODY_H_CH*0.05),(int)(BODY_W*0.5), (int)(BODY_H_CH*0.8)}, {LCD_COLOR_WHITE, LCD_COLOR_DIMGRAY}, 2, 5, -1, 0},
				{FIG, {(int)(BODY_W*0.6),  (int)(BODY_H_CH*0.2), (int)(BODY_W*0.3), (int)(BODY_H_CH*0.6)}, {LCD_COLOR_WHITE, LCD_COLOR_DIMGRAY}, 1, 4, 1, 0},
		},
		{
				{FIG, {0, BODY_H_CH*6, BODY_W, BODY_H_CH}},
				{FIG, {(int)(BODY_W*0.0),  (int)(BODY_H_CH*0.0), (int)(BODY_W*1.0), (int)(BODY_H_CH*0.9)}, LCD_COLOR_BLACK},
				{FIG, {(int)(BODY_W*0.0),  (int)(BODY_H_CH*0.9), (int)(BODY_W*1.0), (int)(BODY_H_CH*0.1)}, {LCD_COLOR_RED, LCD_COLOR_GRAY}, {0}, 0.0},
				{FIG, {(int)(BODY_W*0.55), (int)(BODY_H_CH*0.7)}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_BLACK}, "W"},
				{FIG, {(int)(BODY_W*0.9),  (int)(BODY_H_CH*0.7)}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_BLACK}, "kWh"},
				{FIG, {(int)(BODY_W*0.05), (int)(BODY_H_CH*0.05),(int)(BODY_W*0.5), (int)(BODY_H_CH*0.8)}, {LCD_COLOR_WHITE, LCD_COLOR_DIMGRAY}, 2, 5, -1, 0},
				{FIG, {(int)(BODY_W*0.6),  (int)(BODY_H_CH*0.2), (int)(BODY_W*0.3), (int)(BODY_H_CH*0.6)}, {LCD_COLOR_WHITE, LCD_COLOR_DIMGRAY}, 1, 4, 1, 0},
		},
	},
};


// ****** Now sub screen *******

struct {
	struct lcd_segmentGroup			group;
	struct lcd_segmentRect			bg;
	struct lcd_segmentTextField		title;
	char							titleStr[16];
	struct lcd_segmentStr			l1;
	struct lcd_segmentStr			l2;
	struct lcd_segmentStr			l3;
	struct lcd_segmentStr			l4;
	struct lcd_segmentStr			l5;
	struct lcd_segmentStr			l6;
	struct lcd_segmentStr			l7;
	struct lcd_segmentStr			l8;
	struct lcd_segmentStr			l9;
	struct lcd_segmentFloatField	energyOfMonth;
	struct lcd_segmentFloatField	energyOfDay;
	struct lcd_segmentIntegerField	effectivePower;
	struct lcd_segmentIntegerField	apparentPower;
	struct lcd_segmentIntegerField	powerFactor;
	struct lcd_segmentIntegerField	voltage;
	struct lcd_segmentFloatField	current;
	struct lcd_segmentIntegerField	frequency;
	struct lcd_segmentRect			wave;
	struct lcd_segmentRect			line;
	struct waveFigure				wave1;
	struct waveFigure				wave2;
	struct lcd_segmentButton		setupButton;
	struct lcd_segmentButton		closeButton;
} dispNowDetailScreen = {
	{FIG, {30, 30, 180, 260}},
	{FIG, { 0,  0, 180, 260}, LCD_COLOR_LIGHTGRAY},
	{{FIG, {2, 2,  176, 16}}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_GRAY}, {1,1}, dispNowDetailScreen.titleStr, sizeof(dispNowDetailScreen.titleStr)},
	"ChX",
	{FIG,  {10,  16*0+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "Energy of month"},
	{FIG,  {10,  16*1+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "Energy of day"},
	{FIG,  {10,  16*2+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "Effective power"},
	{FIG,  {10,  16*3+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "Apparent power"},
	{FIG,  {10,  16*4+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "Power factor"},
	{FIG,  {10,  16*5+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "Voltage"},
	{FIG,  {10,  16*6+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "Current"},
	{FIG,  {10,  16*7+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "Frequency"},
	{FIG,  {10,  16*8+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "Wave form"},
	{{FIG, {110, 16*0+26,  60, 14}}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_GRAY}, "%5.1f kWh", 0.0, 9999.9, 0.0},
	{{FIG, {110, 16*1+26,  60, 14}}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_GRAY}, "%5.2f kWh", 0.0, 9999.9, 0.0},
	{{FIG, {110, 16*2+26,  60, 14}}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_GRAY}, "%5d W",     0,   99999,  0},
	{{FIG, {110, 16*3+26,  60, 14}}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_GRAY}, "%5d VA",    0,   99999,  0},
	{{FIG, {110, 16*4+26,  60, 14}}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_GRAY}, "%5d %%",    0,   100,    0},
	{{FIG, {110, 16*5+26,  60, 14}}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_GRAY}, "%5d V",     0,   99999,  0},
	{{FIG, {110, 16*6+26,  60, 14}}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_GRAY}, "%5.2f A",   0.0, 9999.9, 0.0},
	{{FIG, {110, 16*7+26,  60, 14}}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_GRAY}, "%5d Hz",    0.0, 99999,  0},
	{FIG,  {10,  16*9+26, 160, 40},  LCD_COLOR_GRAY},
	{FIG,  {10,  16*9+46, 160, 1},   LCD_COLOR_WHITE},
	{FIG,  {10,  16*9+26, 160, 40}, {LCD_COLOR_LIME, 0}, 160, CH_COUNT, 0, NULL},
	{FIG,  {10,  16*9+26, 160, 40}, {LCD_COLOR_RED,  0}, 160, CH_COUNT, 0, NULL},
	{{FIG, {20,  16*9+80, 40, 18}}, {LCD_COLOR_WHITE, LCD_COLOR_GRAY}, "Setup",  {0, 0}, dispShowSetupChannelScreen},
	{{FIG, {90,  16*9+80, 60, 18}}, {LCD_COLOR_WHITE, LCD_COLOR_GRAY}, "Close",  {0, 0}, dispCloseNowDetailScreen},
};

// ********** Today ***********

//							  rrrrrggggggbbbbb
#define EPCO_RATE_RED		0b1111111000011000
#define EPCO_RATE_N10		0b1111111000011111
#define EPCO_RATE_BLUE		0b0100011000011111
#define EPCO_RATE_ORANGE	0b1111110000000000
struct {
	struct lcd_segmentGroup			group;
	struct lcd_segmentStr			s0;
	struct lcd_segmentStr			s6;
	struct lcd_segmentStr			s12;
	struct lcd_segmentStr			s18;
	struct lcd_segmentStr			unit;
	struct {
		struct lcd_segmentGroup			group;
		struct lcd_segmentRect			border;
		struct lcd_segmentGage			gage[24];
		struct lcd_segmentGage			now;
		struct lcd_segmentStr			scale;

		char							scaleStr[8];
	}								ch[CH_COUNT];
}	dispTodayGraphScreen = {
		{FIG, {0, 0, BODY_W, BODY_H}, dispShowTodayDetailScreen, NULL},
		{FIG, {1,          2}, {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, "0"},
		{FIG, {BODY_W/4,   2}, {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, "6"},
		{FIG, {BODY_W/2,   2}, {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, "12"},
		{FIG, {BODY_W/4*3, 2}, {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, "18"},
		{FIG, {BODY_W-LCD_FONT_WIDTH*3, LCD_FONT_HEIGHT+2}, {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, "kWh"},
		{
			{
				{FIG, {0, BODY_H_CH*0, BODY_W, BODY_H_CH}},
				{FIG, {0, BODY_H_CH-1, BODY_W, 1}, LCD_COLOR_WHITE},
				{
					{FIG, {BODY_W/24*0,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*1,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*2,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*3,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*4,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*5,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*6,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*7,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_N10,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*8,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*9,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*10, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*11, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*12, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*13, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_ORANGE, LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*14, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_ORANGE, LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*15, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_ORANGE, LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*16, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*17, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*18, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*19, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*20, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*21, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*22, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_N10,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*23, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
				},
				{FIG, {0, 0, BODY_W/24/2, BODY_H_CH-1}, {LCD_COLOR_YELLOW, LCD_COLOR_GRAY}, {1}, 0.0},
				{FIG, {BODY_W-LCD_FONT_WIDTH*4, 2}, {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, dispTodayGraphScreen.ch[0].scaleStr},
			},
			{
				{FIG, {0, BODY_H_CH*1, BODY_W,BODY_H_CH}},
				{FIG, {0, BODY_H_CH-1, BODY_W,        1}, LCD_COLOR_WHITE},
				{
					{FIG, {BODY_W/24*0,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*1,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*2,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*3,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*4,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*5,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*6,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*7,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_N10,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*8,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*9,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*10, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*11, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*12, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*13, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_ORANGE, LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*14, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_ORANGE, LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*15, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_ORANGE, LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*16, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*17, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*18, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*19, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*20, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*21, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*22, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_N10,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*23, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
				},
				{FIG, {0, 0, BODY_W/24/2, BODY_H_CH-1}, {LCD_COLOR_YELLOW, LCD_COLOR_GRAY}, {1}, 0.0},
				{FIG, {BODY_W-LCD_FONT_WIDTH*4, 2}, {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, dispTodayGraphScreen.ch[1].scaleStr},
			},
			{
				{FIG, {0, BODY_H_CH*2, BODY_W,BODY_H_CH}},
				{FIG, {0, BODY_H_CH-1, BODY_W,        1}, LCD_COLOR_WHITE},
				{
					{FIG, {BODY_W/24*0,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*1,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*2,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*3,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*4,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*5,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*6,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*7,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_N10,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*8,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*9,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*10, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*11, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*12, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*13, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_ORANGE, LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*14, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_ORANGE, LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*15, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_ORANGE, LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*16, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*17, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*18, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*19, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*20, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*21, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*22, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_N10,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*23, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
				},
				{FIG, {0, 0, BODY_W/24/2, BODY_H_CH-1}, {LCD_COLOR_YELLOW, LCD_COLOR_GRAY}, {1}, 0.0},
				{FIG, {BODY_W-LCD_FONT_WIDTH*4, 2}, {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, dispTodayGraphScreen.ch[2].scaleStr},
			},
			{
				{FIG, {0, BODY_H_CH*3, BODY_W,BODY_H_CH}},
				{FIG, {0, BODY_H_CH-1, BODY_W,        1}, LCD_COLOR_WHITE},
				{
					{FIG, {BODY_W/24*0,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*1,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*2,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*3,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*4,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*5,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*6,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*7,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_N10,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*8,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*9,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*10, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*11, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*12, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*13, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_ORANGE, LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*14, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_ORANGE, LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*15, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_ORANGE, LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*16, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*17, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*18, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*19, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*20, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*21, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*22, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_N10,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*23, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
				},
				{FIG, {0, 0, BODY_W/24/2, BODY_H_CH-1}, {LCD_COLOR_YELLOW, LCD_COLOR_GRAY}, {1}, 0.0},
				{FIG, {BODY_W-LCD_FONT_WIDTH*4, 2}, {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, dispTodayGraphScreen.ch[3].scaleStr},
			},
			{
				{FIG, {0, BODY_H_CH*4, BODY_W,BODY_H_CH}},
				{FIG, {0, BODY_H_CH-1, BODY_W,        1}, LCD_COLOR_WHITE},
				{
					{FIG, {BODY_W/24*0,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*1,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*2,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*3,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*4,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*5,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*6,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*7,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_N10,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*8,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*9,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*10, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*11, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*12, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*13, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_ORANGE, LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*14, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_ORANGE, LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*15, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_ORANGE, LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*16, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*17, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*18, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*19, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*20, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*21, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*22, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_N10,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*23, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
				},
				{FIG, {0, 0, BODY_W/24/2, BODY_H_CH-1}, {LCD_COLOR_YELLOW, LCD_COLOR_GRAY}, {1}, 0.0},
				{FIG, {BODY_W-LCD_FONT_WIDTH*4, 2}, {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, dispTodayGraphScreen.ch[4].scaleStr},
			},
			{
				{FIG, {0, BODY_H_CH*5, BODY_W,BODY_H_CH}},
				{FIG, {0, BODY_H_CH-1, BODY_W,        1}, LCD_COLOR_WHITE},
				{
					{FIG, {BODY_W/24*0,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*1,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*2,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*3,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*4,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*5,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*6,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*7,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_N10,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*8,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*9,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*10, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*11, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*12, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*13, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_ORANGE, LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*14, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_ORANGE, LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*15, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_ORANGE, LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*16, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*17, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*18, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*19, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*20, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*21, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*22, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_N10,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*23, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
				},
				{FIG, {0, 0, BODY_W/24/2, BODY_H_CH-1}, {LCD_COLOR_YELLOW, LCD_COLOR_GRAY}, {1}, 0.0},
				{FIG, {BODY_W-LCD_FONT_WIDTH*4, 2}, {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, dispTodayGraphScreen.ch[5].scaleStr},
			},
			{
				{FIG, {0, BODY_H_CH*6, BODY_W,BODY_H_CH}},
				{FIG, {0, BODY_H_CH-1, BODY_W,        1}, LCD_COLOR_WHITE},
				{
					{FIG, {BODY_W/24*0,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*1,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*2,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*3,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*4,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*5,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*6,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*7,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_N10,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*8,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*9,  0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*10, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*11, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*12, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*13, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_ORANGE, LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*14, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_ORANGE, LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*15, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_ORANGE, LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*16, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*17, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*18, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*19, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*20, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*21, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_RED,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*22, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_N10,    LCD_COLOR_BLACK}, {1}, 0.0},
					{FIG, {BODY_W/24*23, 0, BODY_W/24, BODY_H_CH-1}, {EPCO_RATE_BLUE,   LCD_COLOR_BLACK}, {1}, 0.0},
				},
				{FIG, {0, 0, BODY_W/24/2, BODY_H_CH-1}, {LCD_COLOR_YELLOW, LCD_COLOR_GRAY}, {1}, 0.0},
				{FIG, {BODY_W-LCD_FONT_WIDTH*4, 2}, {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, dispTodayGraphScreen.ch[6].scaleStr},
			},
		}
};

// ****** Today sub screen *******

struct {
	struct lcd_segmentGroup			group;
	struct lcd_segmentRect			bg;
	struct lcd_segmentTextField		title;
	struct lcd_segmentStr			label[24];
	struct lcd_segmentStr			energy[24];
	struct lcd_segmentStr			unit;
	struct lcd_segmentButton		closeButton;

	char							titleStr[16];
	char							energyStr[24][8];
} dispTodayDetailScreen = {
	{FIG, {60, 15, 120, 300}},
	{FIG, { 0,  0, 120, 300}, LCD_COLOR_LIGHTGRAY},
	{{FIG, {2, 2,  116, 16}}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_GRAY}, {1,1}, dispTodayDetailScreen.titleStr, sizeof(dispTodayDetailScreen.titleStr)},
	{
		{FIG,  {10, 10*0+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, " 0:00"},
		{FIG,  {10, 10*1+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, " 1:00"},
		{FIG,  {10, 10*2+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, " 2:00"},
		{FIG,  {10, 10*3+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, " 3:00"},
		{FIG,  {10, 10*4+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, " 4:00"},
		{FIG,  {10, 10*5+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, " 5:00"},
		{FIG,  {10, 10*6+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, " 6:00"},
		{FIG,  {10, 10*7+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, " 7:00"},
		{FIG,  {10, 10*8+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, " 8:00"},
		{FIG,  {10, 10*9+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, " 9:00"},
		{FIG,  {10, 10*10+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "10:00"},
		{FIG,  {10, 10*11+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "11:00"},
		{FIG,  {10, 10*12+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "12:00"},
		{FIG,  {10, 10*13+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "13:00"},
		{FIG,  {10, 10*14+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "14:00"},
		{FIG,  {10, 10*15+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "15:00"},
		{FIG,  {10, 10*16+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "16:00"},
		{FIG,  {10, 10*17+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "17:00"},
		{FIG,  {10, 10*18+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "18:00"},
		{FIG,  {10, 10*19+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "19:00"},
		{FIG,  {10, 10*20+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "20:00"},
		{FIG,  {10, 10*21+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "21:00"},
		{FIG,  {10, 10*22+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "22:00"},
		{FIG,  {10, 10*23+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "23:00"},
	},
	{
		{FIG,  {50, 10*0+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispTodayDetailScreen.energyStr[0]},
		{FIG,  {50, 10*1+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispTodayDetailScreen.energyStr[1]},
		{FIG,  {50, 10*2+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispTodayDetailScreen.energyStr[2]},
		{FIG,  {50, 10*3+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispTodayDetailScreen.energyStr[3]},
		{FIG,  {50, 10*4+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispTodayDetailScreen.energyStr[4]},
		{FIG,  {50, 10*5+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispTodayDetailScreen.energyStr[5]},
		{FIG,  {50, 10*6+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispTodayDetailScreen.energyStr[6]},
		{FIG,  {50, 10*7+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispTodayDetailScreen.energyStr[7]},
		{FIG,  {50, 10*8+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispTodayDetailScreen.energyStr[8]},
		{FIG,  {50, 10*9+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispTodayDetailScreen.energyStr[9]},
		{FIG,  {50, 10*10+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispTodayDetailScreen.energyStr[10]},
		{FIG,  {50, 10*11+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispTodayDetailScreen.energyStr[11]},
		{FIG,  {50, 10*12+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispTodayDetailScreen.energyStr[12]},
		{FIG,  {50, 10*13+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispTodayDetailScreen.energyStr[13]},
		{FIG,  {50, 10*14+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispTodayDetailScreen.energyStr[14]},
		{FIG,  {50, 10*15+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispTodayDetailScreen.energyStr[15]},
		{FIG,  {50, 10*16+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispTodayDetailScreen.energyStr[16]},
		{FIG,  {50, 10*17+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispTodayDetailScreen.energyStr[17]},
		{FIG,  {50, 10*18+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispTodayDetailScreen.energyStr[18]},
		{FIG,  {50, 10*19+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispTodayDetailScreen.energyStr[19]},
		{FIG,  {50, 10*20+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispTodayDetailScreen.energyStr[20]},
		{FIG,  {50, 10*21+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispTodayDetailScreen.energyStr[21]},
		{FIG,  {50, 10*22+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispTodayDetailScreen.energyStr[22]},
		{FIG,  {50, 10*23+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispTodayDetailScreen.energyStr[23]},
	},
	{FIG,  {90, 10*23+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "kWh"},
	{{FIG, {40, 270, 40, 18}}, {LCD_COLOR_WHITE, LCD_COLOR_GRAY}, "Close",  {0, 0}, dispCloseTodayDetailScreen},
};

// ********** Month ***********

struct {
	struct lcd_segmentGroup			group;
	struct lcd_segmentRect			bg;
	struct lcd_segmentStr			s1;
	struct lcd_segmentStr			s10;
	struct lcd_segmentStr			s20;
	struct lcd_segmentStr			s30;
	struct lcd_segmentStr			unit;
	struct {
		struct lcd_segmentGroup			group;
		struct lcd_segmentRect			border;
		struct lcd_segmentPoly			graphLine;
		struct lcd_segmentGage			now;
		struct lcd_segmentStr			scale;

		struct lcd_segmentCoordinate	point[31];
		char							scaleStr[8];
	}								ch[CH_COUNT];
}	dispMonthGraphScreen = {
		{FIG, {0, 0, BODY_W, BODY_H}, dispShowMonthDetailScreen, NULL},
		{FIG, {0, 0, BODY_W, BODY_H}, LCD_COLOR_BLACK},
		{FIG, {0, 2},            {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, "1"},
		{FIG, {BODY_W/31*9, 2},  {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, "10"},
		{FIG, {BODY_W/31*19, 2}, {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, "20"},
		{FIG, {BODY_W/31*29, 2}, {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, "30"},
		{FIG, {BODY_W-LCD_FONT_WIDTH*3, LCD_FONT_HEIGHT+2}, {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, "kWh"},
		{
			{	// ch[0]
				{FIG, {0, BODY_H_CH*0,  BODY_W, BODY_H_CH}},
				{FIG, {0, BODY_H_CH-1,  BODY_W, 1}, LCD_COLOR_WHITE},
				{FIG, LCD_COLOR_LIGHTGREEN, 2, 31, dispMonthGraphScreen.ch[0].point},
				{FIG, {0, 0, BODY_W/31, BODY_H_CH-1}, {LCD_COLOR_YELLOW, LCD_COLOR_GRAY}, {1}, 0.0},
				{FIG, {BODY_W-LCD_FONT_WIDTH*4, 2}, {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, dispMonthGraphScreen.ch[0].scaleStr},
			},
			{	// ch[1]
				{FIG, {0, BODY_H_CH*1,   BODY_W, BODY_H_CH}},
				{FIG, {0, BODY_H_CH-1,  BODY_W, 1}, LCD_COLOR_WHITE},
				{FIG, LCD_COLOR_LIGHTGREEN, 2, 31, dispMonthGraphScreen.ch[1].point},
				{FIG, {0, 0, BODY_W/31, BODY_H_CH-1}, {LCD_COLOR_YELLOW, LCD_COLOR_GRAY}, {1}, 0.0},
				{FIG, {BODY_W-LCD_FONT_WIDTH*4, 2}, {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, dispMonthGraphScreen.ch[1].scaleStr},
			},
			{	// ch[2]
				{FIG, {0, BODY_H_CH*2,   BODY_W, BODY_H_CH}},
				{FIG, {0, BODY_H_CH-1,  BODY_W, 1}, LCD_COLOR_WHITE},
				{FIG, LCD_COLOR_LIGHTGREEN, 2, 31, dispMonthGraphScreen.ch[2].point},
				{FIG, {0, 0, BODY_W/31, BODY_H_CH-1}, {LCD_COLOR_YELLOW, LCD_COLOR_GRAY}, {1}, 0.0},
				{FIG, {BODY_W-LCD_FONT_WIDTH*4, 2}, {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, dispMonthGraphScreen.ch[2].scaleStr},
			},
			{	// ch[3]
				{FIG, {0, BODY_H_CH*3,   BODY_W, BODY_H_CH}},
				{FIG, {0, BODY_H_CH-1,  BODY_W, 1}, LCD_COLOR_WHITE},
				{FIG, LCD_COLOR_LIGHTGREEN, 2, 31, dispMonthGraphScreen.ch[3].point},
				{FIG, {0, 0, BODY_W/31, BODY_H_CH-1}, {LCD_COLOR_YELLOW, LCD_COLOR_GRAY}, {1}, 0.0},
				{FIG, {BODY_W-LCD_FONT_WIDTH*4, 2}, {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, dispMonthGraphScreen.ch[3].scaleStr},
			},
			{	// ch[4]
				{FIG, {0, BODY_H_CH*4,   BODY_W, BODY_H_CH}},
				{FIG, {0, BODY_H_CH-1,  BODY_W, 1}, LCD_COLOR_WHITE},
				{FIG, LCD_COLOR_LIGHTGREEN, 2, 31, dispMonthGraphScreen.ch[4].point},
				{FIG, {0, 0, BODY_W/31, BODY_H_CH-1}, {LCD_COLOR_YELLOW, LCD_COLOR_GRAY}, {1}, 0.0},
				{FIG, {BODY_W-LCD_FONT_WIDTH*4, 2}, {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, dispMonthGraphScreen.ch[4].scaleStr},
			},
			{	// ch[5]
				{FIG, {0, BODY_H_CH*5,   BODY_W, BODY_H_CH}},
				{FIG, {0, BODY_H_CH-1,  BODY_W, 1}, LCD_COLOR_WHITE},
				{FIG, LCD_COLOR_LIGHTGREEN, 2, 31, dispMonthGraphScreen.ch[5].point},
				{FIG, {0, 0, BODY_W/31, BODY_H_CH-1}, {LCD_COLOR_YELLOW, LCD_COLOR_GRAY}, {1}, 0.0},
				{FIG, {BODY_W-LCD_FONT_WIDTH*4, 2}, {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, dispMonthGraphScreen.ch[5].scaleStr},
			},
			{	// ch[6]
				{FIG, {0, BODY_H_CH*6,   BODY_W, BODY_H_CH}},
				{FIG, {0, BODY_H_CH-1,  BODY_W, 1}, LCD_COLOR_WHITE},
				{FIG, LCD_COLOR_LIGHTGREEN, 2, 31, dispMonthGraphScreen.ch[6].point},
				{FIG, {0, 0, BODY_W/31, BODY_H_CH-1}, {LCD_COLOR_YELLOW, LCD_COLOR_GRAY}, {1}, 0.0},
				{FIG, {BODY_W-LCD_FONT_WIDTH*4, 2}, {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, dispMonthGraphScreen.ch[6].scaleStr},
			},
		}
};

// ****** Month sub screen *******

struct {
	struct lcd_segmentGroup			group;
	struct lcd_segmentRect			bg;
	struct lcd_segmentTextField		title;
	struct lcd_segmentStr			label[7];
	struct lcd_segmentStr			energy[38];
	struct lcd_segmentStr			unit;
	struct lcd_segmentButton		closeButton;

	char							titleStr[16];
	char							energyStr[38][8];
} dispMonthDetailScreen = {
	{FIG, { 5, 80, 230, 160}},
	{FIG, { 0,  0, 230, 160}, LCD_COLOR_LIGHTGRAY},
	{{FIG, {2, 2,  226, 16}}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_GRAY}, {1,1}, dispMonthDetailScreen.titleStr, sizeof(dispMonthDetailScreen.titleStr)},
	{
		{FIG,  {30*0+10, 26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, " Mon"},
		{FIG,  {30*1+10, 26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, " Tue"},
		{FIG,  {30*2+10, 26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, " Wed"},
		{FIG,  {30*3+10, 26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, " Tue"},
		{FIG,  {30*4+10, 26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, " Fri"},
		{FIG,  {30*5+10, 26},  {LCD_COLOR_BLUE,     LCD_COLOR_LIGHTGRAY}, " Sat"},
		{FIG,  {30*6+10, 26},  {LCD_COLOR_RED,      LCD_COLOR_LIGHTGRAY}, " Sun"},
	},
	{
		{FIG,  {30*0+10, 16*0+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[0]},
		{FIG,  {30*1+10, 16*0+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[1]},
		{FIG,  {30*2+10, 16*0+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[2]},
		{FIG,  {30*3+10, 16*0+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[3]},
		{FIG,  {30*4+10, 16*0+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[4]},
		{FIG,  {30*5+10, 16*0+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[5]},
		{FIG,  {30*6+10, 16*0+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[6]},

		{FIG,  {30*0+10, 16*1+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[7]},
		{FIG,  {30*1+10, 16*1+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[8]},
		{FIG,  {30*2+10, 16*1+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[9]},
		{FIG,  {30*3+10, 16*1+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[10]},
		{FIG,  {30*4+10, 16*1+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[11]},
		{FIG,  {30*5+10, 16*1+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[12]},
		{FIG,  {30*6+10, 16*1+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[13]},

		{FIG,  {30*0+10, 16*2+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[14]},
		{FIG,  {30*1+10, 16*2+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[15]},
		{FIG,  {30*2+10, 16*2+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[16]},
		{FIG,  {30*3+10, 16*2+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[17]},
		{FIG,  {30*4+10, 16*2+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[18]},
		{FIG,  {30*5+10, 16*2+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[19]},
		{FIG,  {30*6+10, 16*2+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[20]},

		{FIG,  {30*0+10, 16*3+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[21]},
		{FIG,  {30*1+10, 16*3+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[22]},
		{FIG,  {30*2+10, 16*3+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[23]},
		{FIG,  {30*3+10, 16*3+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[24]},
		{FIG,  {30*4+10, 16*3+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[25]},
		{FIG,  {30*5+10, 16*3+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[26]},
		{FIG,  {30*6+10, 16*3+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[27]},

		{FIG,  {30*0+10, 16*4+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[28]},
		{FIG,  {30*1+10, 16*4+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[29]},
		{FIG,  {30*2+10, 16*4+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[30]},
		{FIG,  {30*3+10, 16*4+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[31]},
		{FIG,  {30*4+10, 16*4+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[32]},
		{FIG,  {30*5+10, 16*4+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[33]},
		{FIG,  {30*6+10, 16*4+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[34]},

		{FIG,  {30*0+10, 16*5+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[35]},
		{FIG,  {30*1+10, 16*5+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[36]},
		{FIG,  {30*2+10, 16*5+42},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispMonthDetailScreen.energyStr[37]},
	},
	{FIG,  {200, 16*5+42}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "kWh"},
	{{FIG, {95, 130, 40, 18}}, {LCD_COLOR_WHITE, LCD_COLOR_GRAY}, "Close",  {0, 0}, dispCloseMonthDetailScreen},
};

// ********** Year ***********
#define YEAR_BOX_W		8
#define YEAR_BOX_H		BODY_H
#define YEAR_LABEL_W	(BODY_W - YEAR_BOX_W)
#define YEAR_LABEL_H	10
#define YEAR_CHART_W	YEAR_LABEL_W
#define YEAR_CHART_H	(BODY_H - YEAR_LABEL_H)

struct {
	struct lcd_segmentGroup			group;
	struct lcd_segmentRect			bg;
	struct lcd_segmentText			label[12];
	struct lcd_segmentRect			box[CH_COUNT];
	struct lcd_segmentBarChart		chart[12];
	struct lcd_segmentBarChart		now;
	struct lcd_segmentStr			scale;
	struct lcd_segmentStr			unit;

	unsigned short					color[CH_COUNT];
	float							value[12][CH_COUNT];
	float							nowValue[CH_COUNT];
	char							scaleStr[8];
}	dispYearGraphScreen = {
	{FIG, {0, 0, BODY_W, BODY_H}, dispShowYearDetailScreen, NULL},
	{FIG, {0, 0, BODY_W, BODY_H}, LCD_COLOR_BLACK},
	{
		{FIG, {YEAR_LABEL_W/12*0+YEAR_BOX_W, 2, YEAR_LABEL_W/12, YEAR_LABEL_H}, {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, {1,1}, "1"},
		{FIG, {YEAR_LABEL_W/12*1+YEAR_BOX_W, 2, YEAR_LABEL_W/12, YEAR_LABEL_H}, {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, {1,1}, "2"},
		{FIG, {YEAR_LABEL_W/12*2+YEAR_BOX_W, 2, YEAR_LABEL_W/12, YEAR_LABEL_H}, {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, {1,1}, "3"},
		{FIG, {YEAR_LABEL_W/12*3+YEAR_BOX_W, 2, YEAR_LABEL_W/12, YEAR_LABEL_H}, {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, {1,1}, "4"},
		{FIG, {YEAR_LABEL_W/12*4+YEAR_BOX_W, 2, YEAR_LABEL_W/12, YEAR_LABEL_H}, {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, {1,1}, "5"},
		{FIG, {YEAR_LABEL_W/12*5+YEAR_BOX_W, 2, YEAR_LABEL_W/12, YEAR_LABEL_H}, {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, {1,1}, "6"},
		{FIG, {YEAR_LABEL_W/12*6+YEAR_BOX_W, 2, YEAR_LABEL_W/12, YEAR_LABEL_H}, {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, {1,1}, "7"},
		{FIG, {YEAR_LABEL_W/12*7+YEAR_BOX_W, 2, YEAR_LABEL_W/12, YEAR_LABEL_H}, {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, {1,1}, "8"},
		{FIG, {YEAR_LABEL_W/12*8+YEAR_BOX_W, 2, YEAR_LABEL_W/12, YEAR_LABEL_H}, {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, {1,1}, "9"},
		{FIG, {YEAR_LABEL_W/12*9+YEAR_BOX_W, 2, YEAR_LABEL_W/12, YEAR_LABEL_H}, {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, {1,1}, "10"},
		{FIG, {YEAR_LABEL_W/12*10+YEAR_BOX_W,2, YEAR_LABEL_W/12, YEAR_LABEL_H}, {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, {1,1}, "11"},
		{FIG, {YEAR_LABEL_W/12*11+YEAR_BOX_W,2, YEAR_LABEL_W/12, YEAR_LABEL_H}, {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, {1,1}, "12"},
	},
	{
			{FIG, {0, YEAR_BOX_H/CH_COUNT*0, YEAR_BOX_W/2, YEAR_BOX_H/CH_COUNT}, LCD_COLOR_ORANGE},
			{FIG, {0, YEAR_BOX_H/CH_COUNT*1, YEAR_BOX_W/2, YEAR_BOX_H/CH_COUNT}, LCD_COLOR_YELLOW},
			{FIG, {0, YEAR_BOX_H/CH_COUNT*2, YEAR_BOX_W/2, YEAR_BOX_H/CH_COUNT}, LCD_COLOR_LIGHTGREEN},
			{FIG, {0, YEAR_BOX_H/CH_COUNT*3, YEAR_BOX_W/2, YEAR_BOX_H/CH_COUNT}, LCD_COLOR_CYAN},
			{FIG, {0, YEAR_BOX_H/CH_COUNT*4, YEAR_BOX_W/2, YEAR_BOX_H/CH_COUNT}, LCD_COLOR_ROYALBLUE},
			{FIG, {0, YEAR_BOX_H/CH_COUNT*5, YEAR_BOX_W/2, YEAR_BOX_H/CH_COUNT}, LCD_COLOR_PINK},
			{FIG, {0, YEAR_BOX_H/CH_COUNT*6, YEAR_BOX_W/2, YEAR_BOX_H/CH_COUNT}, LCD_COLOR_WHITE},
	},
	{
			{FIG, {YEAR_CHART_W/12*0+YEAR_BOX_W, YEAR_LABEL_H, YEAR_CHART_W/12, YEAR_CHART_H}, LCD_COLOR_BLACK, {1}, CH_COUNT, dispYearGraphScreen.value[0], dispYearGraphScreen.color},
			{FIG, {YEAR_CHART_W/12*1+YEAR_BOX_W, YEAR_LABEL_H, YEAR_CHART_W/12, YEAR_CHART_H}, LCD_COLOR_BLACK, {1}, CH_COUNT, dispYearGraphScreen.value[1], dispYearGraphScreen.color},
			{FIG, {YEAR_CHART_W/12*2+YEAR_BOX_W, YEAR_LABEL_H, YEAR_CHART_W/12, YEAR_CHART_H}, LCD_COLOR_BLACK, {1}, CH_COUNT, dispYearGraphScreen.value[2], dispYearGraphScreen.color},
			{FIG, {YEAR_CHART_W/12*3+YEAR_BOX_W, YEAR_LABEL_H, YEAR_CHART_W/12, YEAR_CHART_H}, LCD_COLOR_BLACK, {1}, CH_COUNT, dispYearGraphScreen.value[3], dispYearGraphScreen.color},
			{FIG, {YEAR_CHART_W/12*4+YEAR_BOX_W, YEAR_LABEL_H, YEAR_CHART_W/12, YEAR_CHART_H}, LCD_COLOR_BLACK, {1}, CH_COUNT, dispYearGraphScreen.value[4], dispYearGraphScreen.color},
			{FIG, {YEAR_CHART_W/12*5+YEAR_BOX_W, YEAR_LABEL_H, YEAR_CHART_W/12, YEAR_CHART_H}, LCD_COLOR_BLACK, {1}, CH_COUNT, dispYearGraphScreen.value[5], dispYearGraphScreen.color},
			{FIG, {YEAR_CHART_W/12*6+YEAR_BOX_W, YEAR_LABEL_H, YEAR_CHART_W/12, YEAR_CHART_H}, LCD_COLOR_BLACK, {1}, CH_COUNT, dispYearGraphScreen.value[6], dispYearGraphScreen.color},
			{FIG, {YEAR_CHART_W/12*7+YEAR_BOX_W, YEAR_LABEL_H, YEAR_CHART_W/12, YEAR_CHART_H}, LCD_COLOR_BLACK, {1}, CH_COUNT, dispYearGraphScreen.value[7], dispYearGraphScreen.color},
			{FIG, {YEAR_CHART_W/12*8+YEAR_BOX_W, YEAR_LABEL_H, YEAR_CHART_W/12, YEAR_CHART_H}, LCD_COLOR_BLACK, {1}, CH_COUNT, dispYearGraphScreen.value[8], dispYearGraphScreen.color},
			{FIG, {YEAR_CHART_W/12*9+YEAR_BOX_W, YEAR_LABEL_H, YEAR_CHART_W/12, YEAR_CHART_H}, LCD_COLOR_BLACK, {1}, CH_COUNT, dispYearGraphScreen.value[9], dispYearGraphScreen.color},
			{FIG, {YEAR_CHART_W/12*10+YEAR_BOX_W,YEAR_LABEL_H, YEAR_CHART_W/12, YEAR_CHART_H}, LCD_COLOR_BLACK, {1}, CH_COUNT, dispYearGraphScreen.value[10], dispYearGraphScreen.color},
			{FIG, {YEAR_CHART_W/12*11+YEAR_BOX_W,YEAR_LABEL_H, YEAR_CHART_W/12, YEAR_CHART_H}, LCD_COLOR_BLACK, {1}, CH_COUNT, dispYearGraphScreen.value[11], dispYearGraphScreen.color},
	},
	{FIG, {0, YEAR_LABEL_H, YEAR_CHART_W/12/2, YEAR_CHART_H}, LCD_COLOR_GRAY, {1}, CH_COUNT, dispYearGraphScreen.nowValue, dispYearGraphScreen.color},
	{FIG,  {BODY_W-LCD_FONT_WIDTH*6, LCD_FONT_HEIGHT*3},  {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, dispYearGraphScreen.scaleStr},
	{FIG,  {BODY_W-LCD_FONT_WIDTH*3, LCD_FONT_HEIGHT*4},  {LCD_COLOR_WHITE, LCD_COLOR_BLACK}, "kWh"},

	{LCD_COLOR_ORANGE, LCD_COLOR_YELLOW, LCD_COLOR_LIGHTGREEN, LCD_COLOR_CYAN, LCD_COLOR_ROYALBLUE, LCD_COLOR_PINK, LCD_COLOR_WHITE},
};

// ****** Year sub screen *******

struct {
	struct lcd_segmentGroup			group;
	struct lcd_segmentRect			bg;
	struct lcd_segmentTextField		title;
	struct lcd_segmentStr			label[12];
	struct lcd_segmentStr			energy[12];
	struct lcd_segmentStr			unit;
	struct lcd_segmentStr			overTimeLabel;
	struct lcd_segmentStr			overTime[DISP_OVER_CURRENT];
	struct lcd_segmentStr			overTimeLabel2;
	struct lcd_segmentButton		closeButton;
	struct lcd_segmentButton		resetButton;

	char							titleStr[16];
	char							energyStr[12][8];
	char							overTimeStr[DISP_OVER_CURRENT][16];
} dispYearDetailScreen = {
	{FIG, {50, 70, 140, 190}},
	{FIG, { 0,  0, 140, 190}, LCD_COLOR_LIGHTGRAY},
	{{FIG, {2, 2,  136, 16}}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_GRAY}, {1,1}, dispYearDetailScreen.titleStr, sizeof(dispYearDetailScreen.titleStr)},
	{
		{FIG,  {10, 10*0+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, " 1"},
		{FIG,  {10, 10*1+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, " 2"},
		{FIG,  {10, 10*2+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, " 3"},
		{FIG,  {10, 10*3+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, " 4"},
		{FIG,  {10, 10*4+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, " 5"},
		{FIG,  {10, 10*5+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, " 6"},
		{FIG,  {10, 10*6+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, " 7"},
		{FIG,  {10, 10*7+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, " 8"},
		{FIG,  {10, 10*8+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, " 9"},
		{FIG,  {10, 10*9+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "10"},
		{FIG,  {10, 10*10+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "11"},
		{FIG,  {10, 10*11+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "12"},
	},
	{
		{FIG,  {26, 10*0+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispYearDetailScreen.energyStr[0]},
		{FIG,  {26, 10*1+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispYearDetailScreen.energyStr[1]},
		{FIG,  {26, 10*2+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispYearDetailScreen.energyStr[2]},
		{FIG,  {26, 10*3+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispYearDetailScreen.energyStr[3]},
		{FIG,  {26, 10*4+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispYearDetailScreen.energyStr[4]},
		{FIG,  {26, 10*5+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispYearDetailScreen.energyStr[5]},
		{FIG,  {26, 10*6+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispYearDetailScreen.energyStr[6]},
		{FIG,  {26, 10*7+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispYearDetailScreen.energyStr[7]},
		{FIG,  {26, 10*8+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispYearDetailScreen.energyStr[8]},
		{FIG,  {26, 10*9+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispYearDetailScreen.energyStr[9]},
		{FIG,  {26, 10*10+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispYearDetailScreen.energyStr[10]},
		{FIG,  {26, 10*11+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispYearDetailScreen.energyStr[11]},
	},
	{FIG,      {38, 10*12+26}, {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "kWh"},
	{FIG,      {70, 10*0+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "Over time"},
	{
		{FIG,  {70, 10*1+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispYearDetailScreen.overTimeStr[0]},
		{FIG,  {70, 10*2+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispYearDetailScreen.overTimeStr[1]},
		{FIG,  {70, 10*3+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispYearDetailScreen.overTimeStr[2]},
		{FIG,  {70, 10*4+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispYearDetailScreen.overTimeStr[3]},
		{FIG,  {70, 10*5+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispYearDetailScreen.overTimeStr[4]},
		{FIG,  {70, 10*6+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispYearDetailScreen.overTimeStr[5]},
		{FIG,  {70, 10*7+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, dispYearDetailScreen.overTimeStr[6]},
	},
	{FIG,      {106, 10*8+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "sec"},
	{{FIG, {45, 160,     50, 18}}, {LCD_COLOR_WHITE, LCD_COLOR_GRAY}, "Close",  {0, 0}, dispCloseYearDetailScreen},
	{{FIG, {80, 10*9+26, 40, 18}}, {LCD_COLOR_WHITE, LCD_COLOR_GRAY}, "Reset",  {0, 0}, dispResetYearDetailScreen},
};

//	*** setup channel screen ***

struct {
	struct lcd_segmentGroup			group;
	struct lcd_segmentRect			bg;
	struct lcd_segmentTextField		title;
	struct lcd_segmentStr			l1;
	struct lcd_segmentPullDownField	sensorType;
	struct {
		struct lcd_segmentGroup			group;
		struct lcd_segmentStr			l11;
		struct lcd_segmentStr			l12;
		struct lcd_segmentStr			l13;
		struct lcd_segmentStr			l14;
		struct lcd_segmentStr			l15;
		struct lcd_segmentStr			l16;
		struct lcd_segmentStr			l17;
		struct lcd_segmentStr			l18;
		struct lcd_segmentStr			l19;
		struct lcd_segmentTextField		channelName;
		struct lcd_segmentIntegerField	ctN;
		struct lcd_segmentFloatField	ctK;
		struct lcd_segmentIntegerField	ctRL;
		struct lcd_segmentFloatField	offset;
		struct lcd_segmentPullDownField	voltSensNumber;
		struct lcd_segmentIntegerField	fixedVoltage;
		struct lcd_segmentIntegerField	fixedFrequency;
		struct lcd_segmentPullDownField	multiply;
	}								current;
	struct {
		struct lcd_segmentGroup			group;
		struct lcd_segmentStr			l21;
		struct lcd_segmentStr			l22;
		struct lcd_segmentStr			l23;
		struct lcd_segmentFloatField	voltSensR1;
		struct lcd_segmentFloatField	voltSensR2;
		struct lcd_segmentFloatField	offset;
	}								voltage;
	struct lcd_segmentButton		cancelButton;
	struct lcd_segmentButton		closeButton;

	char							*sensorTypeLabel[3];
	char							*voltSensorLabel[CH_COUNT+1];
	char							*multiplyLabel[3];
	char							titleStr[16];
	char							channelNameStr[16];
} dispSetupChannelScreen = {
	{FIG, {30, 20, 180, 270}},
	{FIG, { 0,  0, 180, 270}, LCD_COLOR_LIGHTGRAY},
	{{FIG, {2, 2,  176, 16}}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_GRAY}, {1,1}, dispSetupChannelScreen.titleStr, sizeof(dispSetupChannelScreen.titleStr)},
	{FIG,  {10,  30},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "Sensor type"},
	{{FIG, {110, 30,  60, 16}}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_GRAY}, dispSetupChannelScreen.sensorTypeLabel, 3, 0, dispChangeSetupChannelSensorType},
	{
		{FIG, {10, 50, 160, 180}},
		{FIG,  {  0, 20*0},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "Circuit name"},
		{FIG,  {  0, 20*1},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "CT Coil turn N"},
		{FIG,  {  0, 20*2},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "CT Char. K"},
		{FIG,  {  0, 20*3},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "CT Load reg. RL"},
		{FIG,  {  0, 20*4},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "Current offset"},
		{FIG,  {  0, 20*5},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "Volt sensor#"},
		{FIG,  {  0, 20*6},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "Fixed voltage"},
		{FIG,  {  0, 20*7},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "Fixed frequency"},
		{FIG,  {  0, 20*8},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "Multiply"},
		{{FIG, {100, 20*0,  60, 16}}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_GRAY}, {1,1}, dispSetupChannelScreen.channelNameStr, sizeof(dispSetupChannelScreen.channelNameStr)},
		{{FIG, {100, 20*1,  60, 16}}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_GRAY}, "%5d",       0,   32767, CT_N},
		{{FIG, {100, 20*2,  60, 16}}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_GRAY}, "%5.2f",     0,       2, CT_K},
		{{FIG, {100, 20*3,  60, 16}}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_GRAY}, "%5d ohm",   0,   32767, CT_RL},
		{{FIG, {100, 20*4,  60, 16}}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_GRAY}, "%5.3f A",  -1,       1, 0.0},
		{{FIG, {100, 20*5,  60, 16}}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_GRAY}, dispSetupChannelScreen.voltSensorLabel, CH_COUNT+1, 0, dispChangeSetupChannelVoltSensNumber},
		{{FIG, {100, 20*6,  60, 16}}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_GRAY}, "%5d V",     0,   32767, LINE_VOLT_DFLT},
		{{FIG, {100, 20*7,  60, 16}}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_GRAY}, "%5d Hz",    0,   32767, LINE_FREQ_DFLT},
		{{FIG, {100, 20*8,  60, 16}}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_GRAY}, dispSetupChannelScreen.multiplyLabel, 3, 0},
	},
	{
		{FIG, {10, 50, 160, 180}},
		{FIG,  {  0, 20*0},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "Volt sensor R1"},
		{FIG,  {  0, 20*1},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "Volt sensor R2"},
		{FIG,  {  0, 20*2},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "Volt offset"},
		{{FIG, {100, 20*0,  60, 16}}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_GRAY}, "%g ohm",    0, 1000000, LINE_VOLT_DIV_R1},
		{{FIG, {100, 20*1,  60, 16}}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_GRAY}, "%g ohm",    0, 1000000, LINE_VOLT_DIV_R2},
		{{FIG, {100, 20*2,  60, 16}}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_GRAY}, "%5.3f V",  -1,       1, 0.0},
	},
	{{FIG, {40,  50+180+10,  40, 18}}, {LCD_COLOR_WHITE, LCD_COLOR_GRAY}, "Cancel",  {0, 0}, dispCloseSetupChannelScreen},
	{{FIG, {100, 50+180+10,  40, 18}}, {LCD_COLOR_WHITE, LCD_COLOR_GRAY}, "Save",    {0, 0}, dispSaveSetupChannelScreen},
	{"None", "Current", "Volt"},
	{"CH1", "CH2", "CH3", "CH4", "CH5", "CH6", "CH7", "None"},
	{"x0.5", "x1", "x2"},
};

//	*** Setup date time screen ***

struct {
	struct lcd_segmentGroup			group;
	struct lcd_segmentRect			bg;
	struct lcd_segmentTextField		title;
	struct lcd_segmentStr			l1;
	struct lcd_segmentStr			l2;
	struct lcd_segmentStr			l3;
	struct lcd_segmentIntegerField	yy;
	struct lcd_segmentIntegerField	mm;
	struct lcd_segmentIntegerField	dd;
	struct lcd_segmentIntegerField	hh;
	struct lcd_segmentIntegerField	mi;
	struct lcd_segmentIntegerField	ss;
	struct lcd_segmentIntegerField	logInterval;
	struct lcd_segmentButton		cancel;
	struct lcd_segmentButton		save;
	struct lcd_segmentButton		epromSave;

	char							titleStr[16];
} dispSetupTimeScreen = {
	{FIG, {30, 20, 180, 140}},
	{FIG, {0,0,180,140}, LCD_COLOR_LIGHTGRAY},
	{{FIG, {2, 2,  176, 16}}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_GRAY}, {1,1}, dispSetupTimeScreen.titleStr, sizeof(dispSetupTimeScreen.titleStr)},
	{FIG,  {10,  20*0+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "Date"},
	{FIG,  {10,  20*1+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "Time"},
	{FIG,  {10,  20*2+26},  {LCD_COLOR_DIMGRAY, LCD_COLOR_LIGHTGRAY}, "Log interval"},
	{{FIG, {50,  20*0+26, 30, 14}}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_GRAY}, "%4d",   1970, 2045, 2014},
	{{FIG, {90,  20*0+26, 20, 14}}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_GRAY}, "%2d",   1, 12, 1},
	{{FIG, {120, 20*0+26, 20, 14}}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_GRAY}, "%2d",   1, 31, 1},
	{{FIG, {50,  20*1+26, 20, 14}}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_GRAY}, "%2d",   0, 23, 0},
	{{FIG, {80,  20*1+26, 20, 14}}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_GRAY}, "%2d",   0, 59, 0},
	{{FIG, {110, 20*1+26, 20, 14}}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_GRAY}, "%2d",   0, 59, 0},
	{{FIG, {80,  20*2+26, 30, 14}}, {LCD_COLOR_LIGHTGREEN, LCD_COLOR_GRAY}, "%d", 1, 3600, USB_MEMORY_LOG_INTERVAL},
	{{FIG, {35,  20*4+26, 40, 18}}, {LCD_COLOR_WHITE, LCD_COLOR_GRAY}, "Cancel",  {0, 0}, dispCloseSetupTimeScreen},
	{{FIG, {95,  20*4+26, 40, 18}}, {LCD_COLOR_WHITE, LCD_COLOR_GRAY}, "Save",    {0, 0}, dispSaveSetupTimeScreen},
	{{FIG, {120, 20*2+26, 50, 18}}, {LCD_COLOR_WHITE, LCD_COLOR_GRAY}, "EPROM",   {0, 0}, dispEpromSetupTimeScreen},
	"Setup",
};

const struct lcd_segmentFigureTable dispBatchRegistTable[] = {
		{LCD_SEGMENT_TYPE_GROUP,	NULL,  &dispRootScreen.root},
		{LCD_SEGMENT_TYPE_GROUP,	&dispRootScreen.root, &dispRootScreen.head},
		{LCD_SEGMENT_TYPE_GROUP,	&dispRootScreen.root, &dispRootScreen.left},
		{LCD_SEGMENT_TYPE_GROUP,	&dispRootScreen.root, &dispRootScreen.body},
		{LCD_SEGMENT_TYPE_GROUP,	&dispRootScreen.root, &dispRootScreen.foot},
		{LCD_SEGMENT_TYPE_RECT,		&dispRootScreen.head, &dispRootScreen.hbg},
		{LCD_SEGMENT_TYPE_RECT,		&dispRootScreen.head, &dispRootScreen.hBorder},
		{LCD_SEGMENT_TYPE_STR,		&dispRootScreen.head, &dispRootScreen.hname},
		{LCD_SEGMENT_TYPE_IMAGE,	&dispRootScreen.head, &dispRootScreen.hicon},
		{LCD_SEGMENT_TYPE_STR,		&dispRootScreen.head, &dispRootScreen.hdate},
		{LCD_SEGMENT_TYPE_STR,		&dispRootScreen.head, &dispRootScreen.htime},
		{LCD_SEGMENT_TYPE_STR,		&dispRootScreen.head, &dispRootScreen.hmsg},
		{LCD_SEGMENT_TYPE_RECT,		&dispRootScreen.left, &dispRootScreen.lBorder1},
		{LCD_SEGMENT_TYPE_RECT,		&dispRootScreen.left, &dispRootScreen.lBorder2[0]},
		{LCD_SEGMENT_TYPE_RECT,		&dispRootScreen.left, &dispRootScreen.lBorder2[1]},
		{LCD_SEGMENT_TYPE_RECT,		&dispRootScreen.left, &dispRootScreen.lBorder2[2]},
		{LCD_SEGMENT_TYPE_RECT,		&dispRootScreen.left, &dispRootScreen.lBorder2[3]},
		{LCD_SEGMENT_TYPE_RECT,		&dispRootScreen.left, &dispRootScreen.lBorder2[4]},
		{LCD_SEGMENT_TYPE_RECT,		&dispRootScreen.left, &dispRootScreen.lBorder2[5]},
		{LCD_SEGMENT_TYPE_RECT,		&dispRootScreen.left, &dispRootScreen.lBorder2[6]},
		{LCD_SEGMENT_TYPE_TEXT,		&dispRootScreen.left, &dispRootScreen.lineName[0]},
		{LCD_SEGMENT_TYPE_TEXT,		&dispRootScreen.left, &dispRootScreen.lineName[1]},
		{LCD_SEGMENT_TYPE_TEXT,		&dispRootScreen.left, &dispRootScreen.lineName[2]},
		{LCD_SEGMENT_TYPE_TEXT,		&dispRootScreen.left, &dispRootScreen.lineName[3]},
		{LCD_SEGMENT_TYPE_TEXT,		&dispRootScreen.left, &dispRootScreen.lineName[4]},
		{LCD_SEGMENT_TYPE_TEXT,		&dispRootScreen.left, &dispRootScreen.lineName[5]},
		{LCD_SEGMENT_TYPE_TEXT,		&dispRootScreen.left, &dispRootScreen.lineName[6]},
		{LCD_SEGMENT_TYPE_RECT,		&dispRootScreen.foot, &dispRootScreen.fbg},
		{LCD_SEGMENT_TYPE_BUTTON,	&dispRootScreen.foot, &dispRootScreen.nowButton},
		{LCD_SEGMENT_TYPE_BUTTON,	&dispRootScreen.foot, &dispRootScreen.todayButton},
		{LCD_SEGMENT_TYPE_BUTTON,	&dispRootScreen.foot, &dispRootScreen.monthButton},
		{LCD_SEGMENT_TYPE_BUTTON,	&dispRootScreen.foot, &dispRootScreen.yearButton},

		{LCD_SEGMENT_TYPE_GROUP,  &dispRootScreen.body, &dispNowDigestScreen.group},
		{LCD_SEGMENT_TYPE_GROUP,  &dispNowDigestScreen.group, &dispNowDigestScreen.ch[0].group},
		{LCD_SEGMENT_TYPE_RECT,   &dispNowDigestScreen.ch[0].group, &dispNowDigestScreen.ch[0].bg},
		{LCD_SEGMENT_TYPE_STR,    &dispNowDigestScreen.ch[0].group, &dispNowDigestScreen.ch[0].w},
		{LCD_SEGMENT_TYPE_STR,    &dispNowDigestScreen.ch[0].group, &dispNowDigestScreen.ch[0].kwh},
		{LCD_SEGMENT_TYPE_7SEG,   &dispNowDigestScreen.ch[0].group, &dispNowDigestScreen.ch[0].power},
		{LCD_SEGMENT_TYPE_7SEG,   &dispNowDigestScreen.ch[0].group, &dispNowDigestScreen.ch[0].energy},
		{LCD_SEGMENT_TYPE_GAGE,   &dispNowDigestScreen.ch[0].group, &dispNowDigestScreen.ch[0].gage},
		{LCD_SEGMENT_TYPE_GROUP,  &dispNowDigestScreen.group, &dispNowDigestScreen.ch[1].group},
		{LCD_SEGMENT_TYPE_RECT,   &dispNowDigestScreen.ch[1].group, &dispNowDigestScreen.ch[1].bg},
		{LCD_SEGMENT_TYPE_STR,    &dispNowDigestScreen.ch[1].group, &dispNowDigestScreen.ch[1].w},
		{LCD_SEGMENT_TYPE_STR,    &dispNowDigestScreen.ch[1].group, &dispNowDigestScreen.ch[1].kwh},
		{LCD_SEGMENT_TYPE_7SEG,   &dispNowDigestScreen.ch[1].group, &dispNowDigestScreen.ch[1].power},
		{LCD_SEGMENT_TYPE_7SEG,   &dispNowDigestScreen.ch[1].group, &dispNowDigestScreen.ch[1].energy},
		{LCD_SEGMENT_TYPE_GAGE,   &dispNowDigestScreen.ch[1].group, &dispNowDigestScreen.ch[1].gage},
		{LCD_SEGMENT_TYPE_GROUP,  &dispNowDigestScreen.group, &dispNowDigestScreen.ch[2].group},
		{LCD_SEGMENT_TYPE_RECT,   &dispNowDigestScreen.ch[2].group, &dispNowDigestScreen.ch[2].bg},
		{LCD_SEGMENT_TYPE_STR,    &dispNowDigestScreen.ch[2].group, &dispNowDigestScreen.ch[2].w},
		{LCD_SEGMENT_TYPE_STR,    &dispNowDigestScreen.ch[2].group, &dispNowDigestScreen.ch[2].kwh},
		{LCD_SEGMENT_TYPE_7SEG,   &dispNowDigestScreen.ch[2].group, &dispNowDigestScreen.ch[2].power},
		{LCD_SEGMENT_TYPE_7SEG,   &dispNowDigestScreen.ch[2].group, &dispNowDigestScreen.ch[2].energy},
		{LCD_SEGMENT_TYPE_GAGE,   &dispNowDigestScreen.ch[2].group, &dispNowDigestScreen.ch[2].gage},
		{LCD_SEGMENT_TYPE_GROUP,  &dispNowDigestScreen.group, &dispNowDigestScreen.ch[3].group},
		{LCD_SEGMENT_TYPE_RECT,   &dispNowDigestScreen.ch[3].group, &dispNowDigestScreen.ch[3].bg},
		{LCD_SEGMENT_TYPE_STR,    &dispNowDigestScreen.ch[3].group, &dispNowDigestScreen.ch[3].w},
		{LCD_SEGMENT_TYPE_STR,    &dispNowDigestScreen.ch[3].group, &dispNowDigestScreen.ch[3].kwh},
		{LCD_SEGMENT_TYPE_7SEG,   &dispNowDigestScreen.ch[3].group, &dispNowDigestScreen.ch[3].power},
		{LCD_SEGMENT_TYPE_7SEG,   &dispNowDigestScreen.ch[3].group, &dispNowDigestScreen.ch[3].energy},
		{LCD_SEGMENT_TYPE_GAGE,   &dispNowDigestScreen.ch[3].group, &dispNowDigestScreen.ch[3].gage},
		{LCD_SEGMENT_TYPE_GROUP,  &dispNowDigestScreen.group, &dispNowDigestScreen.ch[4].group},
		{LCD_SEGMENT_TYPE_RECT,   &dispNowDigestScreen.ch[4].group, &dispNowDigestScreen.ch[4].bg},
		{LCD_SEGMENT_TYPE_STR,    &dispNowDigestScreen.ch[4].group, &dispNowDigestScreen.ch[4].w},
		{LCD_SEGMENT_TYPE_STR,    &dispNowDigestScreen.ch[4].group, &dispNowDigestScreen.ch[4].kwh},
		{LCD_SEGMENT_TYPE_7SEG,   &dispNowDigestScreen.ch[4].group, &dispNowDigestScreen.ch[4].power},
		{LCD_SEGMENT_TYPE_7SEG,   &dispNowDigestScreen.ch[4].group, &dispNowDigestScreen.ch[4].energy},
		{LCD_SEGMENT_TYPE_GAGE,   &dispNowDigestScreen.ch[4].group, &dispNowDigestScreen.ch[4].gage},
		{LCD_SEGMENT_TYPE_GROUP,  &dispNowDigestScreen.group, &dispNowDigestScreen.ch[5].group},
		{LCD_SEGMENT_TYPE_RECT,   &dispNowDigestScreen.ch[5].group, &dispNowDigestScreen.ch[5].bg},
		{LCD_SEGMENT_TYPE_STR,    &dispNowDigestScreen.ch[5].group, &dispNowDigestScreen.ch[5].w},
		{LCD_SEGMENT_TYPE_STR,    &dispNowDigestScreen.ch[5].group, &dispNowDigestScreen.ch[5].kwh},
		{LCD_SEGMENT_TYPE_7SEG,   &dispNowDigestScreen.ch[5].group, &dispNowDigestScreen.ch[5].power},
		{LCD_SEGMENT_TYPE_7SEG,   &dispNowDigestScreen.ch[5].group, &dispNowDigestScreen.ch[5].energy},
		{LCD_SEGMENT_TYPE_GAGE,   &dispNowDigestScreen.ch[5].group, &dispNowDigestScreen.ch[5].gage},
		{LCD_SEGMENT_TYPE_GROUP,  &dispNowDigestScreen.group, &dispNowDigestScreen.ch[6].group},
		{LCD_SEGMENT_TYPE_RECT,   &dispNowDigestScreen.ch[6].group, &dispNowDigestScreen.ch[6].bg},
		{LCD_SEGMENT_TYPE_STR,    &dispNowDigestScreen.ch[6].group, &dispNowDigestScreen.ch[6].w},
		{LCD_SEGMENT_TYPE_STR,    &dispNowDigestScreen.ch[6].group, &dispNowDigestScreen.ch[6].kwh},
		{LCD_SEGMENT_TYPE_7SEG,   &dispNowDigestScreen.ch[6].group, &dispNowDigestScreen.ch[6].power},
		{LCD_SEGMENT_TYPE_7SEG,   &dispNowDigestScreen.ch[6].group, &dispNowDigestScreen.ch[6].energy},
		{LCD_SEGMENT_TYPE_GAGE,   &dispNowDigestScreen.ch[6].group, &dispNowDigestScreen.ch[6].gage},

		{LCD_SEGMENT_TYPE_GROUP,         &dispRootScreen.root, &dispNowDetailScreen.group},
		{LCD_SEGMENT_TYPE_RECT,          &dispNowDetailScreen.group, &dispNowDetailScreen.bg},
		{LCD_SEGMENT_TYPE_TEXT_FIELD,    &dispNowDetailScreen.group, &dispNowDetailScreen.title},
		{LCD_SEGMENT_TYPE_STR,           &dispNowDetailScreen.group, &dispNowDetailScreen.l1},
		{LCD_SEGMENT_TYPE_STR,           &dispNowDetailScreen.group, &dispNowDetailScreen.l2},
		{LCD_SEGMENT_TYPE_STR,           &dispNowDetailScreen.group, &dispNowDetailScreen.l3},
		{LCD_SEGMENT_TYPE_STR,           &dispNowDetailScreen.group, &dispNowDetailScreen.l4},
		{LCD_SEGMENT_TYPE_STR,           &dispNowDetailScreen.group, &dispNowDetailScreen.l5},
		{LCD_SEGMENT_TYPE_STR,           &dispNowDetailScreen.group, &dispNowDetailScreen.l6},
		{LCD_SEGMENT_TYPE_STR,           &dispNowDetailScreen.group, &dispNowDetailScreen.l7},
		{LCD_SEGMENT_TYPE_STR,           &dispNowDetailScreen.group, &dispNowDetailScreen.l8},
		{LCD_SEGMENT_TYPE_STR,           &dispNowDetailScreen.group, &dispNowDetailScreen.l9},
		{LCD_SEGMENT_TYPE_FLOAT_FIELD,   &dispNowDetailScreen.group, &dispNowDetailScreen.energyOfMonth},
		{LCD_SEGMENT_TYPE_FLOAT_FIELD,   &dispNowDetailScreen.group, &dispNowDetailScreen.energyOfDay},
		{LCD_SEGMENT_TYPE_INTEGER_FIELD, &dispNowDetailScreen.group, &dispNowDetailScreen.effectivePower},
		{LCD_SEGMENT_TYPE_INTEGER_FIELD, &dispNowDetailScreen.group, &dispNowDetailScreen.apparentPower},
		{LCD_SEGMENT_TYPE_INTEGER_FIELD, &dispNowDetailScreen.group, &dispNowDetailScreen.powerFactor},
		{LCD_SEGMENT_TYPE_INTEGER_FIELD, &dispNowDetailScreen.group, &dispNowDetailScreen.voltage},
		{LCD_SEGMENT_TYPE_FLOAT_FIELD,   &dispNowDetailScreen.group, &dispNowDetailScreen.current},
		{LCD_SEGMENT_TYPE_INTEGER_FIELD, &dispNowDetailScreen.group, &dispNowDetailScreen.frequency},
		{LCD_SEGMENT_TYPE_RECT,          &dispNowDetailScreen.group, &dispNowDetailScreen.wave},
		{LCD_SEGMENT_TYPE_RECT,          &dispNowDetailScreen.group, &dispNowDetailScreen.line},
		{LCD_SEGMENT_TYPE_BUTTON,        &dispNowDetailScreen.group, &dispNowDetailScreen.setupButton},
		{LCD_SEGMENT_TYPE_BUTTON,        &dispNowDetailScreen.group, &dispNowDetailScreen.closeButton},

		{LCD_SEGMENT_TYPE_GROUP,		&dispRootScreen.body, &dispTodayGraphScreen.group},
		{LCD_SEGMENT_TYPE_GROUP,		&dispTodayGraphScreen.group, &dispTodayGraphScreen.ch[0].group},
		{LCD_SEGMENT_TYPE_GROUP,		&dispTodayGraphScreen.group, &dispTodayGraphScreen.ch[1].group},
		{LCD_SEGMENT_TYPE_GROUP,		&dispTodayGraphScreen.group, &dispTodayGraphScreen.ch[2].group},
		{LCD_SEGMENT_TYPE_GROUP,		&dispTodayGraphScreen.group, &dispTodayGraphScreen.ch[3].group},
		{LCD_SEGMENT_TYPE_GROUP,		&dispTodayGraphScreen.group, &dispTodayGraphScreen.ch[4].group},
		{LCD_SEGMENT_TYPE_GROUP,		&dispTodayGraphScreen.group, &dispTodayGraphScreen.ch[5].group},
		{LCD_SEGMENT_TYPE_GROUP,		&dispTodayGraphScreen.group, &dispTodayGraphScreen.ch[6].group},
		{LCD_SEGMENT_TYPE_STR,			&dispTodayGraphScreen.group, &dispTodayGraphScreen.s0},
		{LCD_SEGMENT_TYPE_STR,			&dispTodayGraphScreen.group, &dispTodayGraphScreen.s6},
		{LCD_SEGMENT_TYPE_STR,			&dispTodayGraphScreen.group, &dispTodayGraphScreen.s12},
		{LCD_SEGMENT_TYPE_STR,			&dispTodayGraphScreen.group, &dispTodayGraphScreen.s18},
		{LCD_SEGMENT_TYPE_STR,			&dispTodayGraphScreen.group, &dispTodayGraphScreen.unit},

		{LCD_SEGMENT_TYPE_GROUP,         &dispRootScreen.root, &dispTodayDetailScreen.group},
		{LCD_SEGMENT_TYPE_RECT,          &dispTodayDetailScreen.group, &dispTodayDetailScreen.bg},
		{LCD_SEGMENT_TYPE_TEXT_FIELD,    &dispTodayDetailScreen.group, &dispTodayDetailScreen.title},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.label[0]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.label[1]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.label[2]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.label[3]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.label[4]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.label[5]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.label[6]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.label[7]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.label[8]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.label[9]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.label[10]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.label[11]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.label[12]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.label[13]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.label[14]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.label[15]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.label[16]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.label[17]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.label[18]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.label[19]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.label[20]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.label[21]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.label[22]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.label[23]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.energy[0]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.energy[1]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.energy[2]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.energy[3]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.energy[4]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.energy[5]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.energy[6]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.energy[7]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.energy[8]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.energy[9]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.energy[10]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.energy[11]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.energy[12]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.energy[13]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.energy[14]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.energy[15]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.energy[16]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.energy[17]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.energy[18]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.energy[19]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.energy[20]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.energy[21]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.energy[22]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.energy[23]},
		{LCD_SEGMENT_TYPE_STR,           &dispTodayDetailScreen.group, &dispTodayDetailScreen.unit},
		{LCD_SEGMENT_TYPE_BUTTON,        &dispTodayDetailScreen.group, &dispTodayDetailScreen.closeButton},

		{LCD_SEGMENT_TYPE_GROUP,  &dispRootScreen.body, &dispMonthGraphScreen.group},
		{LCD_SEGMENT_TYPE_RECT,   &dispMonthGraphScreen.group, &dispMonthGraphScreen.bg},
		{LCD_SEGMENT_TYPE_GROUP,  &dispMonthGraphScreen.group, &dispMonthGraphScreen.ch[0].group},
		{LCD_SEGMENT_TYPE_RECT,   &dispMonthGraphScreen.ch[0].group, &dispMonthGraphScreen.ch[0].border},
		{LCD_SEGMENT_TYPE_GAGE,   &dispMonthGraphScreen.ch[0].group, &dispMonthGraphScreen.ch[0].now},
		{LCD_SEGMENT_TYPE_POLY,   &dispMonthGraphScreen.ch[0].group, &dispMonthGraphScreen.ch[0].graphLine},
		{LCD_SEGMENT_TYPE_STR,    &dispMonthGraphScreen.ch[0].group, &dispMonthGraphScreen.ch[0].scale},
		{LCD_SEGMENT_TYPE_GROUP,  &dispMonthGraphScreen.group, &dispMonthGraphScreen.ch[1].group},
		{LCD_SEGMENT_TYPE_RECT,   &dispMonthGraphScreen.ch[1].group, &dispMonthGraphScreen.ch[1].border},
		{LCD_SEGMENT_TYPE_GAGE,   &dispMonthGraphScreen.ch[1].group, &dispMonthGraphScreen.ch[1].now},
		{LCD_SEGMENT_TYPE_POLY,   &dispMonthGraphScreen.ch[1].group, &dispMonthGraphScreen.ch[1].graphLine},
		{LCD_SEGMENT_TYPE_STR,    &dispMonthGraphScreen.ch[1].group, &dispMonthGraphScreen.ch[1].scale},
		{LCD_SEGMENT_TYPE_GROUP,  &dispMonthGraphScreen.group, &dispMonthGraphScreen.ch[2].group},
		{LCD_SEGMENT_TYPE_RECT,   &dispMonthGraphScreen.ch[2].group, &dispMonthGraphScreen.ch[2].border},
		{LCD_SEGMENT_TYPE_GAGE,   &dispMonthGraphScreen.ch[2].group, &dispMonthGraphScreen.ch[2].now},
		{LCD_SEGMENT_TYPE_POLY,   &dispMonthGraphScreen.ch[2].group, &dispMonthGraphScreen.ch[2].graphLine},
		{LCD_SEGMENT_TYPE_STR,    &dispMonthGraphScreen.ch[2].group, &dispMonthGraphScreen.ch[2].scale},
		{LCD_SEGMENT_TYPE_GROUP,  &dispMonthGraphScreen.group, &dispMonthGraphScreen.ch[3].group},
		{LCD_SEGMENT_TYPE_RECT,   &dispMonthGraphScreen.ch[3].group, &dispMonthGraphScreen.ch[3].border},
		{LCD_SEGMENT_TYPE_GAGE,   &dispMonthGraphScreen.ch[3].group, &dispMonthGraphScreen.ch[3].now},
		{LCD_SEGMENT_TYPE_POLY,   &dispMonthGraphScreen.ch[3].group, &dispMonthGraphScreen.ch[3].graphLine},
		{LCD_SEGMENT_TYPE_STR,    &dispMonthGraphScreen.ch[3].group, &dispMonthGraphScreen.ch[3].scale},
		{LCD_SEGMENT_TYPE_GROUP,  &dispMonthGraphScreen.group, &dispMonthGraphScreen.ch[4].group},
		{LCD_SEGMENT_TYPE_RECT,   &dispMonthGraphScreen.ch[4].group, &dispMonthGraphScreen.ch[4].border},
		{LCD_SEGMENT_TYPE_GAGE,   &dispMonthGraphScreen.ch[4].group, &dispMonthGraphScreen.ch[4].now},
		{LCD_SEGMENT_TYPE_POLY,   &dispMonthGraphScreen.ch[4].group, &dispMonthGraphScreen.ch[4].graphLine},
		{LCD_SEGMENT_TYPE_STR,    &dispMonthGraphScreen.ch[4].group, &dispMonthGraphScreen.ch[4].scale},
		{LCD_SEGMENT_TYPE_GROUP,  &dispMonthGraphScreen.group, &dispMonthGraphScreen.ch[5].group},
		{LCD_SEGMENT_TYPE_RECT,   &dispMonthGraphScreen.ch[5].group, &dispMonthGraphScreen.ch[5].border},
		{LCD_SEGMENT_TYPE_GAGE,   &dispMonthGraphScreen.ch[5].group, &dispMonthGraphScreen.ch[5].now},
		{LCD_SEGMENT_TYPE_POLY,   &dispMonthGraphScreen.ch[5].group, &dispMonthGraphScreen.ch[5].graphLine},
		{LCD_SEGMENT_TYPE_STR,    &dispMonthGraphScreen.ch[5].group, &dispMonthGraphScreen.ch[5].scale},
		{LCD_SEGMENT_TYPE_GROUP,  &dispMonthGraphScreen.group, &dispMonthGraphScreen.ch[6].group},
		{LCD_SEGMENT_TYPE_RECT,   &dispMonthGraphScreen.ch[6].group, &dispMonthGraphScreen.ch[6].border},
		{LCD_SEGMENT_TYPE_GAGE,   &dispMonthGraphScreen.ch[6].group, &dispMonthGraphScreen.ch[6].now},
		{LCD_SEGMENT_TYPE_POLY,   &dispMonthGraphScreen.ch[6].group, &dispMonthGraphScreen.ch[6].graphLine},
		{LCD_SEGMENT_TYPE_STR,    &dispMonthGraphScreen.ch[6].group, &dispMonthGraphScreen.ch[6].scale},
		{LCD_SEGMENT_TYPE_STR,    &dispMonthGraphScreen.group, &dispMonthGraphScreen.s1},
		{LCD_SEGMENT_TYPE_STR,    &dispMonthGraphScreen.group, &dispMonthGraphScreen.s10},
		{LCD_SEGMENT_TYPE_STR,    &dispMonthGraphScreen.group, &dispMonthGraphScreen.s20},
		{LCD_SEGMENT_TYPE_STR,    &dispMonthGraphScreen.group, &dispMonthGraphScreen.s30},
		{LCD_SEGMENT_TYPE_STR,    &dispMonthGraphScreen.group, &dispMonthGraphScreen.unit},

		{LCD_SEGMENT_TYPE_GROUP,         &dispRootScreen.root, &dispMonthDetailScreen.group},
		{LCD_SEGMENT_TYPE_RECT,          &dispMonthDetailScreen.group, &dispMonthDetailScreen.bg},
		{LCD_SEGMENT_TYPE_TEXT_FIELD,    &dispMonthDetailScreen.group, &dispMonthDetailScreen.title},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.label[0]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.label[1]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.label[2]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.label[3]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.label[4]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.label[5]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.label[6]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[0]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[1]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[2]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[3]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[4]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[5]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[6]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[7]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[8]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[9]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[10]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[11]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[12]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[13]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[14]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[15]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[16]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[17]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[18]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[19]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[20]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[21]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[22]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[23]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[24]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[25]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[26]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[27]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[28]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[29]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[30]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[31]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[32]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[33]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[34]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[35]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[36]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.energy[37]},
		{LCD_SEGMENT_TYPE_STR,           &dispMonthDetailScreen.group, &dispMonthDetailScreen.unit},
		{LCD_SEGMENT_TYPE_BUTTON,        &dispMonthDetailScreen.group, &dispMonthDetailScreen.closeButton},

		{LCD_SEGMENT_TYPE_GROUP,		&dispRootScreen.body, &dispYearGraphScreen.group},
		{LCD_SEGMENT_TYPE_RECT,			&dispYearGraphScreen.group, &dispYearGraphScreen.bg},
		{LCD_SEGMENT_TYPE_RECT,			&dispYearGraphScreen.group, &dispYearGraphScreen.box[0]},
		{LCD_SEGMENT_TYPE_RECT,			&dispYearGraphScreen.group, &dispYearGraphScreen.box[1]},
		{LCD_SEGMENT_TYPE_RECT,			&dispYearGraphScreen.group, &dispYearGraphScreen.box[2]},
		{LCD_SEGMENT_TYPE_RECT,			&dispYearGraphScreen.group, &dispYearGraphScreen.box[3]},
		{LCD_SEGMENT_TYPE_RECT,			&dispYearGraphScreen.group, &dispYearGraphScreen.box[4]},
		{LCD_SEGMENT_TYPE_RECT,			&dispYearGraphScreen.group, &dispYearGraphScreen.box[5]},
		{LCD_SEGMENT_TYPE_RECT,			&dispYearGraphScreen.group, &dispYearGraphScreen.box[6]},
		{LCD_SEGMENT_TYPE_TEXT,			&dispYearGraphScreen.group, &dispYearGraphScreen.label[0]},
		{LCD_SEGMENT_TYPE_TEXT,			&dispYearGraphScreen.group, &dispYearGraphScreen.label[1]},
		{LCD_SEGMENT_TYPE_TEXT,			&dispYearGraphScreen.group, &dispYearGraphScreen.label[2]},
		{LCD_SEGMENT_TYPE_TEXT,			&dispYearGraphScreen.group, &dispYearGraphScreen.label[3]},
		{LCD_SEGMENT_TYPE_TEXT,			&dispYearGraphScreen.group, &dispYearGraphScreen.label[4]},
		{LCD_SEGMENT_TYPE_TEXT,			&dispYearGraphScreen.group, &dispYearGraphScreen.label[5]},
		{LCD_SEGMENT_TYPE_TEXT,			&dispYearGraphScreen.group, &dispYearGraphScreen.label[6]},
		{LCD_SEGMENT_TYPE_TEXT,			&dispYearGraphScreen.group, &dispYearGraphScreen.label[7]},
		{LCD_SEGMENT_TYPE_TEXT,			&dispYearGraphScreen.group, &dispYearGraphScreen.label[8]},
		{LCD_SEGMENT_TYPE_TEXT,			&dispYearGraphScreen.group, &dispYearGraphScreen.label[9]},
		{LCD_SEGMENT_TYPE_TEXT,			&dispYearGraphScreen.group, &dispYearGraphScreen.label[10]},
		{LCD_SEGMENT_TYPE_TEXT,			&dispYearGraphScreen.group, &dispYearGraphScreen.label[11]},
		{LCD_SEGMENT_TYPE_BARC,			&dispYearGraphScreen.group, &dispYearGraphScreen.chart[0]},
		{LCD_SEGMENT_TYPE_BARC,			&dispYearGraphScreen.group, &dispYearGraphScreen.chart[1]},
		{LCD_SEGMENT_TYPE_BARC,			&dispYearGraphScreen.group, &dispYearGraphScreen.chart[2]},
		{LCD_SEGMENT_TYPE_BARC,			&dispYearGraphScreen.group, &dispYearGraphScreen.chart[3]},
		{LCD_SEGMENT_TYPE_BARC,			&dispYearGraphScreen.group, &dispYearGraphScreen.chart[4]},
		{LCD_SEGMENT_TYPE_BARC,			&dispYearGraphScreen.group, &dispYearGraphScreen.chart[5]},
		{LCD_SEGMENT_TYPE_BARC,			&dispYearGraphScreen.group, &dispYearGraphScreen.chart[6]},
		{LCD_SEGMENT_TYPE_BARC,			&dispYearGraphScreen.group, &dispYearGraphScreen.chart[7]},
		{LCD_SEGMENT_TYPE_BARC,			&dispYearGraphScreen.group, &dispYearGraphScreen.chart[8]},
		{LCD_SEGMENT_TYPE_BARC,			&dispYearGraphScreen.group, &dispYearGraphScreen.chart[9]},
		{LCD_SEGMENT_TYPE_BARC,			&dispYearGraphScreen.group, &dispYearGraphScreen.chart[10]},
		{LCD_SEGMENT_TYPE_BARC,			&dispYearGraphScreen.group, &dispYearGraphScreen.chart[11]},
		{LCD_SEGMENT_TYPE_BARC,			&dispYearGraphScreen.group, &dispYearGraphScreen.now},
		{LCD_SEGMENT_TYPE_STR,			&dispYearGraphScreen.group, &dispYearGraphScreen.scale},
		{LCD_SEGMENT_TYPE_STR,			&dispYearGraphScreen.group, &dispYearGraphScreen.unit},

		{LCD_SEGMENT_TYPE_GROUP,         &dispRootScreen.root, &dispYearDetailScreen.group},
		{LCD_SEGMENT_TYPE_RECT,          &dispYearDetailScreen.group, &dispYearDetailScreen.bg},
		{LCD_SEGMENT_TYPE_TEXT_FIELD,    &dispYearDetailScreen.group, &dispYearDetailScreen.title},
		{LCD_SEGMENT_TYPE_STR,           &dispYearDetailScreen.group, &dispYearDetailScreen.label[0]},
		{LCD_SEGMENT_TYPE_STR,           &dispYearDetailScreen.group, &dispYearDetailScreen.label[1]},
		{LCD_SEGMENT_TYPE_STR,           &dispYearDetailScreen.group, &dispYearDetailScreen.label[2]},
		{LCD_SEGMENT_TYPE_STR,           &dispYearDetailScreen.group, &dispYearDetailScreen.label[3]},
		{LCD_SEGMENT_TYPE_STR,           &dispYearDetailScreen.group, &dispYearDetailScreen.label[4]},
		{LCD_SEGMENT_TYPE_STR,           &dispYearDetailScreen.group, &dispYearDetailScreen.label[5]},
		{LCD_SEGMENT_TYPE_STR,           &dispYearDetailScreen.group, &dispYearDetailScreen.label[6]},
		{LCD_SEGMENT_TYPE_STR,           &dispYearDetailScreen.group, &dispYearDetailScreen.label[7]},
		{LCD_SEGMENT_TYPE_STR,           &dispYearDetailScreen.group, &dispYearDetailScreen.label[8]},
		{LCD_SEGMENT_TYPE_STR,           &dispYearDetailScreen.group, &dispYearDetailScreen.label[9]},
		{LCD_SEGMENT_TYPE_STR,           &dispYearDetailScreen.group, &dispYearDetailScreen.label[10]},
		{LCD_SEGMENT_TYPE_STR,           &dispYearDetailScreen.group, &dispYearDetailScreen.label[11]},
		{LCD_SEGMENT_TYPE_STR,           &dispYearDetailScreen.group, &dispYearDetailScreen.energy[0]},
		{LCD_SEGMENT_TYPE_STR,           &dispYearDetailScreen.group, &dispYearDetailScreen.energy[1]},
		{LCD_SEGMENT_TYPE_STR,           &dispYearDetailScreen.group, &dispYearDetailScreen.energy[2]},
		{LCD_SEGMENT_TYPE_STR,           &dispYearDetailScreen.group, &dispYearDetailScreen.energy[3]},
		{LCD_SEGMENT_TYPE_STR,           &dispYearDetailScreen.group, &dispYearDetailScreen.energy[4]},
		{LCD_SEGMENT_TYPE_STR,           &dispYearDetailScreen.group, &dispYearDetailScreen.energy[5]},
		{LCD_SEGMENT_TYPE_STR,           &dispYearDetailScreen.group, &dispYearDetailScreen.energy[6]},
		{LCD_SEGMENT_TYPE_STR,           &dispYearDetailScreen.group, &dispYearDetailScreen.energy[7]},
		{LCD_SEGMENT_TYPE_STR,           &dispYearDetailScreen.group, &dispYearDetailScreen.energy[8]},
		{LCD_SEGMENT_TYPE_STR,           &dispYearDetailScreen.group, &dispYearDetailScreen.energy[9]},
		{LCD_SEGMENT_TYPE_STR,           &dispYearDetailScreen.group, &dispYearDetailScreen.energy[10]},
		{LCD_SEGMENT_TYPE_STR,           &dispYearDetailScreen.group, &dispYearDetailScreen.energy[11]},
		{LCD_SEGMENT_TYPE_STR,           &dispYearDetailScreen.group, &dispYearDetailScreen.unit},
		{LCD_SEGMENT_TYPE_STR,           &dispYearDetailScreen.group, &dispYearDetailScreen.overTimeLabel},
		{LCD_SEGMENT_TYPE_STR,           &dispYearDetailScreen.group, &dispYearDetailScreen.overTime[0]},
		{LCD_SEGMENT_TYPE_STR,           &dispYearDetailScreen.group, &dispYearDetailScreen.overTime[1]},
		{LCD_SEGMENT_TYPE_STR,           &dispYearDetailScreen.group, &dispYearDetailScreen.overTime[2]},
		{LCD_SEGMENT_TYPE_STR,           &dispYearDetailScreen.group, &dispYearDetailScreen.overTime[3]},
		{LCD_SEGMENT_TYPE_STR,           &dispYearDetailScreen.group, &dispYearDetailScreen.overTime[4]},
		{LCD_SEGMENT_TYPE_STR,           &dispYearDetailScreen.group, &dispYearDetailScreen.overTime[5]},
		{LCD_SEGMENT_TYPE_STR,           &dispYearDetailScreen.group, &dispYearDetailScreen.overTime[6]},
		{LCD_SEGMENT_TYPE_STR,           &dispYearDetailScreen.group, &dispYearDetailScreen.overTimeLabel2},
		{LCD_SEGMENT_TYPE_BUTTON,        &dispYearDetailScreen.group, &dispYearDetailScreen.closeButton},
		{LCD_SEGMENT_TYPE_BUTTON,        &dispYearDetailScreen.group, &dispYearDetailScreen.resetButton},

		{LCD_SEGMENT_TYPE_GROUP,         &dispRootScreen.root, &dispSetupChannelScreen.group},
		{LCD_SEGMENT_TYPE_RECT,          &dispSetupChannelScreen.group, &dispSetupChannelScreen.bg},
		{LCD_SEGMENT_TYPE_TEXT_FIELD,    &dispSetupChannelScreen.group, &dispSetupChannelScreen.title},
		{LCD_SEGMENT_TYPE_STR,           &dispSetupChannelScreen.group, &dispSetupChannelScreen.l1},
		{LCD_SEGMENT_TYPE_PULLDOWN,      &dispSetupChannelScreen.group, &dispSetupChannelScreen.sensorType},
		{LCD_SEGMENT_TYPE_GROUP,         &dispSetupChannelScreen.group, &dispSetupChannelScreen.current.group},
		{LCD_SEGMENT_TYPE_STR,           &dispSetupChannelScreen.current.group, &dispSetupChannelScreen.current.l11},
		{LCD_SEGMENT_TYPE_STR,           &dispSetupChannelScreen.current.group, &dispSetupChannelScreen.current.l12},
		{LCD_SEGMENT_TYPE_STR,           &dispSetupChannelScreen.current.group, &dispSetupChannelScreen.current.l13},
		{LCD_SEGMENT_TYPE_STR,           &dispSetupChannelScreen.current.group, &dispSetupChannelScreen.current.l14},
		{LCD_SEGMENT_TYPE_STR,           &dispSetupChannelScreen.current.group, &dispSetupChannelScreen.current.l15},
		{LCD_SEGMENT_TYPE_STR,           &dispSetupChannelScreen.current.group, &dispSetupChannelScreen.current.l16},
		{LCD_SEGMENT_TYPE_STR,           &dispSetupChannelScreen.current.group, &dispSetupChannelScreen.current.l17},
		{LCD_SEGMENT_TYPE_STR,           &dispSetupChannelScreen.current.group, &dispSetupChannelScreen.current.l18},
		{LCD_SEGMENT_TYPE_STR,           &dispSetupChannelScreen.current.group, &dispSetupChannelScreen.current.l19},
		{LCD_SEGMENT_TYPE_TEXT_FIELD,    &dispSetupChannelScreen.current.group, &dispSetupChannelScreen.current.channelName},
		{LCD_SEGMENT_TYPE_INTEGER_FIELD, &dispSetupChannelScreen.current.group, &dispSetupChannelScreen.current.ctN},
		{LCD_SEGMENT_TYPE_FLOAT_FIELD,   &dispSetupChannelScreen.current.group, &dispSetupChannelScreen.current.ctK},
		{LCD_SEGMENT_TYPE_INTEGER_FIELD, &dispSetupChannelScreen.current.group, &dispSetupChannelScreen.current.ctRL},
		{LCD_SEGMENT_TYPE_FLOAT_FIELD,   &dispSetupChannelScreen.current.group, &dispSetupChannelScreen.current.offset},
		{LCD_SEGMENT_TYPE_PULLDOWN,      &dispSetupChannelScreen.current.group, &dispSetupChannelScreen.current.voltSensNumber},
		{LCD_SEGMENT_TYPE_INTEGER_FIELD, &dispSetupChannelScreen.current.group, &dispSetupChannelScreen.current.fixedVoltage},
		{LCD_SEGMENT_TYPE_INTEGER_FIELD, &dispSetupChannelScreen.current.group, &dispSetupChannelScreen.current.fixedFrequency},
		{LCD_SEGMENT_TYPE_PULLDOWN,      &dispSetupChannelScreen.current.group, &dispSetupChannelScreen.current.multiply},
		{LCD_SEGMENT_TYPE_GROUP,         &dispSetupChannelScreen.group, &dispSetupChannelScreen.voltage.group},
		{LCD_SEGMENT_TYPE_STR,           &dispSetupChannelScreen.voltage.group, &dispSetupChannelScreen.voltage.l21},
		{LCD_SEGMENT_TYPE_STR,           &dispSetupChannelScreen.voltage.group, &dispSetupChannelScreen.voltage.l22},
		{LCD_SEGMENT_TYPE_STR,           &dispSetupChannelScreen.voltage.group, &dispSetupChannelScreen.voltage.l23},
		{LCD_SEGMENT_TYPE_FLOAT_FIELD,   &dispSetupChannelScreen.voltage.group, &dispSetupChannelScreen.voltage.voltSensR1},
		{LCD_SEGMENT_TYPE_FLOAT_FIELD,   &dispSetupChannelScreen.voltage.group, &dispSetupChannelScreen.voltage.voltSensR2},
		{LCD_SEGMENT_TYPE_FLOAT_FIELD,   &dispSetupChannelScreen.voltage.group, &dispSetupChannelScreen.voltage.offset},
		{LCD_SEGMENT_TYPE_BUTTON,        &dispSetupChannelScreen.group, &dispSetupChannelScreen.cancelButton},
		{LCD_SEGMENT_TYPE_BUTTON,        &dispSetupChannelScreen.group, &dispSetupChannelScreen.closeButton},

		{LCD_SEGMENT_TYPE_GROUP,         &dispRootScreen.root, &dispSetupTimeScreen.group},
		{LCD_SEGMENT_TYPE_RECT,          &dispSetupTimeScreen.group, &dispSetupTimeScreen.bg},
		{LCD_SEGMENT_TYPE_TEXT_FIELD,    &dispSetupTimeScreen.group, &dispSetupTimeScreen.title},
		{LCD_SEGMENT_TYPE_STR,           &dispSetupTimeScreen.group, &dispSetupTimeScreen.l1},
		{LCD_SEGMENT_TYPE_STR,           &dispSetupTimeScreen.group, &dispSetupTimeScreen.l2},
		{LCD_SEGMENT_TYPE_STR,           &dispSetupTimeScreen.group, &dispSetupTimeScreen.l3},
		{LCD_SEGMENT_TYPE_INTEGER_FIELD, &dispSetupTimeScreen.group, &dispSetupTimeScreen.yy},
		{LCD_SEGMENT_TYPE_INTEGER_FIELD, &dispSetupTimeScreen.group, &dispSetupTimeScreen.mm},
		{LCD_SEGMENT_TYPE_INTEGER_FIELD, &dispSetupTimeScreen.group, &dispSetupTimeScreen.dd},
		{LCD_SEGMENT_TYPE_INTEGER_FIELD, &dispSetupTimeScreen.group, &dispSetupTimeScreen.hh},
		{LCD_SEGMENT_TYPE_INTEGER_FIELD, &dispSetupTimeScreen.group, &dispSetupTimeScreen.mi},
		{LCD_SEGMENT_TYPE_INTEGER_FIELD, &dispSetupTimeScreen.group, &dispSetupTimeScreen.ss},
		{LCD_SEGMENT_TYPE_INTEGER_FIELD, &dispSetupTimeScreen.group, &dispSetupTimeScreen.logInterval},
		{LCD_SEGMENT_TYPE_BUTTON,        &dispSetupTimeScreen.group, &dispSetupTimeScreen.cancel},
		{LCD_SEGMENT_TYPE_BUTTON,        &dispSetupTimeScreen.group, &dispSetupTimeScreen.save},
		{LCD_SEGMENT_TYPE_BUTTON,        &dispSetupTimeScreen.group, &dispSetupTimeScreen.epromSave},

		{LCD_SEGMENT_TYPE_END}
};


//
//	Subroutines
//
void dispOffScreen(void){
	lcd_segmentSetHidden(&dispNowDigestScreen.group,    1);
	lcd_segmentSetHidden(&dispTodayGraphScreen.group,   1);
	lcd_segmentSetHidden(&dispMonthGraphScreen.group,   1);
	lcd_segmentSetHidden(&dispYearGraphScreen.group,    1);
	lcd_segmentSetHidden(&dispNowDetailScreen.group,    1);
	lcd_segmentSetHidden(&dispTodayDetailScreen.group,  1);
	lcd_segmentSetHidden(&dispMonthDetailScreen.group,  1);
	lcd_segmentSetHidden(&dispYearDetailScreen.group,   1);
	lcd_segmentSetHidden(&dispSetupChannelScreen.group, 1);
	lcd_segmentSetHidden(&dispSetupTimeScreen.group,    1);
	return;
}
void dispOpen(short *val, struct sensorData *sensorData, struct tm *tm){
	int	ch;
	int	i;

	dispData = sensorData;
	dispTm   = tm;
	printf("Open\n");	// LCD init

	lcd_segmentNewFigureTables((struct lcd_segmentFigureTable *)dispBatchRegistTable, 10000);

	for(ch = 0; ch < CH_COUNT; ch++) {
		lcd_segmentNewRect((struct lcd_segmentFigure *)&dispTodayGraphScreen.ch[ch].group, &dispTodayGraphScreen.ch[ch].border);
		for(i = 0; i < 24; i++) {
			lcd_segmentNewGage((struct lcd_segmentFigure *)&dispTodayGraphScreen.ch[ch].group, &dispTodayGraphScreen.ch[ch].gage[i]);
		}
		lcd_segmentNewGage((struct lcd_segmentFigure *)&dispTodayGraphScreen.ch[ch].group, &dispTodayGraphScreen.ch[ch].now);
		lcd_segmentNewStr((struct lcd_segmentFigure *)&dispTodayGraphScreen.ch[ch].group, &dispTodayGraphScreen.ch[ch].scale);
	}

	dispNowDetailScreen.title.grp.touchOutProg          = NULL;		// Inhibit input
	dispNowDetailScreen.voltage.grp.touchOutProg        = NULL;
	dispNowDetailScreen.current.grp.touchOutProg        = NULL;
	dispNowDetailScreen.frequency.grp.touchOutProg      = NULL;
	dispNowDetailScreen.effectivePower.grp.touchOutProg = NULL;
	dispNowDetailScreen.apparentPower.grp.touchOutProg  = NULL;
	dispNowDetailScreen.powerFactor.grp.touchOutProg    = NULL;
	dispNowDetailScreen.energyOfDay.grp.touchOutProg    = NULL;
	dispNowDetailScreen.energyOfMonth.grp.touchOutProg  = NULL;
	dispTodayDetailScreen.title.grp.touchOutProg        = NULL;
	dispMonthDetailScreen.title.grp.touchOutProg        = NULL;
	dispYearDetailScreen.title.grp.touchOutProg         = NULL;
	dispSetupChannelScreen.title.grp.touchOutProg       = NULL;
	dispSetupTimeScreen.title.grp.touchOutProg          = NULL;

	newWaveFigure(&dispNowDetailScreen.group, &dispNowDetailScreen.wave1);
	newWaveFigure(&dispNowDetailScreen.group, &dispNowDetailScreen.wave2);
	dispNowDetailScreen.wave1.val = val;
	dispNowDetailScreen.wave2.val = val;

	dispOffScreen();

	// Starting a screen
	for(ch = 0; ch < CH_COUNT; ch++) {
		switch(sensorData[ch].infType){
		case ct:
			strncpy(dispRootScreen.lineNameStr[ch], sensorData[ch].inf.current.setup.name, sizeof(dispRootScreen.lineNameStr[ch])-1);
			dispRootScreen.lineNameStr[ch][sizeof(dispRootScreen.lineNameStr[ch]-1)] = '\0';
			break;
		default:
			strcpy(dispRootScreen.lineNameStr[ch], "SUM");
			break;
		}
	}
	dispMode = DISP_TODAY_GRAPH;	// Dummy
	dispShowNowDigestScreen(NULL);
//	dispShowSetupTimeScreen(NULL);

	return;
}

// *** Now ***

void dispUpdateNowDigestScreen(void) {
	struct sensorData	*d;
	int	ch;
	int	sumPower;
	int	sumEnergy;

	sumPower  = 0;
	sumEnergy = 0;
	for(ch = 0, d = dispData; ch < CH_COUNT; ch++, d++) {
		if (d->infType == ct) {
			dispNowDigestScreen.ch[ch].power.value  = d->inf.current.w;	// W
			dispNowDigestScreen.ch[ch].energy.value = (d->inf.current.ws_mon / 3600 + 50) / 100;	// Ws -> 100Wh
			sumPower  += dispNowDigestScreen.ch[ch].power.value;
			sumEnergy += dispNowDigestScreen.ch[ch].energy.value;
		} else {
			dispNowDigestScreen.ch[ch].power.value  = sumPower;
			dispNowDigestScreen.ch[ch].energy.value = sumEnergy;
			sumPower  = 0;
			sumEnergy = 0;
		}
		dispNowDigestScreen.ch[ch].gage.value = log10((double)dispNowDigestScreen.ch[ch].power.value) / 4.0;
		if (dispNowDigestScreen.ch[ch].power.value < 3) {
			dispNowDigestScreen.ch[ch].gage.color.face = LCD_COLOR_ROYALBLUE;
		} else if (dispNowDigestScreen.ch[ch].power.value < 10) {
			dispNowDigestScreen.ch[ch].gage.color.face = LCD_COLOR_CYAN;
		} else if (dispNowDigestScreen.ch[ch].power.value < 30) {
			dispNowDigestScreen.ch[ch].gage.color.face = LCD_COLOR_LIGHTGREEN;
		} else if (dispNowDigestScreen.ch[ch].power.value < 100) {
			dispNowDigestScreen.ch[ch].gage.color.face = LCD_COLOR_LIME;
		} else if (dispNowDigestScreen.ch[ch].power.value < 300) {
			dispNowDigestScreen.ch[ch].gage.color.face = LCD_COLOR_YELLOW;
		} else if (dispNowDigestScreen.ch[ch].power.value < 1000) {
			dispNowDigestScreen.ch[ch].gage.color.face = LCD_COLOR_ORANGE;
		} else if (dispNowDigestScreen.ch[ch].power.value < 3000) {
			dispNowDigestScreen.ch[ch].gage.color.face = LCD_COLOR_RED;
		} else if (dispNowDigestScreen.ch[ch].power.value < 10000) {
			dispNowDigestScreen.ch[ch].gage.color.face = LCD_COLOR_PURPLE;
		} else {
			dispNowDigestScreen.ch[ch].gage.color.face = LCD_COLOR_BLUE;
		}
		lcd_segmentSetDirty(&dispNowDigestScreen.ch[ch].power);
		lcd_segmentSetDirty(&dispNowDigestScreen.ch[ch].energy);
		lcd_segmentSetDirty(&dispNowDigestScreen.ch[ch].gage);
	}
	return;
}
void dispShowNowDigestScreen(struct lcd_segmentButton *s){
	if ((dispMode == DISP_TODAY_GRAPH) ||
		(dispMode == DISP_MONTH_GRAPH) ||
		(dispMode == DISP_YEAR_GRAPH)) {

		dispOffScreen();
		dispUpdateNowDigestScreen();
		lcd_segmentSetHidden(&dispNowDigestScreen.group, 0);
		dispMode = DISP_NOW_DIGEST;
		dispRootScreen.nowButton.internal.text.color.face   = LCD_COLOR_LIME;
		dispRootScreen.todayButton.internal.text.color.face = LCD_COLOR_LIGHTGRAY;
		dispRootScreen.monthButton.internal.text.color.face = LCD_COLOR_LIGHTGRAY;
		dispRootScreen.yearButton.internal.text.color.face  = LCD_COLOR_LIGHTGRAY;
		lcd_segmentSetDirty(&dispRootScreen.nowButton.internal.text);
		lcd_segmentSetDirty(&dispRootScreen.todayButton.internal.text);
		lcd_segmentSetDirty(&dispRootScreen.monthButton.internal.text);
		lcd_segmentSetDirty(&dispRootScreen.yearButton.internal.text);
	}
	return;
}
void dispUpdateNowDetailScreen(void) {
	struct sensorData	*d;

	d = dispData + dispCh;
	sprintf(dispNowDetailScreen.titleStr, "%s now", dispRootScreen.lineNameStr[dispCh]);
	switch(d->infType){
	case ct:
		lcd_segmentSetIntegerFieldValue(&dispNowDetailScreen.voltage,        d->inf.current.v);
		lcd_segmentSetFloatFieldValue(  &dispNowDetailScreen.current,        d->inf.current.a / 1000.0);
		lcd_segmentSetIntegerFieldValue(&dispNowDetailScreen.frequency,      d->inf.current.f);
		lcd_segmentSetIntegerFieldValue(&dispNowDetailScreen.effectivePower, d->inf.current.w);
		lcd_segmentSetIntegerFieldValue(&dispNowDetailScreen.apparentPower,  (d->inf.current.v * d->inf.current.a + 500) / 1000);
		lcd_segmentSetIntegerFieldValue(&dispNowDetailScreen.powerFactor,    lcd_segmentGetIntegerFieldValue(&dispNowDetailScreen.effectivePower) * 100 / lcd_segmentGetIntegerFieldValue(&dispNowDetailScreen.apparentPower));
		lcd_segmentSetFloatFieldValue(  &dispNowDetailScreen.energyOfDay,    (d->inf.current.ws_day / 3600) / 1000.0);
		lcd_segmentSetFloatFieldValue(  &dispNowDetailScreen.energyOfMonth,  (d->inf.current.ws_mon / 3600) / 1000.0);
		lcd_segmentSetHidden(&dispNowDetailScreen.voltage,        0);
		lcd_segmentSetHidden(&dispNowDetailScreen.current,        0);
		lcd_segmentSetHidden(&dispNowDetailScreen.frequency,      0);
		lcd_segmentSetHidden(&dispNowDetailScreen.effectivePower, 0);
		lcd_segmentSetHidden(&dispNowDetailScreen.apparentPower,  0);
		lcd_segmentSetHidden(&dispNowDetailScreen.powerFactor,    0);
		lcd_segmentSetHidden(&dispNowDetailScreen.energyOfDay,    0);
		lcd_segmentSetHidden(&dispNowDetailScreen.energyOfMonth,  0);
		lcd_segmentSetHidden(&dispNowDetailScreen.wave1, 0);
		dispNowDetailScreen.wave1.offset = dispCh;
		if (d->inf.current.setup.vSenseChannel < CH_COUNT) {
			lcd_segmentSetHidden(&dispNowDetailScreen.wave2, 0);
			dispNowDetailScreen.wave2.offset = d->inf.current.setup.vSenseChannel;
		} else {
			lcd_segmentSetHidden(&dispNowDetailScreen.wave2, 1);
		}
		break;
	case vsense:
		lcd_segmentSetIntegerFieldValue(&dispNowDetailScreen.voltage,        d->inf.voltage.v);
		lcd_segmentSetFloatFieldValue(  &dispNowDetailScreen.current,        0.0);
		lcd_segmentSetIntegerFieldValue(&dispNowDetailScreen.frequency,      d->inf.voltage.f);
		lcd_segmentSetIntegerFieldValue(&dispNowDetailScreen.effectivePower, 0);
		lcd_segmentSetIntegerFieldValue(&dispNowDetailScreen.apparentPower,  0);
		lcd_segmentSetIntegerFieldValue(&dispNowDetailScreen.powerFactor,    0);
		lcd_segmentSetFloatFieldValue(  &dispNowDetailScreen.energyOfDay,    0.0);
		lcd_segmentSetFloatFieldValue(  &dispNowDetailScreen.energyOfMonth,  0.0);
		lcd_segmentSetHidden(&dispNowDetailScreen.voltage,        0);
		lcd_segmentSetHidden(&dispNowDetailScreen.current,        1);
		lcd_segmentSetHidden(&dispNowDetailScreen.frequency,      0);
		lcd_segmentSetHidden(&dispNowDetailScreen.effectivePower, 1);
		lcd_segmentSetHidden(&dispNowDetailScreen.apparentPower,  1);
		lcd_segmentSetHidden(&dispNowDetailScreen.powerFactor,    1);
		lcd_segmentSetHidden(&dispNowDetailScreen.energyOfDay,    1);
		lcd_segmentSetHidden(&dispNowDetailScreen.energyOfMonth,  1);
		lcd_segmentSetHidden(&dispNowDetailScreen.wave1, 1);
		lcd_segmentSetHidden(&dispNowDetailScreen.wave2, 0);
		dispNowDetailScreen.wave2.offset = dispCh;
		lcd_segmentSetDirty(&dispNowDetailScreen.wave2);
		break;
	default:
		lcd_segmentSetHidden(&dispNowDetailScreen.voltage,        1);
		lcd_segmentSetHidden(&dispNowDetailScreen.current,        1);
		lcd_segmentSetHidden(&dispNowDetailScreen.frequency,      1);
		lcd_segmentSetHidden(&dispNowDetailScreen.effectivePower, 1);
		lcd_segmentSetHidden(&dispNowDetailScreen.apparentPower,  1);
		lcd_segmentSetHidden(&dispNowDetailScreen.powerFactor,    1);
		lcd_segmentSetHidden(&dispNowDetailScreen.energyOfDay,    1);
		lcd_segmentSetHidden(&dispNowDetailScreen.energyOfMonth,  1);
		lcd_segmentSetHidden(&dispNowDetailScreen.wave1, 1);
		lcd_segmentSetHidden(&dispNowDetailScreen.wave2, 1);
		break;
	}
	return;
}
void dispShowNowDetailScreen(struct lcd_segmentGroup *s, struct lcd_segmentCoordinate *touch){
	if (dispMode == DISP_NOW_DIGEST) {
		dispMode = DISP_NOW_DETAIL;
		dispCh   = touch->y * CH_COUNT / s->area.height;
		lcd_segmentSetHidden(&dispNowDetailScreen.group, 0);
		dispUpdateNowDetailScreen();
	}
	return;
}
void dispCloseNowDetailScreen(struct lcd_segmentButton *s){
	dispMode = DISP_NOW_DIGEST;
	lcd_segmentSetHidden(&dispNowDetailScreen.group, 1);
	return;
}

// *** Today ***

long dispTodayMaxValue[CH_COUNT];
void dispSetTodayGraphScreenScale(void) {
	struct sensorData	*d;
	int		ch;
	int		hh;
	long	max;
	long	sumEnergy[24];

	for(hh = 0; hh < 24; hh++) {
		sumEnergy[hh] = 0;
	}
	for(ch = 0, d = dispData; ch < CH_COUNT; ch++, d++) {
		max = 100 * 3600;				// Minimum scale = 100Wh
		for(hh = 0; hh < 24; hh++) {
			if (d->infType == ct) {
				max = MAX(max, d->inf.current.ws_hurs[hh]);
				sumEnergy[hh] += d->inf.current.ws_hurs[hh];
			} else {
				max = MAX(max, sumEnergy[hh]);
				d->inf.current.ws_hurs[hh] = sumEnergy[hh];
				sumEnergy[hh] = 0;
			}
		}
		dispTodayMaxValue[ch] = max;
	}
	return;
}
void dispUpdateTodayGraphScreen1(void) {
	struct sensorData	*d;
	int		ch;
	int		hh;

	for(ch = 0, d = dispData; ch < CH_COUNT; ch++, d++) {
		for(hh = 0; hh < 24; hh++) {
			dispTodayGraphScreen.ch[ch].gage[hh].value = (float)d->inf.current.ws_hurs[hh] / (float)dispTodayMaxValue[ch];
			lcd_segmentSetDirty(&dispTodayGraphScreen.ch[ch].gage[hh]);
		}
		sprintf(dispTodayGraphScreen.ch[ch].scaleStr, "%4.1f", dispTodayMaxValue[ch] / 3600 / 1000.0);	//kWh
		lcd_segmentSetDirty(&dispTodayGraphScreen.ch[ch].scale);
	}
	return;
}
void dispUpdateTodayGraphScreen2(void) {
	struct sensorData	*d;
	int		ch;
	long	sumEnergy;
	short	xPosition;
	int		recalc;

	recalc    = 0;
	sumEnergy = 0;
	xPosition = (BODY_W / 24) * dispTm->tm_hour;
	for(ch = 0, d = dispData; ch < CH_COUNT; ch++, d++) {
		long	value;

		if (d->infType == ct) {
			value      = (float)d->inf.current.ws_hur;
			sumEnergy += d->inf.current.ws_hur;
		} else {
			value      = sumEnergy;
			sumEnergy  = 0;
		}
		if (dispTodayMaxValue[ch] < value) {
			dispTodayMaxValue[ch] = (value / 3600 / 100 + 1) * 3600 * 100 ;		// Round up 100Wh
			recalc = 1;
		}
		dispTodayGraphScreen.ch[ch].now.value  = value / (float)dispTodayMaxValue[ch];
		dispTodayGraphScreen.ch[ch].now.area.x = xPosition;
		lcd_segmentSetDirty(&dispTodayGraphScreen.ch[ch].now);
	}
	if (recalc) {
		dispUpdateTodayGraphScreen1();
	}
	return;
}
void dispShowTodayGraphScreen(struct lcd_segmentButton *s){
	if ((dispMode == DISP_NOW_DIGEST)  ||
		(dispMode == DISP_MONTH_GRAPH) ||
		(dispMode == DISP_YEAR_GRAPH)) {

		dispOffScreen();
		dispMode = DISP_TODAY_GRAPH;
		lcd_segmentSetHidden(&dispTodayGraphScreen.group, 0);
		if (dispTodayMaxValue[0] == 0) {
			dispSetTodayGraphScreenScale();
			dispUpdateTodayGraphScreen1();
		}
		dispUpdateTodayGraphScreen2();
		dispRootScreen.nowButton.internal.text.color.face   = LCD_COLOR_LIGHTGRAY;
		dispRootScreen.todayButton.internal.text.color.face = LCD_COLOR_LIME;
		dispRootScreen.monthButton.internal.text.color.face = LCD_COLOR_LIGHTGRAY;
		dispRootScreen.yearButton.internal.text.color.face  = LCD_COLOR_LIGHTGRAY;
		lcd_segmentSetDirty(&dispRootScreen.nowButton.internal.text);
		lcd_segmentSetDirty(&dispRootScreen.todayButton.internal.text);
		lcd_segmentSetDirty(&dispRootScreen.monthButton.internal.text);
		lcd_segmentSetDirty(&dispRootScreen.yearButton.internal.text);
	}
	return;
}
void dispUpdateTodayDetailScreen(void) {
	struct sensorData	*d;
	int					i;

	d = dispData + dispCh;
	sprintf(dispTodayDetailScreen.titleStr, "%s today", dispRootScreen.lineNameStr[dispCh]);
	for(i = 0; i < 24; i++) {
		sprintf(dispTodayDetailScreen.energyStr[i], "%5.2f", d->inf.current.ws_hurs[i] / 3600 / 1000.0);
	}
	lcd_segmentSetDirty(dispTodayDetailScreen.energy);
	return;
}
void dispShowTodayDetailScreen(struct lcd_segmentGroup *s, struct lcd_segmentCoordinate *touch){
	if (dispMode == DISP_TODAY_GRAPH) {
		dispCh = touch->y * CH_COUNT / s->area.height;
		dispMode = DISP_TODAY_DETAIL;
		lcd_segmentSetHidden(&dispTodayDetailScreen.group, 0);
		dispUpdateTodayDetailScreen();
	}
	return;
}
void dispCloseTodayDetailScreen(struct lcd_segmentButton *s){
	dispMode = DISP_TODAY_GRAPH;
	lcd_segmentSetHidden(&dispTodayDetailScreen.group, 1);
	return;
}

// *** Month ***

long dispMonthMaxValue[CH_COUNT];
void dispSetMonthGraphScreenScale(void) {
	struct sensorData	*d;
	int		ch;
	int		dd;
	long	max;
	long	sumEnergy[32];

	for(dd = 1; dd <= 31; dd++) {
		sumEnergy[dd] = 0;
	}
	for(ch = 0, d = dispData; ch < CH_COUNT; ch++, d++) {
		max = 1000 * 3600;				// Minimum scale = 1kWh
		for(dd = 1; dd <= 31; dd++) {
			if (d->infType == ct) {
				max = MAX(max, d->inf.current.ws_days[dd]);
				sumEnergy[dd] += d->inf.current.ws_days[dd];
			} else {
				max = MAX(max, sumEnergy[dd]);
				d->inf.current.ws_days[dd] = sumEnergy[dd];
				sumEnergy[dd] = 0;
			}
		}
		dispMonthMaxValue[ch] = max;
	}
	return;
}
void dispUpdateMonthGraphScreen1(void) {
	struct sensorData	*d;
	int		ch;
	int		dd;
	short	x;
	short	y;
	int		f;

	for(ch = 0, d = dispData; ch < CH_COUNT; ch++, d++) {
		f = 0;
		x = 0;
		for(dd = 1; dd <= 31; dd++) {
			dispMonthGraphScreen.ch[ch].point[dd-1].x = x;
			y = (BODY_H_CH - 3) - (d->inf.current.ws_days[dd] * (BODY_H_CH - 3) / dispMonthMaxValue[ch]);
			if (y != dispMonthGraphScreen.ch[ch].point[dd-1].y) {
				f = 1;
				dispMonthGraphScreen.ch[ch].point[dd-1].y = y;
			}
			x += (BODY_W / 31);
		}
		if (f) {
			lcd_segmentSetDirty(&dispMonthGraphScreen.ch[ch].graphLine);
		}
		sprintf(dispMonthGraphScreen.ch[ch].scaleStr, "%4.1f", dispMonthMaxValue[ch] / 3600 / 1000.0);
		lcd_segmentSetDirty(&dispMonthGraphScreen.ch[ch].scale);
	}
	return;
}
void dispUpdateMonthGraphScreen2(void) {
	struct sensorData	*d;
	int		ch;
	long	sumEnergy;
	short	xPosition;
	int		recalc;

	recalc    = 0;
	sumEnergy = 0;
	xPosition = (BODY_W / 31) * (dispTm->tm_mday - 1);
	for(ch = 0, d = dispData; ch < CH_COUNT; ch++, d++) {
		long	value;

		if (d->infType == ct) {
			value      = d->inf.current.ws_day;
			sumEnergy += d->inf.current.ws_day;
		} else {
			value      = sumEnergy;
			sumEnergy  = 0;
		}
		if (dispMonthMaxValue[ch] < value) {		// Changing the scale
			dispMonthMaxValue[ch] = (value / 3600 / 1000 + 1) * 3600 * 1000 ;		// Round up 1kWh
			recalc = 1;
		}
		dispMonthGraphScreen.ch[ch].now.value  = (float)value / (float)dispMonthMaxValue[ch];
		dispMonthGraphScreen.ch[ch].now.area.x = xPosition;
		lcd_segmentSetDirty(&dispMonthGraphScreen.ch[ch].now);
	}
	if (recalc) {
		dispUpdateMonthGraphScreen1();
	}
	return;
}
void dispShowMonthGraphScreen(struct lcd_segmentButton *s){
	if ((dispMode == DISP_NOW_DIGEST) ||
		(dispMode == DISP_TODAY_GRAPH) ||
		(dispMode == DISP_YEAR_GRAPH)) {

		dispOffScreen();
		dispMode = DISP_MONTH_GRAPH;
		lcd_segmentSetHidden(&dispMonthGraphScreen.group, 0);
		if (dispMonthMaxValue[0] == 0) {
			dispSetMonthGraphScreenScale();
			dispUpdateMonthGraphScreen1();
		}
		dispUpdateMonthGraphScreen2();
		dispRootScreen.nowButton.internal.text.color.face   = LCD_COLOR_LIGHTGRAY;
		dispRootScreen.todayButton.internal.text.color.face = LCD_COLOR_LIGHTGRAY;
		dispRootScreen.monthButton.internal.text.color.face = LCD_COLOR_LIME;
		dispRootScreen.yearButton.internal.text.color.face  = LCD_COLOR_LIGHTGRAY;
		lcd_segmentSetDirty(&dispRootScreen.nowButton.internal.text);
		lcd_segmentSetDirty(&dispRootScreen.todayButton.internal.text);
		lcd_segmentSetDirty(&dispRootScreen.monthButton.internal.text);
		lcd_segmentSetDirty(&dispRootScreen.yearButton.internal.text);
	}
	return;
}
void dispUpdateMonthDetailScreen(void) {
	struct sensorData	*d;
	int			i;
	int			s;


	d = dispData + dispCh;
	s = (dispTm->tm_wday - dispTm->tm_mday) % 7;
	if (s < 0) {
		s += 7;
	}
	sprintf(dispMonthDetailScreen.titleStr, "%s this month", dispRootScreen.lineNameStr[dispCh]);
	for(i = 0; i < 38; i++) {
		strcpy(dispMonthDetailScreen.energyStr[i], "");
	}
	if (d->infType == ct) {
		for(i = 1; i <= 31; i++) {
			sprintf(dispMonthDetailScreen.energyStr[s++], "%4.1f%s", d->inf.current.ws_days[i] / 3600 / 1000.0, (i == dispTm->tm_mday) ? "*":"");
		}
	} else {
		for(i = 1; i <= 31; i++) {
			long	sumEnergy;
			int		k;

			sumEnergy = 0;
			for(k = 0, d = dispData; k < dispCh; k++, d++) {
				if (d->infType == ct) {
					sumEnergy += d->inf.current.ws_days[i];
				} else {
					sumEnergy = 0;
				}
			}
			sprintf(dispMonthDetailScreen.energyStr[s++], "%4.1f%s", sumEnergy / 3600 / 1000.0, (i == dispTm->tm_mday) ? "*":"");
		}
	}
	lcd_segmentSetDirty(dispMonthDetailScreen.energy);
	return;
}
void dispShowMonthDetailScreen(struct lcd_segmentGroup *s, struct lcd_segmentCoordinate *touch){
	if (dispMode == DISP_MONTH_GRAPH) {
		dispCh = touch->y * CH_COUNT / s->area.height;
		dispMode = DISP_MONTH_DETAIL;
		lcd_segmentSetHidden(&dispMonthDetailScreen.group, 0);	// Screen on
		dispUpdateMonthDetailScreen();
	}
	return;
}
void dispCloseMonthDetailScreen(struct lcd_segmentButton *s){
	dispMode = DISP_MONTH_GRAPH;
	lcd_segmentSetHidden(&dispMonthDetailScreen.group, 1);
	return;
}

// *** Year ***

long dispYearMaxValue;
void dispSetYearGraphScreenScale(void) {
	struct sensorData	*d;
	int		ch;
	int		mm;
	long	max;

	max = 3600 * 1000;			// Minimum scale = 1kWh
	for(mm = 0; mm < 12; mm++) {
		long	sum;

		sum = 0;
		for(ch = 0, d = dispData; ch < CH_COUNT; ch++, d++) {
			if (d->infType == ct) {
				sum += d->inf.current.ws_mons[mm];
				max = MAX(max, sum);
			} else {
				d->inf.current.ws_mons[mm] = sum;
				sum = 0;
			}
		}
	}
	dispYearMaxValue = max;
	return;
}
void dispUpdateYearGraphScreen1(void) {
	struct sensorData	*d;
	int		ch;
	int		mm;

	for(mm = 0; mm < 12; mm++) {
		for(ch = 0, d = dispData; ch < CH_COUNT; ch++, d++) {
			if (d->infType == ct) {
				dispYearGraphScreen.value[mm][ch] = (float)d->inf.current.ws_mons[mm] / (float)dispYearMaxValue;
			} else {
				dispYearGraphScreen.value[mm][ch] = 0;
			}
		}
		lcd_segmentSetDirty(&dispYearGraphScreen.chart[mm]);
	}
	sprintf(dispYearGraphScreen.scaleStr, "%4.1f", dispYearMaxValue / 3600 / 1000.0);
	lcd_segmentSetDirty(&dispYearGraphScreen.scale);
	return;
}
void dispUpdateYearGraphScreen2(void) {
	struct sensorData	*d;
	int		ch;
	long	sum;

	sum = 0;
	for(ch = 0, d = dispData; ch < CH_COUNT; ch++, d++) {
		sum += d->inf.current.ws_mon;
	}
	if (dispYearMaxValue < sum) {
		dispYearMaxValue = (sum / 3600 / 1000 + 1) * 3600 * 1000 ;		// Round up 1kWh
		dispUpdateYearGraphScreen1();
	}
	for(ch = 0, d = dispData; ch < CH_COUNT; ch++, d++) {
		dispYearGraphScreen.nowValue[ch] = (float)d->inf.current.ws_mon / (float)dispYearMaxValue;
	}
	dispYearGraphScreen.now.area.x = (YEAR_CHART_W / 12) * dispTm->tm_mon + YEAR_BOX_W;
	lcd_segmentSetDirty(&dispYearGraphScreen.now);
	return;
}
void dispShowYearGraphScreen(struct lcd_segmentButton *s){
	if ((dispMode == DISP_NOW_DIGEST)  ||
		(dispMode == DISP_TODAY_GRAPH)  ||
		(dispMode == DISP_MONTH_GRAPH)) {

		dispOffScreen();
		dispMode = DISP_YEAR_GRAPH;
		lcd_segmentSetHidden(&dispYearGraphScreen.group, 0);
		if (dispYearMaxValue == 0) {
			dispSetYearGraphScreenScale();
			dispUpdateYearGraphScreen1();
		}
		dispUpdateYearGraphScreen2();
		dispRootScreen.nowButton.internal.text.color.face   = LCD_COLOR_LIGHTGRAY;
		dispRootScreen.todayButton.internal.text.color.face = LCD_COLOR_LIGHTGRAY;
		dispRootScreen.monthButton.internal.text.color.face = LCD_COLOR_LIGHTGRAY;
		dispRootScreen.yearButton.internal.text.color.face  = LCD_COLOR_LIME;
		lcd_segmentSetDirty(&dispRootScreen.nowButton.internal.text);
		lcd_segmentSetDirty(&dispRootScreen.todayButton.internal.text);
		lcd_segmentSetDirty(&dispRootScreen.monthButton.internal.text);
		lcd_segmentSetDirty(&dispRootScreen.yearButton.internal.text);
	}
	return;
}
void dispUpdateYearDetailScreen(void) {
	struct sensorData	*d;
	int					i;

	d = dispData + dispCh;
	sprintf(dispYearDetailScreen.titleStr, "%s this year", dispRootScreen.lineNameStr[dispCh]);
	if (d->infType == ct) {
		for(i = 0; i < 12; i++) {
			sprintf(dispYearDetailScreen.energyStr[i], "%5.1f", d->inf.current.ws_mons[i] / 3600000.0);
		}
	} else {
		for(i = 0; i < 12; i++) {
			long	sumEnergy;
			int		k;

			sumEnergy = 0;
			for(k = 0, d = dispData; k < dispCh; k++, d++) {
				if (d->infType == ct) {
					sumEnergy += d->inf.current.ws_mons[i];
				} else {
					sumEnergy = 0;
				}
			}
			sprintf(dispYearDetailScreen.energyStr[i], "%5.1f", sumEnergy / 3600 / 1000.0);
		}
	}
	for(i = 0; i < DISP_OVER_CURRENT; i++) {
		sprintf(dispYearDetailScreen.overTimeStr[i], "%ldA%6ld", dispOverCurrent[i].check/1000, dispOverCurrent[i].sec);
	}
	lcd_segmentSetDirty(dispYearDetailScreen.energy);
	return;
}
void dispShowYearDetailScreen(struct lcd_segmentGroup *s, struct lcd_segmentCoordinate *touch){
	if (dispMode == DISP_YEAR_GRAPH) {
		dispCh   = touch->y * CH_COUNT / s->area.height;
		dispMode = DISP_YEAR_DETAIL;
		lcd_segmentSetHidden(&dispYearDetailScreen.group, 0);
		dispUpdateYearDetailScreen();
	}
	return;
}
void dispCloseYearDetailScreen(struct lcd_segmentButton *s){
	dispMode = DISP_YEAR_GRAPH;
	lcd_segmentSetHidden(&dispYearDetailScreen.group, 1);
	return;
}
void dispResetYearDetailScreen(struct lcd_segmentButton *s){
	int	i;

	for(i = 0; i < DISP_OVER_CURRENT; i++) {
		dispOverCurrent[i].sec = 0;
	}
	dispUpdateYearDetailScreen();
	return;
}

// Setup channel
void dispChangeSetupChannelVoltSensNumber(struct lcd_segmentPullDownField *f){
	if (lcd_segmentGetPullDownFieldSelect(&dispSetupChannelScreen.current.voltSensNumber) < CH_COUNT) {
		lcd_segmentSetHidden(&dispSetupChannelScreen.current.multiply,       0);
		lcd_segmentSetHidden(&dispSetupChannelScreen.current.fixedVoltage,   1);
		lcd_segmentSetHidden(&dispSetupChannelScreen.current.fixedFrequency, 1);
	} else {
		lcd_segmentSetHidden(&dispSetupChannelScreen.current.multiply,       1);
		lcd_segmentSetHidden(&dispSetupChannelScreen.current.fixedVoltage,   0);
		lcd_segmentSetHidden(&dispSetupChannelScreen.current.fixedFrequency, 0);
	}
	return;
}
void dispChangeSetupChannelSensorType(struct lcd_segmentPullDownField *f){
	switch(lcd_segmentGetPullDownFieldSelect(&dispSetupChannelScreen.sensorType)) {
	case ct:
		lcd_segmentSetHidden(&dispSetupChannelScreen.current.group, 0);
		lcd_segmentSetHidden(&dispSetupChannelScreen.voltage.group, 1);
		dispChangeSetupChannelVoltSensNumber(f);
		break;
	case vsense:
		lcd_segmentSetHidden(&dispSetupChannelScreen.current.group, 1);
		lcd_segmentSetHidden(&dispSetupChannelScreen.voltage.group, 0);
		break;
	default:
		lcd_segmentSetHidden(&dispSetupChannelScreen.current.group, 1);
		lcd_segmentSetHidden(&dispSetupChannelScreen.voltage.group, 1);
		break;
	}
	return;
}
void dispShowSetupChannelScreen(struct lcd_segmentButton *s){
	struct sensorData	*d;

	d = dispData + dispCh;
	lcd_segmentSetHidden(&dispNowDetailScreen.group, 1);
	lcd_segmentSetHidden(&dispSetupChannelScreen.group, 0);
	dispMode = DISP_SETUP_CHANNEL;
	sprintf(dispSetupChannelScreen.titleStr, "#%d Setup", dispCh+1);
	lcd_segmentSetPullDownFieldSelect(&dispSetupChannelScreen.sensorType, d->infType);
	if (d->infType == ct) {
		strcpy(dispSetupChannelScreen.channelNameStr, d->inf.current.setup.name);
		lcd_segmentSetIntegerFieldValue(  &dispSetupChannelScreen.current.ctN,            d->inf.current.setup.n);
		lcd_segmentSetFloatFieldValue(    &dispSetupChannelScreen.current.ctK,            d->inf.current.setup.k);
		lcd_segmentSetIntegerFieldValue(  &dispSetupChannelScreen.current.ctRL,           d->inf.current.setup.rl);
		lcd_segmentSetFloatFieldValue(    &dispSetupChannelScreen.current.offset,         d->inf.current.setup.offset);
		lcd_segmentSetPullDownFieldSelect(&dispSetupChannelScreen.current.voltSensNumber, d->inf.current.setup.vSenseChannel);
		lcd_segmentSetPullDownFieldSelect(&dispSetupChannelScreen.current.multiply,       d->inf.current.setup.vSenseMulti);
		lcd_segmentSetIntegerFieldValue(  &dispSetupChannelScreen.current.fixedVoltage,   d->inf.current.setup.v);
		lcd_segmentSetIntegerFieldValue(  &dispSetupChannelScreen.current.fixedFrequency, d->inf.current.setup.f);
	} else if (d->infType == vsense) {
		lcd_segmentSetFloatFieldValue(    &dispSetupChannelScreen.voltage.voltSensR1,     d->inf.voltage.setup.r1);
		lcd_segmentSetFloatFieldValue(    &dispSetupChannelScreen.voltage.voltSensR2,     d->inf.voltage.setup.r2);
		lcd_segmentSetFloatFieldValue(    &dispSetupChannelScreen.voltage.offset,         d->inf.voltage.setup.offset);
	}
	dispChangeSetupChannelSensorType(NULL);
	return;
}
void dispSaveSetupChannelScreen(struct lcd_segmentButton *s){
	struct sensorData	*d;

	d = dispData + dispCh;
	d->infType = lcd_segmentGetPullDownFieldSelect(&dispSetupChannelScreen.sensorType);
	switch(d->infType){
	case ct:
		strncpy(d->inf.current.setup.name, dispSetupChannelScreen.channelNameStr, sizeof(d->inf.current.setup.name));
		strcpy(dispRootScreen.lineNameStr[dispCh], dispSetupChannelScreen.channelNameStr);
		d->inf.current.setup.n             = lcd_segmentGetIntegerFieldValue(  &dispSetupChannelScreen.current.ctN);
		d->inf.current.setup.k             = lcd_segmentGetFloatFieldValue(    &dispSetupChannelScreen.current.ctK);
		d->inf.current.setup.rl            = lcd_segmentGetIntegerFieldValue(  &dispSetupChannelScreen.current.ctRL);
		d->inf.current.setup.offset        = lcd_segmentGetFloatFieldValue(    &dispSetupChannelScreen.current.offset);
		d->inf.current.setup.vSenseChannel = lcd_segmentGetPullDownFieldSelect(&dispSetupChannelScreen.current.voltSensNumber);
		d->inf.current.setup.vSenseMulti   = lcd_segmentGetPullDownFieldSelect(&dispSetupChannelScreen.current.multiply);
		d->inf.current.setup.v             = lcd_segmentGetIntegerFieldValue(  &dispSetupChannelScreen.current.fixedVoltage);
		d->inf.current.setup.f             = lcd_segmentGetIntegerFieldValue(  &dispSetupChannelScreen.current.fixedFrequency);
		break;
	case vsense:
		strcpy(dispRootScreen.lineNameStr[dispCh], "SUM");
		d->inf.voltage.setup.r1            = lcd_segmentGetFloatFieldValue(    &dispSetupChannelScreen.voltage.voltSensR1);
		d->inf.voltage.setup.r2            = lcd_segmentGetFloatFieldValue(    &dispSetupChannelScreen.voltage.voltSensR2);
		d->inf.voltage.setup.offset        = lcd_segmentGetFloatFieldValue(    &dispSetupChannelScreen.voltage.offset);
		break;
	default:
		break;
	}

	// Save to EPROM
	saveSetup(d, dispCh, dispTm);

	// Post update
	adcToValuePrepare(dispData);

	// Previous screen
	dispMode = DISP_NOW_DIGEST;
	lcd_segmentSetHidden(&dispSetupChannelScreen.group, 1);
	return;
}
void dispCloseSetupChannelScreen(struct lcd_segmentButton *s){
	dispMode = DISP_NOW_DIGEST;
	lcd_segmentSetHidden(&dispSetupChannelScreen.group, 1);
	return;
}

// *** Set date time ***
int	dispSaveDispMode;
void dispShowSetupTimeScreen(struct lcd_segmentGroup *s) {
	if (dispMode != DISP_SETUP_TIME) {
		dispSaveDispMode = dispMode;
		dispMode = DISP_SETUP_TIME;
		lcd_segmentSetHidden(&dispSetupTimeScreen.group, 0);
		lcd_segmentSetIntegerFieldValue(&dispSetupTimeScreen.yy, dispTm->tm_year + 1900);
		lcd_segmentSetIntegerFieldValue(&dispSetupTimeScreen.mm, dispTm->tm_mon + 1);
		lcd_segmentSetIntegerFieldValue(&dispSetupTimeScreen.dd, dispTm->tm_mday);
		lcd_segmentSetIntegerFieldValue(&dispSetupTimeScreen.hh, dispTm->tm_hour);
		lcd_segmentSetIntegerFieldValue(&dispSetupTimeScreen.mi, dispTm->tm_min);
		lcd_segmentSetIntegerFieldValue(&dispSetupTimeScreen.ss, dispTm->tm_sec);
		lcd_segmentSetIntegerFieldValue(&dispSetupTimeScreen.ss, usbMemoryLogInterval);
	}
	return;
}
void dispCloseSetupTimeScreen(struct lcd_segmentButton *s){
	dispMode = dispSaveDispMode;
	lcd_segmentSetHidden(&dispSetupTimeScreen.group, 1);
	return;
}
void dispSaveSetupTimeScreen(struct lcd_segmentButton *s){
	struct tm		tm;
	struct timeval	tv;

	tm.tm_year = lcd_segmentGetIntegerFieldValue(&dispSetupTimeScreen.yy) - 1900;
	tm.tm_mon  = lcd_segmentGetIntegerFieldValue(&dispSetupTimeScreen.mm) - 1;
	tm.tm_mday = lcd_segmentGetIntegerFieldValue(&dispSetupTimeScreen.dd);
	tm.tm_hour = lcd_segmentGetIntegerFieldValue(&dispSetupTimeScreen.hh);
	tm.tm_min  = lcd_segmentGetIntegerFieldValue(&dispSetupTimeScreen.mi);
	tm.tm_sec  = lcd_segmentGetIntegerFieldValue(&dispSetupTimeScreen.ss);
	tv.tv_sec  = mktime(&tm);
	tv.tv_usec = 0;
	settimeofday(&tv, NULL);
	usbMemoryLogInterval = lcd_segmentGetIntegerFieldValue(&dispSetupTimeScreen.logInterval);

	dispMode = dispSaveDispMode;
	lcd_segmentSetHidden(&dispSetupTimeScreen.group, 1);
	return;
}
void dispEpromSetupTimeScreen(struct lcd_segmentButton *s){
	usbMemorySaveRequest = 1;
	return;
}

//
//	Handler
//
void dispRefresh(void){
	static struct lcd_segmentCoordinate	base = {0,0};
	static int			before_ss;
	static int			before_hh;
	static int			before_dd;
	static int			before_mm;
	static int			speedCounter;
	struct bt_l2capLink	*bt;
	const unsigned short *icon;

	speedCounter++;
	if (before_ss != dispTm->tm_sec) {
		before_ss = dispTm->tm_sec;
		sprintf(dispRootScreen.dateStr, "%4d.%02d.%02d", dispTm->tm_year+1900, dispTm->tm_mon+1, dispTm->tm_mday);
		sprintf(dispRootScreen.timeStr, "%2d:%02d:%02d", dispTm->tm_hour, dispTm->tm_min, dispTm->tm_sec);
		bt = NULL;
		sprintf(dispRootScreen.msgStr, "%4.1f", (speedCounter / 1000.0));
		speedCounter = 0;
		lcd_segmentSetDirty(&dispRootScreen.hdate);
		lcd_segmentSetDirty(&dispRootScreen.htime);
		lcd_segmentSetDirty(&dispRootScreen.hmsg);
		if (usbDeviceReady) {
			switch(*usbDeviceClassName) {
			case 'W':
				if (bt_l2capLinkGetNext(&bt)) {
					icon = iconBluetoothConnect;
				} else {
					icon = iconBluetoothAdvertisement;
				}
				break;
			case 'S':
				if (usbMemoryStatus < 0) {
					icon = iconUsbError;
				} else if (usbMemoryStatus <= 2) {
					icon = iconUsbMemoryWrite;
				} else {
					icon = iconUsbMemory;
				}
				break;
			default:
				icon = iconUsbError;
				break;
			}
		} else {
			icon = iconUsbUnplug;
		}
		if (dispRootScreen.hicon.image != (unsigned short *)icon) {
			dispRootScreen.hicon.image = (unsigned short *)icon;
			lcd_segmentSetDirty(&dispRootScreen.hicon);
		}

		if (before_hh != dispTm->tm_hour) {
			before_hh = dispTm->tm_hour;
			dispSetTodayGraphScreenScale();
			dispUpdateTodayGraphScreen1();
		}
		if (before_dd != dispTm->tm_mday) {
			before_dd = dispTm->tm_mday;
			dispSetMonthGraphScreenScale();
			dispUpdateMonthGraphScreen1();
		}
		if (before_mm != dispTm->tm_mon) {
			before_mm = dispTm->tm_mon;
			dispSetYearGraphScreenScale();
			dispUpdateYearGraphScreen1();
		}

		{
			int					i;
			struct sensorData	*d;
			long				a;

			a = 0;
			for(i = 0, d = dispData; i < CH_COUNT; i++, d++) {
				switch(d->infType){
				case ct:
					a += d->inf.current.a;
					break;
				case vsense:
					break;
				default:
					break;
				}
			}
			for(i = 0; i < DISP_OVER_CURRENT; i++) {
				if (dispOverCurrent[i].check < a) {
					dispOverCurrent[i].sec++;
				}
			}
		}

		switch(dispMode){
		case DISP_NOW_DIGEST:
			dispUpdateNowDigestScreen();
			break;
		case DISP_NOW_DETAIL:
			dispUpdateNowDetailScreen();
			break;
		case DISP_TODAY_GRAPH:
			dispUpdateTodayGraphScreen2();
			break;
		case DISP_MONTH_GRAPH:
			dispUpdateMonthGraphScreen2();
			break;
		case DISP_YEAR_GRAPH:
			dispUpdateYearGraphScreen2();
			break;
		default:
			break;
		}
	}
	lcd_segmentHandler(&dispRootScreen.root, &base);

	return;
}

